@@ -1,6 +1,20 @@
 #ifndef __ACCESS_H
 #define __ACCESS_H
 
+/** Access information for a host-pattern.
+ * This structure holds access information for a given host-pattern.
+ * It's organized into a linked list of access rules.
+ */
+struct access {
+  char host[BUFFER_LEN];	/**< The host pattern */
+  char comment[BUFFER_LEN];	/**< A comment about the rule */
+  dbref who;			/**< Who created this rule if sitelock used */
+  int can;			/**< Bitflags of what the host can do */
+  int cant;			/**< Bitflags of what the host can't do */
+  struct access *next;		/**< Pointer to next rule in the list */
+};
+
+
 /* These flags are can/can't - a site may or may not be allowed to do them */
 #define ACS_CONNECT     0x1	/* Connect to non-guests */
 #define ACS_CREATE      0x2	/* Create new players */
@@ -33,6 +47,10 @@
 int read_access_file(void);
 void write_access_file(void);
 int site_can_access(const char *hname, int flag, dbref who);
+struct access *site_check_access(const char *hname, dbref who, int *rulenum);
+int format_access(struct access *ap, int rulenum,
+		  dbref who
+		  __attribute__ ((__unused__)), char *buff, char **bp);
 int add_access_sitelock(dbref player, const char *host, dbref who, int can,
 			int cant);
 int remove_access_sitelock(const char *pattern);
