@@ -6,7 +6,7 @@ typedef unsigned char switch_mask[NUM_BYTES];
 #define SW_SET(m,n)     (m[(n) >> 3] |= (1 << ((n) & 0x7)))
 #define SW_CLR(m,n)     (m[(n) >> 3] &= ~(1 << ((n) & 0x7)))
 #define SW_ISSET(m,n)   (m[(n) >> 3] & (1 << ((n) & 0x7)))
-#define SW_ZERO(m)      bzero(m, NUM_BYTES)
+#define SW_ZERO(m)      memset(m, 0, NUM_BYTES)
 
 /* These are type restrictors */
 #define CMD_T_ROOM      0x80000000
@@ -39,7 +39,11 @@ typedef unsigned char switch_mask[NUM_BYTES];
  */
 #define CMD_T_INTERNAL  0x00040000
 
-/* Split arguments at = */
+/* Logging for commands */
+#define CMD_T_LOGNAME   0x00020000
+#define CMD_T_LOGARGS   0x00010000
+
+/* Split arguments at =, but don't abort if there's no = */
 #define CMD_T_EQSPLIT    0x0001
 
 /* Split into argv[] at ,s */
@@ -58,7 +62,8 @@ typedef unsigned char switch_mask[NUM_BYTES];
 #define CMD_T_RS_SPACE   CMD_T_ARG_SPACE<<4
 #define CMD_T_RS_NOPARSE CMD_T_NOPARSE<<4
 
-/* COMMAND prototype 
+/** COMMAND prototype.
+ * \verbatim
    Passed arguments:
    executor : Object issuing command.
    sw : switch_mask, check with the SW_ macros.
@@ -75,6 +80,7 @@ typedef unsigned char switch_mask[NUM_BYTES];
    Special case:
    If the NOEVAL switch is given, AND EQSPLIT is defined, the right-side will not be parsed.
    If NOEVAL is givean the EQSPLIT isn't defined, the left-side won't be parsed.
+ * \endverbatim
  */
 
 #define COMMAND(command_name) \
@@ -94,65 +100,89 @@ void command_name(COMMAND_INFO *cmd __attribute__ ((__unused__)), \
                   char *arg_right __attribute__ ((__unused__)), \
                   char *args_right[MAX_ARG] __attribute__ ((__unused__)))
 
+/** Common command prototype macro */
 #define COMMAND_PROTO(command_name) \
 void command_name (COMMAND_INFO *cmd, dbref player, dbref cause, switch_mask sw,char *raw, char *switches, char *args_raw, \
                   char *arg_left, char *args_left[MAX_ARG], \
                   char *arg_right, char *args_right[MAX_ARG])
 
-/* For things that DON'T get auto-aliased by mkalias, like 'i' for 'inv' and 'l' for 'look' */
-
-typedef struct command_alias COMALIAS;
-struct command_alias {
-  const char *name;
-  const char *alias;
-};
-
 typedef struct command_info COMMAND_INFO;
 typedef void (*command_func) (COMMAND_INFO *, dbref, dbref, switch_mask, char *,
 			      char *, char *, char *, char *[MAX_ARG], char *,
 			      char *[MAX_ARG]);
 
+/** A hook specification.
+ */
 struct hook_data {
-  dbref obj;
-  char *attrname;
+  dbref obj;		/**< Object where the hook attribute is stored. */
+  char *attrname;	/**< Attribute name of the hook attribute */
 };
 
+/** A command.
+ * This structure represents a command in the table of available commands.
+ */
 struct command_info {
-  const char *name;
-  command_func func;
-  unsigned int type;
-  unsigned int flags, toggles, powers;
-  switch_mask sw;
+  const char *name;	/**< Canonical name of the command */
+  const char *restrict_message;	/**< Message sent when command is restricted */
+  command_func func;	/**< Function to call when command is run */
+  unsigned int type;	/**< Types of objects that can use the command */
+  object_flag_type flagmask;	/**< Flags to which the command is restricted */
+  object_flag_type powers;	/**< Powers to which the command is restricted */
+  switch_mask sw;	/**< Bitflags of switches this command can take */
+  /** Hooks on this command.
+   */
   struct {
-    struct hook_data before;
-    struct hook_data after;
+    struct hook_data before;	/**< Hook to evaluate before command */
+    struct hook_data after;	/**< Hook to evaluate after command */
+    struct hook_data ignore;	/**< Hook to evaluate to decide if we should ignore hardcoded command */
+    struct hook_data override;	/**< Hook to override command with $command */
   } hooks;
 };
 
 typedef struct command_list COMLIST;
+/** A command list entry.
+ * This structure stores the static array of commands that are 
+ * initially loaded into the command table. Commands can also be
+ * added dynamically, outside of this array.
+ */
 struct command_list {
-  const char *name;
-  const char *switches;
-  command_func func;
-  unsigned int type;
-  unsigned int flags, toggles, powers;
+  const char *name;	/**< Command name */
+  const char *switches;	/**< Space-separated list of switch names */
+  command_func func;	/**< Function to call when command is run */
+  unsigned int type;	/**< Types of objects that can use the command */
+  const char *flagstr;	/**< Space-separated list of flags that can use */
+  const char *powers;	/**< Powers to which the command is restricted */
 };
 
 typedef struct switch_value SWITCH_VALUE;
+/** The value associated with a switch.
+ * Command switches are given integral values at compile time when
+ * the switchinc.c and switches.h files are rebuilt. This structure
+ * associates switch names with switch numbers
+ */
 struct switch_value {
-  const char *name;
-  int value;
+  const char *name;	/**< Name of the switch */
+  int value;		/**< Number of the switch */
 };
 
 typedef struct com_sort_struc COMSORTSTRUC;
+
+/** Sorted linked list of commands.
+ * This structure is used to build a sorted linked list of pointers
+ * to command data.
+ */
 struct com_sort_struc {
-  struct com_sort_struc *next;
-  COMMAND_INFO *cmd;
+  struct com_sort_struc *next;	/**< Pointer to next in list */
+  COMMAND_INFO *cmd;		/**< Command data */
 };
 
+/** Permissions for commands.
+ * This structure is used to associate names for command permissions
+ * (e.g. "player") with the appropriate bitmask
+ */
 struct command_perms_t {
-  const char *name;
-  unsigned int type;
+  const char *name;	/**< Permission name */
+  unsigned int type;	/**< Bitmask for this permission */
 };
 
 #define SWITCH_NONE 0
@@ -162,14 +192,15 @@ extern switch_mask *switchmask(const char *switches);
 extern COMMAND_INFO *command_find(const char *name);
 extern COMMAND_INFO *command_find_exact(const char *name);
 extern COMMAND_INFO *command_add
-  (const char *name, int type, int flags, int toggles, int powers,
-   switch_mask *sw, command_func func);
+  (const char *name, int type, const char *flagstr, const char *powers,
+   const char *switchstr, command_func func);
 extern COMMAND_INFO *make_command
-  (const char *name, int type, int flags, int toggles, int powers,
-   switch_mask *sw, command_func func);
-extern COMMAND_INFO *command_modify
-  (const char *name, int type, int flags, int toggles, int powers,
-   switch_mask *sw, command_func func);
+  (const char *name, int type, object_flag_type flagmask,
+   object_flag_type powers, switch_mask *sw, command_func func);
+extern COMMAND_INFO *command_modify(const char *name, int type,
+				    object_flag_type flagmask,
+				    object_flag_type powers, switch_mask *sw,
+				    command_func func);
 extern void reserve_alias(const char *a);
 extern int alias_command(const char *command, const char *alias);
 extern void command_init_preconfig(void);
@@ -188,5 +219,8 @@ extern int command_check_byname(dbref player, const char *name);
 extern int restrict_command(const char *name, const char *restriction);
 extern void reserve_aliases(void);
 extern void local_commands(void);
+extern void do_command_add(dbref player, char *name, int flags);
+extern void do_command_delete(dbref player, char *name);
+
 
 #endif				/* __COMMAND_H */
