@@ -2,7 +2,6 @@
 
 #ifndef _EXTMAIL_H
 #define _EXTMAIL_H
-#ifdef USE_MAILER
 /* Some of this isn't implemented yet, but heralds the future! */
 #define M_MSGREAD       0x0001
 #define M_UNREAD        0x0FFE
@@ -19,7 +18,6 @@
 #define M_MSUNREAD      0x2000	/* Mail selectors */
 #define M_REPLY         0x4000
 #define M_FOLDER        0x8000	/* In mail selector, all msgs in cur folder */
-/* 0x4000 - 0x8000 available */
 
 #define MAX_FOLDERS     15
 #define FOLDER_NAME_LEN (BUFFER_LEN / 30)
@@ -41,11 +39,17 @@
 
 typedef unsigned int mail_flag;
 
+/** A mail selection.
+ * This structure maintains information about a selected list of
+ * messages. Messages can be selected in several ways.
+ */
 struct mail_selector {
-  int low, high;
-  mail_flag flags;
-  dbref player;
-  int days, day_comp;
+  int low;		/**< Minimum message number */
+  int high;		/**< Maximum message number */
+  mail_flag flags;	/**< Message flags */
+  dbref player;		/**< Message sender's dbref */
+  int days;		/**< Target message age in days */
+  int day_comp;		/**< Direction of comparison to target age */
 };
 
 typedef int folder_array[MAX_FOLDERS + 1];
@@ -62,6 +66,9 @@ fa[x] = 0
 /* Database ends with ***END OF DUMP*** not *** END OF DUMP *** */
 #define MDBF_NEW_EOD    0x4
 
+/* Database contains sender ctimes */
+#define MDBF_SENDERCTIME        0x8
+
 /* From extmail.c */
 extern struct mail *maildb;
 extern void set_player_folder(dbref player, int fnum);
@@ -73,7 +80,8 @@ extern int load_mail(FILE * fp);
 extern void mail_init(void);
 extern int mdb_top;
 extern void do_mail(dbref player, char *arg1, char *arg2);
-extern void do_mail_stats(dbref player, char *name, int full);
+enum mail_stats_type { MSTATS_COUNT, MSTATS_READ, MSTATS_SIZE };
+extern void do_mail_stats(dbref player, char *name, enum mail_stats_type full);
 extern void do_mail_debug(dbref player, const char *action, const char *victim);
 extern void do_mail_nuke(dbref player);
 extern void do_mail_change_folder(dbref player, char *fld, char *newname);
@@ -81,11 +89,11 @@ extern void do_mail_unfolder(dbref player, char *fld);
 extern void do_mail_list(dbref player, const char *msglist);
 extern void do_mail_read(dbref player, char *msglist);
 extern void do_mail_clear(dbref player, const char *msglist);
-extern void do_mail_unclear(dbref player, char *msglist);
+extern void do_mail_unclear(dbref player, const char *msglist);
 extern void do_mail_purge(dbref player);
 extern void do_mail_file(dbref player, char *msglist, char *folder);
-extern void do_mail_tag(dbref player, char *msglist);
-extern void do_mail_untag(dbref player, char *msglist);
+extern void do_mail_tag(dbref player, const char *msglist);
+extern void do_mail_untag(dbref player, const char *msglist);
 extern void do_mail_fwd(dbref player, char *msglist, char *tolist);
 extern void do_mail_send
   (dbref player, char *tolist, char *message, mail_flag flags,
@@ -96,5 +104,4 @@ extern struct mail *desc_mail(dbref player);
 extern void desc_mail_set(dbref player, struct mail *mp);
 extern void desc_mail_clear(void);
 
-#endif				/* USE_MAILER */
 #endif				/* _EXTMAIL_H */
