@@ -2,6 +2,9 @@
 #define __GETPGSIZ_H
 
 #ifndef HAS_GETPAGESIZE
+#ifdef PAGESIZE_VALUE
+#define getpagesize() PAGESIZE_VALUE
+#else
 #ifndef WIN32
 #ifdef I_SYS_PARAM
 #include <sys/param.h>
@@ -23,11 +26,15 @@
 #ifdef NBPC
 #define getpagesize() NBPC
 #else				/* no NBPC either? Bummer */
+#ifdef PAGESIZE
 #define getpagesize() PAGESIZE
+#else				/* Sigh. Time for a total guess. */
+#define getpagesize() 1024
+#endif				/* no PAGESIZE */
 #endif				/* no NBPC */
 #endif				/* no NBPG */
 #endif				/* no EXEC_PAGESIZE */
-
+#endif				/* no PAGESIZE_VALUE */
 #endif				/* not HAS_GETPAGESIZE */
 
 #endif				/* __GETPGSIZ_H */
