@@ -40,11 +40,14 @@ extern "C" {
 #define IDPORT  113
 #endif
 
+  /** Ident call result.
+   * This structure stores the result of an ident call.
+   */
   typedef struct {
-    char *identifier;		/* Normally user name */
-    char *opsys;		/* OS */
-    char *charset;		/* Charset (what did you expect?) */
-  } IDENT;			/* For higher-level routines */
+    char *identifier;		/**< Normally user name */
+    char *opsys;		/**< Operating system */
+    char *charset;		/**< Character set */
+  } IDENT;
 
 /* High-level calls */
 
