@@ -24,10 +24,12 @@
 #if (MALLOC_PACKAGE == 1)
 #define CSRI
 #elif (MALLOC_PACKAGE == 2)
+#include <stdlib.h>
 #define CSRI
 #define CSRI_TRACE
 #define CSRI_PROFILESIZES
 #define CSRI_DEBUG
+#include "csrimalloc.h"
 #endif
 
 #endif				/* _MYMALLOC_H */
