@@ -10,11 +10,16 @@
 
 
 typedef struct priv_info PRIV;
+/** Privileges.
+ * This structure represents a privilege and its associated data.
+ * Privileges tables are used to provide a unified way to parse
+ * a string of restrictions into a bitmask.
+ */
 struct priv_info {
-  const char *name;
-  char letter;
-  long int bits_to_set;
-  long int bits_to_show;
+  const char *name;	/**< Name of the privilege */
+  char letter;		/**< One-letter abbreviation */
+  long int bits_to_set;	/**< Bitflags required to set this privilege */
+  long int bits_to_show;	/**< Bitflags required to see this privilege */
 };
 
 #define PrivName(x)     ((x)->name)
