@@ -25,7 +25,7 @@
 #define notify_noenter_by(t,a,b) notify_anything(t, na_one, &(a), NULL, NA_NOENTER, b)
 #define notify_noenter(a,b) notify_noenter_by(GOD, a, b)
 
-#define tag_wrap(a,b,c) safe_tag_wrap(a,b,c,pbuff,&pp)
+#define tag_wrap(a,b,c) safe_tag_wrap(a,b,c,pbuff,&pp,NOTHING)
 #define tag(a) safe_tag(a,pbuff,&pp)
 #define tag_cancel(a) safe_tag_cancel(a,pbuff,&pp)
 
@@ -34,7 +34,7 @@
 int safe_tag(char const *a_tag, char *buf, char **bp);
 int safe_tag_cancel(char const *a_tag, char *buf, char **bp);
 int safe_tag_wrap(char const *a_tag, char const *params,
-		  char const *data, char *buf, char **bp);
+		  char const *data, char *buf, char **bp, dbref player);
 
 /* Please STAY SANE when modifying. 
  * Making this something like 'x' and 'y' is a BAD IDEA 
