@@ -9,18 +9,25 @@
  * terminated string.
  */
 typedef struct strnode StrNode;
+
+/** A strtree node.
+ * This is a node in a red/black binary strtree.
+ */
 struct strnode {
-  StrNode *left;
-  StrNode *right;
-  unsigned char info;
-  char string[BUFFER_LEN];
+  StrNode *left;		/**< Pointer to left child */
+  StrNode *right;		/**< Pointer to right child */
+  unsigned char info;		/**< Red/black and other internal state */
+  char string[BUFFER_LEN];	/**< Node label (value) */
 };
 
 typedef struct strtree StrTree;
+/** A strtree.
+ * A red/black binary tree of strings.
+ */
 struct strtree {
-  StrNode *root;
-  Size_t count;
-  Size_t mem;
+  StrNode *root;	/**< Pointer to root node */
+  size_t count;		/**< Number of nodes in the tree */
+  size_t mem;		/**< Memory used by the tree */
 };
 
 void st_init(StrTree *root);
