@@ -1,57 +1,63 @@
-
-/*
-   * The file access.cnf in the game directory will control all 
-   access-related directives, replacing lockout.cnf and sites.cnf
-
-   * The format of entries in the file will be:
-
-   wild-host-name    [!]option [!]option [!]option ... # comment
-
-   A wild-host-name is a wildcard pattern to match hostnames with. 
-   The wildcard "*" will work like UNIX filename globbing, so
-   *.edu will match all sites with names ending in .edu, and
-   *.*.*.*.* will match all sites with 4 periods in their name.
-   128.32.*.* will match all sites starting with 128.32 (UC Berkeley).
-   You can also use user@host to match specific users if you know that
-   the host is running ident and you trust its responses (nontrivial).
-
-   The options that can be specified are:
-   *CONNECT              Allow connections to non-guest players
-   *GUEST                Allow connection to guests
-   *CREATE               Allow player creation at login screen
-   DEFAULT               All of the above
-   NONE                 None of the above
-   SUSPECT              Set all players connecting from the site suspect
-   REGISTER             Allow players to use the "register" connect command
-   DENY_SILENT          Don't log when someone's denied access from here
-   REGEXP               Treat the hostname pattern as a regular expression
-   *GOD                  God can connect from this pattern.
-   *WIZARD               Wizards can connect from this pattern.
-   *ADMIN                Admins can connect from this pattern.
-
-   Options that are *'d can be prefaced by a !, meaning "Don't allow".
-
-   * The file is parsed line-by-line in order. This makes it possible
-   to explicitly allow only certain sites to connect and deny all others,
-   or vice versa. Sites can only do the options that are specified
-   in the first line they match.
-
-   * If a site is listed in the file with no options at all, it is
-   disallowed from any access (treated as !CONNECT, basically)
-
-   * If a site doesn't match any line in the file, it is allowed any
-   toggleable access (treated as DEFAULT) but isn't SUSPECT or REGISTER.
-
-   * "make access" produces access.cnf from lockout.cnf/sites.cnf
-
-   * @sitelock'd sites appear after the line "@sitelock" in the file
-   Using @sitelock writes out the file.
-
+/**
+ * \file
+ *
+ * \brief Access control lists for PennMUSH.
+ * \verbatim
+ *
+ * The file access.cnf in the game directory will control all 
+ * access-related directives, replacing lockout.cnf and sites.cnf
+ *
+ * The format of entries in the file will be:
+ *
+ * wild-host-name    [!]option [!]option [!]option ... # comment
+ *
+ * A wild-host-name is a wildcard pattern to match hostnames with. 
+ * The wildcard "*" will work like UNIX filename globbing, so
+ * *.edu will match all sites with names ending in .edu, and
+ * *.*.*.*.* will match all sites with 4 periods in their name.
+ * 128.32.*.* will match all sites starting with 128.32 (UC Berkeley).
+ * You can also use user@host to match specific users if you know that
+ * the host is running ident and you trust its responses (nontrivial).
+ *
+ * The options that can be specified are:
+ * *CONNECT              Allow connections to non-guest players
+ * *GUEST                Allow connection to guests
+ * *CREATE               Allow player creation at login screen
+ * DEFAULT               All of the above
+ * NONE                 None of the above
+ * SUSPECT              Set all players connecting from the site suspect
+ * REGISTER             Allow players to use the "register" connect command
+ * DENY_SILENT          Don't log when someone's denied access from here
+ * REGEXP               Treat the hostname pattern as a regular expression
+ * *GOD                  God can connect from this pattern.
+ * *WIZARD               Wizards can connect from this pattern.
+ * *ADMIN                Admins can connect from this pattern.
+ *
+ * Options that are *'d can be prefaced by a !, meaning "Don't allow".
+ *
+ * The file is parsed line-by-line in order. This makes it possible
+ * to explicitly allow only certain sites to connect and deny all others,
+ * or vice versa. Sites can only do the options that are specified
+ * in the first line they match.
+ *
+ * If a site is listed in the file with no options at all, it is
+ * disallowed from any access (treated as !CONNECT, basically)
+ *
+ * If a site doesn't match any line in the file, it is allowed any
+ * toggleable access (treated as DEFAULT) but isn't SUSPECT or REGISTER.
+ *
+ * "make access" produces access.cnf from lockout.cnf/sites.cnf
+ *
+ * @sitelock'd sites appear after the line "@sitelock" in the file
+ * Using @sitelock writes out the file.
+ * 
+ * \endverbatim
  */
 
 #include "config.h"
 #include "copyrite.h"
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #ifdef I_SYS_TYPES
@@ -68,9 +74,6 @@
 #endif
 #include "conf.h"
 #include "externs.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
 #include "access.h"
 #include "mymalloc.h"
 #include "match.h"
@@ -81,22 +84,16 @@
 #include "flags.h"
 #include "confmagic.h"
 
-/* A linked list data structure to hold the access info */
-struct access {
-  char host[BUFFER_LEN];
-  char comment[BUFFER_LEN];
-  dbref who;
-  int can;
-  int cant;
-  struct access *next;
-};
-
 
+/** An access flag. */
 typedef struct a_acsflag acsflag;
+/** An access flag.
+ * This structure is used to build a table of access control flags.
+ */
 struct a_acsflag {
-  const char *name;
-  int toggle;			/* Is this a negatable flag? */
-  int flag;
+  const char *name;		/**< Name of the access flag */
+  int toggle;			/**< Is this a negatable flag? */
+  int flag;			/**< Bitmask of the flag */
 };
 static acsflag acslist[] = {
   {"connect", 1, ACS_CONNECT},
@@ -114,13 +111,11 @@ static acsflag acslist[] = {
 };
 
 static struct access *access_top;
-extern int reserved;		/* reserved file descriptor */
 static int add_access_node
   (const char *host, const dbref who, const int can, const int cant,
    const char *comment);
 static void free_access_list(void);
 
-/* Allocate a new node and add to the end of the list. */
 static int
 add_access_node(const char *host, const dbref who, const int can,
 		const int cant, const char *comment)
@@ -155,11 +150,12 @@ add_access_node(const char *host, const dbref who, const int can,
 }
 
 
-/* Initialize the linked list and read in the access file.
+/** Read the access.cnf file.
+ * Initialize the access rules linked list and read in the access.cnf file.
  * Return 1 if successful, 0 if not
  */
 int
-read_access_file()
+read_access_file(void)
 {
   FILE *fp;
   char buf[BUFFER_LEN];
@@ -177,22 +173,18 @@ read_access_file()
   }
   access_top = NULL;
   /* Be sure we have a file descriptor */
-#ifndef WIN32
-  close(reserved);
-#endif
-#ifdef macintosh
-  fp = fopen(ACCESS_FILE, "rb");
-#else
-  fp = fopen(ACCESS_FILE, "r");
-#endif
+  release_fd();
+  fp = fopen(ACCESS_FILE, FOPEN_READ);
   if (!fp) {
     do_log(LT_ERR, GOD, GOD, T("No %s file found."), ACCESS_FILE);
     retval = 0;
   } else {
     do_rawlog(LT_ERR, "Reading %s", ACCESS_FILE);
     while (fgets(buf, BUFFER_LEN, fp)) {
-      /* Strip newline */
-      if ((p = strchr(buf, '\n')))
+      /* Strip end of line if it's \r\n or \n */
+      if ((p = strchr(buf, '\r')))
+	*p = '\0';
+      else if ((p = strchr(buf, '\n')))
 	*p = '\0';
       /* Find beginning of line; ignore blank lines */
       p = buf;
@@ -227,15 +219,15 @@ read_access_file()
     retval = 1;
     fclose(fp);
   }
-#ifndef WIN32
-  reserved = open("/dev/null", O_RDWR);
-#endif
+  reserve_fd();
   return retval;
 }
 
-/* Write out the access file from the linked list */
+/** Write the access.cnf file.
+ * Writes out the access.cnf file from the linked list
+ */
 void
-write_access_file()
+write_access_file(void)
 {
   FILE *fp;
   char tmpf[BUFFER_LEN];
@@ -244,14 +236,8 @@ write_access_file()
 
   sprintf(tmpf, "%s.tmp", ACCESS_FILE);
   /* Be sure we have a file descriptor */
-#ifndef WIN32
-  close(reserved);
-#endif
-#ifdef macintosh
-  fp = fopen(tmpf, "wb");
-#else
-  fp = fopen(tmpf, "w");
-#endif
+  release_fd();
+  fp = fopen(tmpf, FOPEN_WRITE);
   if (!fp) {
     do_log(LT_ERR, GOD, GOD, T("Unable to open %s."), tmpf);
   } else {
@@ -289,18 +275,20 @@ write_access_file()
 	fprintf(fp, "\n");
     }
     fclose(fp);
-#ifdef WIN32
-    unlink(ACCESS_FILE);
-#endif
-    rename(tmpf, ACCESS_FILE);
+    rename_file(tmpf, ACCESS_FILE);
   }
-#ifndef WIN32
-  reserved = open("/dev/null", O_RDWR);
-#endif
+  reserve_fd();
   return;
 }
 
-/* Given a hostname and a flag decide if the host can do it.
+/** Decide if a host can access someway.
+ * \param hname a host or user+host pattern.
+ * \param flag the access type we're testing.
+ * \param who the player attempting access.
+ * \retval 1 access permitted.
+ * \retval 0 access denied.
+ * \verbatim
+ * Given a hostname and a flag decide if the host can do it.
  * Here's how it works:
  * We run the linked list and take the first match.
  *  (If the hostname is user@host, we try to match both user@host
@@ -310,6 +298,7 @@ write_access_file()
  * Otherwise, we assume that the host can do any toggleable option
  *   (can create, connect, guest), and don't have any special
  *   flags (can't register, isn't suspect)
+ * \endverbatim
  */
 int
 site_can_access(const char *hname, int flag, dbref who)
@@ -363,10 +352,110 @@ site_can_access(const char *hname, int flag, dbref who)
   return 1;
 }
 
-/* Add an entry to the linked list after the @sitelock entry.
+
+/** Return the first access rule that matches a host.
+ * \param hname a host or user+host pattern.
+ * \param who the player attempting access.
+ * \param rulenum pointer to rule position.
+ * \return pointer to first matching access rule or NULL.
+ */
+struct access *
+site_check_access(const char *hname, dbref who, int *rulenum)
+{
+  struct access *ap;
+  char *p;
+
+  *rulenum = 0;
+  if (!hname || !*hname)
+    return 0;
+
+  if ((p = strchr(hname, '@')))
+    p++;
+
+  for (ap = access_top; ap; ap = ap->next) {
+    (*rulenum)++;
+    if (!(ap->can & ACS_SITELOCK)
+	&& ((ap->can & ACS_REGEXP)
+	    ? (regexp_match_case(ap->host, hname, 0)
+	       || (p && regexp_match_case(ap->host, p, 0)))
+	    : (quick_wild(ap->host, hname)
+	       || (p && quick_wild(ap->host, p))))
+	&& (ap->who == AMBIGUOUS || ap->who == who)) {
+      /* Got one */
+      return ap;
+    }
+  }
+  return NULL;
+}
+
+/** Display an access rule.
+ * \param ap pointer to access rule.
+ * \param rulenum access rule's number in the list.
+ * \param who unused.
+ * \param buff buffer to store output.
+ * \param bp pointer into buff.
+ * This function provides an appealing display of an access rule
+ * in the list.
+ */
+int
+format_access(struct access *ap, int rulenum,
+	      dbref who __attribute__ ((__unused__)), char *buff, char **bp)
+{
+  if (ap) {
+    safe_format(buff, bp, T("Matched line %d: %s %s"), rulenum, ap->host,
+		(ap->can & ACS_REGEXP) ? "(regexp)" : "");
+    safe_chr('\n', buff, bp);
+    safe_format(buff, bp, T("Comment: %s"), ap->comment);
+    safe_chr('\n', buff, bp);
+    safe_str(T("Connections allowed by: "), buff, bp);
+    if (ap->cant & ACS_CONNECT)
+      safe_str(T("No one"), buff, bp);
+    else if (ap->cant & ACS_ADMIN)
+      safe_str(T("All but admin"), buff, bp);
+    else if (ap->cant & ACS_WIZARD)
+      safe_str(T("All but wizards"), buff, bp);
+    else if (ap->cant & ACS_GOD)
+      safe_str(T("All but God"), buff, bp);
+    else
+      safe_str(T("All"), buff, bp);
+    safe_chr('\n', buff, bp);
+    if (ap->cant & ACS_GUEST)
+      safe_str(T("Guest connections are NOT allowed"), buff, bp);
+    else
+      safe_str(T("Guest connections are allowed"), buff, bp);
+    safe_chr('\n', buff, bp);
+    if (ap->cant & ACS_CREATE)
+      safe_str(T("Creation is NOT allowed"), buff, bp);
+    else
+      safe_str(T("Creation is allowed"), buff, bp);
+    safe_chr('\n', buff, bp);
+    if (ap->can & ACS_REGISTER)
+      safe_str(T("Email registration is allowed"), buff, bp);
+    if (ap->can & ACS_SUSPECT)
+      safe_str(T("Players connecting are set SUSPECT"), buff, bp);
+    if (ap->can & ACS_DENY_SILENT)
+      safe_str(T("Denied connections are not logged"), buff, bp);
+  } else {
+    safe_str(T("No matching access rule"), buff, bp);
+  }
+  return 0;
+}
+
+
+/** Add an access rule to the linked list.
+ * \param player enactor.
+ * \param host host pattern to add.
+ * \param who player to which rule applies, or AMBIGUOUS.
+ * \param can flags of allowed actions.
+ * \param cant flags of disallowed actions.
+ * \retval 1 success.
+ * \retval 0 failure.
+ * \verbatim
+ * This function adds an access rule after the @sitelock entry.
  * If there is no @sitelock entry, add one to the end of the list
  * and then add the entry.
  * Build an appropriate comment based on the player and date
+ * \endverbatim
  */
 int
 add_access_sitelock(dbref player, const char *host, dbref who, int can,
@@ -374,7 +463,6 @@ add_access_sitelock(dbref player, const char *host, dbref who, int can,
 {
   struct access *end;
   struct access *tmp;
-  char *date;
 
   tmp = (struct access *) mush_malloc(sizeof(struct access), "struct_access");
   if (!tmp)
@@ -384,9 +472,8 @@ add_access_sitelock(dbref player, const char *host, dbref who, int can,
   tmp->can = can;
   tmp->cant = cant;
   strcpy(tmp->host, host);
-  date = (char *) ctime(&mudtime);
-  date[strlen(date) - 1] = '\0';
-  sprintf(tmp->comment, "By %s(#%d) on %s", Name(player), player, date);
+  sprintf(tmp->comment, "By %s(#%d) on %s", Name(player), player,
+	  show_time(mudtime, 0));
   tmp->next = NULL;
 
   if (!access_top) {
@@ -413,7 +500,15 @@ add_access_sitelock(dbref player, const char *host, dbref who, int can,
   return 1;
 }
 
-/* Remove an entry from the access list and dump out to file */
+/** Remove an access rule from the linked list.
+ * \param pattern access rule host pattern to match.
+ * \return number of rule removed.
+ * \verbatim
+ * This function removes an access rule from the list.
+ * Only rules that appear after the "@sitelock" rule can be
+ * removed with this function.
+ * \endverbatim
+ */
 int
 remove_access_sitelock(const char *pattern)
 {
@@ -448,7 +543,7 @@ remove_access_sitelock(const char *pattern)
 
 /* Free the entire access list */
 static void
-free_access_list()
+free_access_list(void)
 {
   struct access *ap, *next;
   ap = access_top;
@@ -461,16 +556,21 @@ free_access_list()
 }
 
 
-/* Dump the access list for the player */
+/** Display the access list.
+ * \param player enactor.
+ * Sends the complete access list to the player.
+ */
 void
 do_list_access(dbref player)
 {
   struct access *ap;
   acsflag *c;
   char flaglist[BUFFER_LEN];
+  int rulenum = 0;
   char *bp;
 
   for (ap = access_top; ap; ap = ap->next) {
+    rulenum++;
     if (ap->can != ACS_SITELOCK) {
       bp = flaglist;
       for (c = acslist; c->name; c++) {
@@ -488,8 +588,8 @@ do_list_access(dbref player)
       }
       *bp = '\0';
       notify_format(player,
-		    "SITE: %-20s  DBREF: %-6s FLAGS:%s", ap->host,
-		    unparse_dbref(ap->who), flaglist);
+		    "%3d SITE: %-20s  DBREF: %-6s FLAGS:%s", rulenum,
+		    ap->host, unparse_dbref(ap->who), flaglist);
       notify_format(player, " COMMENT: %s", ap->comment);
     } else {
       notify(player,
@@ -500,9 +600,16 @@ do_list_access(dbref player)
   }
 }
 
-/* Parse options and return the appropriate can and cant bits.
+/** Parse access options into fields.
+ * \param opts access options to read from.
+ * \param who pointer to player to whom rule applies, or AMBIGUOUS.
+ * \param can pointer to flags of allowed actions.
+ * \param cant pointer to flags of disallowed actions.
+ * \param player enactor.
+ * \return number of options successfully parsed.
+ * Parse options and return the appropriate can and cant bits.
  * Return the number of options successfully parsed.
- * This makes a copy of the options string, so it's not modified
+ * This makes a copy of the options string, so it's not modified.
  */
 int
 parse_access_options(const char *opts, dbref *who, int *can, int *cant,
