@@ -103,6 +103,10 @@
 #include "strtree.h"
 #include "confmagic.h"
 
+#ifdef WIN32
+#pragma warning( disable : 4761)	/* disable warning re conversion */
+#endif
+
 /* #define DEBUG_BYTECODE */
 
 /** Parse tree node types */
