@@ -95,16 +95,28 @@ COMMAND (cmd_boot) {
 COMMAND (cmd_break) {
   if (parse_boolean(arg_left)) {
     global_eval_context.break_called = 1;
-    if (arg_right && *arg_right)
-      strcpy(global_eval_context.break_replace, arg_right);
+    if (arg_right && *arg_right) {
+      char const *sp = arg_right;
+      char *bp = global_eval_context.break_replace;
+      process_expression(global_eval_context.break_replace, &bp, &sp,
+			 player, player, cause,
+			 PE_COMMAND_BRACES, PT_DEFAULT, NULL);
+      *bp++ = '\0';
+    }
   }
 }
 
 COMMAND (cmd_assert) {
   if (!parse_boolean(arg_left)) {
     global_eval_context.break_called = 1;
-    if (arg_right && *arg_right)
-      strcpy(global_eval_context.break_replace, arg_right);
+    if (arg_right && *arg_right) {
+      char const *sp = arg_right;
+      char *bp = global_eval_context.break_replace;
+      process_expression(global_eval_context.break_replace, &bp, &sp,
+			 player, player, cause,
+			 PE_COMMAND_BRACES, PT_DEFAULT, NULL);
+      *bp++ = '\0';
+    }
   }
 }
 
@@ -613,7 +625,7 @@ COMMAND (cmd_name) {
 }
 
 COMMAND (cmd_newpassword) {
-  do_newpassword(player, arg_left, arg_right);
+  do_newpassword(player, cause, arg_left, arg_right);
 }
 
 COMMAND (cmd_nuke) {
@@ -635,7 +647,7 @@ COMMAND (cmd_parent) {
 }
 
 COMMAND (cmd_password) {
-  do_password(player, arg_left, arg_right);
+  do_password(player, cause, arg_left, arg_right);
 }
 
 COMMAND (cmd_pcreate) {
