@@ -64,14 +64,16 @@ COMLIST commands[] = {
   {"@@", NULL, cmd_null, CMD_T_ANY | CMD_T_NOPARSE, 0, 0},
   {"@ALLHALT", NULL, cmd_allhalt, CMD_T_ANY, "WIZARD", "HALT"},
   {"@ALLQUOTA", "QUIET", cmd_allquota, CMD_T_ANY, "WIZARD", "QUOTA"},
-  {"@ASSERT", NULL, cmd_assert, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
+  {"@ASSERT", NULL, cmd_assert, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE, 0,
+   0},
   {"@ATRLOCK", NULL, cmd_atrlock, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@ATRCHOWN", NULL, cmd_atrchown, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
 
   {"@ATTRIBUTE", "ACCESS DELETE RENAME RETROACTIVE", cmd_attribute,
    CMD_T_ANY | CMD_T_EQSPLIT, "WIZARD", 0},
   {"@BOOT", "PORT ME SILENT", cmd_boot, CMD_T_ANY, 0, 0},
-  {"@BREAK", NULL, cmd_break, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
+  {"@BREAK", NULL, cmd_break, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_NOPARSE, 0,
+   0},
   {"@CEMIT", "NOEVAL NOISY SPOOF", cmd_cemit,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, 0, 0},
   {"@CHANNEL",
@@ -91,7 +93,8 @@ COMLIST commands[] = {
   {"@CONFIG",
    "SET LOWERCASE LIST GLOBALS DEFAULTS COSTS FLAGS FUNCTIONS COMMANDS ATTRIBS",
    cmd_config, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
-  {"@CPATTR", "CONVERT", cmd_cpattr, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS,
+  {"@CPATTR", "CONVERT NOFLAGCOPY", cmd_cpattr,
+   CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS,
    0, 0},
   {"@CREATE", NULL, cmd_create, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED,
    0, 0},
@@ -178,12 +181,13 @@ COMLIST commands[] = {
    0, 0},
   {"@MOTD", "CONNECT LIST WIZARD DOWN FULL", cmd_motd,
    CMD_T_ANY | CMD_T_NOGAGGED, 0, 0},
-  {"@MVATTR", "CONVERT", cmd_mvattr, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS,
+  {"@MVATTR", "CONVERT NOFLAGCOPY", cmd_mvattr,
+   CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS,
    0, 0},
   {"@NAME", NULL, cmd_name, CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED
    | CMD_T_NOGUEST, 0, 0},
-  {"@NEWPASSWORD", NULL, cmd_newpassword, CMD_T_ANY | CMD_T_EQSPLIT, "WIZARD",
-   0},
+  {"@NEWPASSWORD", NULL, cmd_newpassword, CMD_T_ANY | CMD_T_EQSPLIT
+   | CMD_T_RS_NOPARSE, "WIZARD", 0},
   {"@NOTIFY", "ALL ANY", cmd_notify_drain, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@NSCEMIT", "NOEVAL NOISY SPOOF", cmd_cemit,
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_NOGAGGED, "WIZARD", "CAN_NSPEMIT"},
@@ -207,7 +211,7 @@ COMLIST commands[] = {
    CMD_T_ANY | CMD_T_EQSPLIT | CMD_T_RS_ARGS | CMD_T_NOGAGGED, 0, 0},
   {"@PARENT", NULL, cmd_parent, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
   {"@PASSWORD", NULL, cmd_password, CMD_T_PLAYER | CMD_T_EQSPLIT
-   | CMD_T_NOGUEST, 0, 0},
+   | CMD_T_NOPARSE | CMD_T_RS_NOPARSE | CMD_T_NOGUEST, 0, 0},
   {"@PCREATE", NULL, cmd_pcreate, CMD_T_ANY | CMD_T_EQSPLIT, 0, 0},
 
   {"@PEMIT", "LIST CONTENTS SILENT NOISY NOEVAL SPOOF", cmd_pemit,
@@ -896,6 +900,8 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
     return NULL;
   case SAY_TOKEN:
     replacer = "SAY";
+    if (CHAT_STRIP_QUOTE)
+      p--;			/* Since 'say' strips out the '"' */
     break;
   case POSE_TOKEN:
     replacer = "POSE";
@@ -923,27 +929,27 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
       replacer = "@CHAT";
       noevtoken = 1;
     }
+  case NUMBER_TOKEN:
+    /* parse_force() destructively modifies the command to replace
+     * the first space with a '=' if the command is an actual
+     * chat command */
+    if (Mobile(player) && parse_force(p)) {
+      /* This is a "#obj foo" force style
+       * We set noevtoken to keep its noeval way, and
+       * set the cmd to allow @hook. */
+      replacer = "@FORCE";
+      noevtoken = 1;
+    }
   }
 
   if (replacer) {
     cmd = command_find(replacer);
-    p++;
+    if (*p != NUMBER_TOKEN)
+      p++;
   } else {
-    /* A few other options, where we might want to fall through
-     * to the default parsing if the syntax we're given doesn't
-     * quite match.
-     */
-    if (*p == '#') {
-      if (!Gagged(player) && Mobile(player)
-	  && force_by_number(player, string)) {
-	command_parse_free_args;
-	return NULL;
-      }
-    }
-    /* At this point, we have not done a replacer, and neither chat nor
-     * force by number applies, so we continue with the usual
-     * processing. Exits have next priority.  We still pass them through
-     * the parser so @hook on GOTO can work on them.
+    /* At this point, we have not done a replacer, so we continue with the 
+     * usual processing. Exits have next priority.  We still pass them 
+     * through the parser so @hook on GOTO can work on them.
      */
     if (can_move(player, p)) {
       ec = exit_command;
@@ -1166,7 +1172,8 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
     do_rawlog(LT_ERR, T("No command vector on command %s."), cmd->name);
     return NULL;
   } else {
-    char *saveregs[NUMQ] = { NULL };
+    char *saveregs[NUMQ];
+    init_global_regs(saveregs);
     /* If we have a hook/ignore that returns false, we don't do the command */
     if (run_hook(player, cause, &cmd->hooks.ignore, saveregs, 1)) {
       /* If we have a hook/override, we use that instead */
@@ -1186,6 +1193,7 @@ command_parse(dbref player, dbref cause, char *string, int fromport)
     } else {
       retval = commandraw;
     }
+    free_global_regs("hook.regs", saveregs);
   }
 
   command_parse_free_args;
@@ -1203,10 +1211,11 @@ void
 generic_command_failure(dbref player, dbref cause, char *string)
 {
   COMMAND_INFO *cmd;
-  char *saveregs[NUMQ] = { NULL };
+  char *saveregs[NUMQ];
 
   if ((cmd = command_find("HUH_COMMAND"))) {
     if (!(cmd->type & CMD_T_DISABLED)) {
+      init_global_regs(saveregs);
       if (run_hook(player, cause, &cmd->hooks.ignore, saveregs, 1)) {
 	/* If we have a hook/override, we use that instead */
 	if (!has_hook(&cmd->hooks.override) ||
@@ -1222,6 +1231,7 @@ generic_command_failure(dbref player, dbref cause, char *string)
 	if (cmd->type & CMD_T_LOGARGS)
 	  do_log(LT_HUH, player, cause, "%s", string);
       }
+      free_global_regs("hook.regs", saveregs);
     }
   }
 }
