@@ -1,3 +1,13 @@
+/**
+ * \file compress.c
+ *
+ * \brief Compression routine wrapper file for PennMUSH.
+ *
+ * This file does nothing but conditionally include the appropriate
+ * attribute compression source code.
+ *
+ */
+
 #include "config.h"
 #include "options.h"
 
@@ -7,7 +17,7 @@
 
 #if defined(COMPRESSION_TYPE) && (COMPRESSION_TYPE == 0)
 /* No compression */
-char ucbuff[BUFFER_LEN];
+char ucbuff[BUFFER_LEN];	/**< Dummy buffer for no compression */
 
 #elif (COMPRESSION_TYPE == 1) || (COMPRESSION_TYPE == 2)
 /* Huffman compression */
