@@ -1,4 +1,10 @@
-/* cque.c */
+/**
+ * \file cque.c
+ *
+ * \brief Queue for PennMUSH.
+ *
+ *
+ */
 
 #include "copyrite.h"
 #include "config.h"
@@ -19,9 +25,6 @@
 #include "mushdb.h"
 #include "match.h"
 #include "externs.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
 #include "parse.h"
 #include "strtree.h"
 #include "mymalloc.h"
@@ -33,27 +36,24 @@
 #include "confmagic.h"
 
 
-extern char ccom[];
-extern dbref cplr;
+EVAL_CONTEXT global_eval_context;
 
-char *wenv[10];			/* working environment (wptr/rptr equiv) */
-char renv[NUMQ][BUFFER_LEN];
-char *wnxt[10], *rnxt[NUMQ];	/* stuff to be shoved into the queue */
-int break_called = 0;
-typedef struct bque BQUE;
-
-struct bque {
-  BQUE *next;
-  dbref player;			/* player who will do command */
-  dbref queued;			/* object who's QUEUE gets incremented for this command */
-  dbref cause;			/* player causing command (for %N) */
-  dbref sem;			/* semaphore object to block on */
-  char *semattr;		/* semaphore attribute to block on */
-  int left;			/* seconds left until execution */
-  char *env[10];		/* environment, from wild match */
-  char *rval[NUMQ];		/* environment, from setq() */
-  char *comm;			/* command to be executed */
-};
+/** A queue entry.
+ * This structure reprsents a queue entry on a linked list of queue
+ * entries (a queue). It is used for all of the queues.
+ */
+typedef struct bque {
+  struct bque *next;			/**< pointer to next entry on queue */
+  dbref player;			/**< player who will do command */
+  dbref queued;			/**< object whose QUEUE gets incremented for this command */
+  dbref cause;			/**< player causing command (for %N) */
+  dbref sem;			/**< semaphore object to block on */
+  char *semattr;		/**< semaphore attribute to block on */
+  int left;			/**< seconds left until execution */
+  char *env[10];		/**< environment, from wild match */
+  char *rval[NUMQ];		/**< environment, from setq() */
+  char *comm;			/**< command to be executed */
+} BQUE;
 
 static BQUE *qfirst = NULL, *qlast = NULL, *qwait = NULL;
 static BQUE *qlfirst = NULL, *qllast = NULL;
@@ -65,25 +65,30 @@ static int add_to_sem(dbref player, int am, const char *name);
 static int queue_limit(dbref player);
 void free_qentry(BQUE *point);
 static int pay_queue(dbref player, const char *command);
-void wait_que
-  (dbref player, int wait, char *command,
-   dbref cause, dbref sem, const char *semattr, int until);
+void wait_que(dbref player, int wait, char *command,
+	      dbref cause, dbref sem, const char *semattr, int until);
 int que_next(void);
 
-static void show_queue
-  (dbref player, dbref victim, int q_type,
-   int q_quiet, int q_all, BQUE *q_ptr, int *tot, int *self, int *del);
+static void show_queue(dbref player, dbref victim, int q_type,
+		       int q_quiet, int q_all, BQUE *q_ptr, int *tot, int *self,
+		       int *del);
 static void do_raw_restart(dbref victim);
 static int waitable_attr(dbref thing, const char *atr);
+static void shutdown_a_queue(BQUE **head, BQUE **tail);
 
-extern sig_atomic_t cpu_time_limit_hit;
+extern sig_atomic_t cpu_time_limit_hit;	/**< Have we used too much CPU? */
 
-/* Returns true if the attribute on thing can be used as a semaphore */
+/** Attribute flags to be set or checked on attributes to be used
+ * as semaphores.
+ */
 #define SEMAPHORE_FLAGS (AF_LOCKED | AF_PRIVATE | AF_NOCOPY | AF_NODUMP)
+
+
+/* Returns true if the attribute on thing can be used as a semaphore.
+ * atr should be given in UPPERCASE. 
+ */
 static int
-waitable_attr(thing, atr)
-    dbref thing;
-    const char *atr;		/* Should be uppercase */
+waitable_attr(dbref thing, const char *atr)
 {
   ATTR *a;
   if (!atr || !*atr)
@@ -96,7 +101,7 @@ waitable_attr(thing, atr)
     return !strcmp(AL_NAME(a), "SEMAPHORE");	/* Only allow SEMAPHORE for now */
   } else {			/* Attribute is set. Check for proper owner and flags and value */
     if ((AL_CREATOR(a) == GOD) && (AL_FLAGS(a) == SEMAPHORE_FLAGS)) {
-      char *v = uncompress(AL_STR(a));
+      char *v = atr_value(a);
       if (!*v || is_integer(v))
 	return 1;
       else
@@ -109,18 +114,14 @@ waitable_attr(thing, atr)
 }
 
 static int
-add_to_generic(player, am, name, flags)
-    dbref player;
-    int am;
-    const char *name;
-    int flags;
+add_to_generic(dbref player, int am, const char *name, int flags)
 {
   int num = 0;
   ATTR *a;
   char buff[MAX_COMMAND_LEN];
   a = atr_get_noparent(player, name);
   if (a)
-    num = parse_integer(uncompress(AL_STR(a)));
+    num = parse_integer(atr_value(a));
   num += am;
   if (num) {
     sprintf(buff, "%d", num);
@@ -132,26 +133,20 @@ add_to_generic(player, am, name, flags)
 }
 
 static int
-add_to(player, am)
-    dbref player;
-    int am;
+add_to(dbref player, int am)
 {
   return (add_to_generic(player, am, "QUEUE", NOTHING));
 }
 
 static int
-add_to_sem(player, am, name)
-    dbref player;
-    int am;
-    const char *name;
+add_to_sem(dbref player, int am, const char *name)
 {
   return (add_to_generic
 	  (player, am, name ? name : "SEMAPHORE", SEMAPHORE_FLAGS));
 }
 
 static int
-queue_limit(player)
-    dbref player;
+queue_limit(dbref player)
 {
   /* returns 1 if player has exceeded his queue limit, and always
      increments QUEUE by one. */
@@ -164,9 +159,11 @@ queue_limit(player)
     return nlimit > QUEUE_QUOTA;
 }
 
+/** Free a queue entry.
+ * \param point queue entry to free.
+ */
 void
-free_qentry(point)
-    BQUE *point;
+free_qentry(BQUE *point)
 {
   int a;
   for (a = 0; a < 10; a++)
@@ -185,12 +182,12 @@ free_qentry(point)
 }
 
 static int
-pay_queue(player, command)
-    dbref player;
-    const char *command;
+pay_queue(dbref player, const char *command)
 {
   int estcost;
-  estcost = QUEUE_COST + ((get_random_long(0, QUEUE_LOSS - 1) == 0) ? 1 : 0);
+  estcost =
+    QUEUE_COST +
+    (QUEUE_LOSS ? ((get_random_long(0, QUEUE_LOSS - 1) == 0) ? 1 : 0) : 0);
   if (!payfor(player, estcost)) {
     notify(Owner(player), T("Not enough money to queue command."));
     return 0;
@@ -206,17 +203,22 @@ pay_queue(player, command)
     add_to(QUEUE_PER_OWNER ? Owner(player) : player, -1);
     /* wipe out that object's queue and set it HALT */
     do_halt(Owner(player), "", player);
-    Flags(player) |= HALT;
+    set_flag_internal(player, "HALT");
     return 0;
   }
   return 1;
 }
 
+/** Add a new entry onto the player or object command queues.
+ * This function adds a new entry to the back of the player or
+ * object command queues (depending on whether the call was
+ * caused by a player or an object).
+ * \param player the enactor for the queued command.
+ * \param command the command to enqueue.
+ * \param cause the player or object causing the command to be queued.
+ */
 void
-parse_que(player, command, cause)
-    dbref player;
-    const char *command;
-    dbref cause;
+parse_que(dbref player, const char *command, dbref cause)
 {
   int a;
   BQUE *tmp;
@@ -233,16 +235,16 @@ parse_que(player, command, cause)
   tmp->left = 0;
   tmp->cause = cause;
   for (a = 0; a < 10; a++)
-    if (!wnxt[a])
+    if (!global_eval_context.wnxt[a])
       tmp->env[a] = NULL;
     else {
-      tmp->env[a] = mush_strdup(wnxt[a], "bqueue_env");
+      tmp->env[a] = mush_strdup(global_eval_context.wnxt[a], "bqueue_env");
     }
   for (a = 0; a < NUMQ; a++)
-    if (!rnxt[a] || !rnxt[a][0])
+    if (!global_eval_context.rnxt[a] || !global_eval_context.rnxt[a][0])
       tmp->rval[a] = NULL;
     else {
-      tmp->rval[a] = mush_strdup(rnxt[a], "bqueue_rval");
+      tmp->rval[a] = mush_strdup(global_eval_context.rnxt[a], "bqueue_rval");
     }
   if (IsPlayer(cause)) {
     if (qlast) {
@@ -260,8 +262,15 @@ parse_que(player, command, cause)
 }
 
 
-/* Front end to parse_que that takes an attribute, removes
- * ^....: or $....: from its value, and queues what's left.
+/** Enqueue the action part of an attribute.
+ * This function is a front-end to parse_que() that takes an attribute, 
+ * removes ^....: or $....: from its value, and queues what's left.
+ * \param executor object containing the attribute.
+ * \param atrname attribute name.
+ * \param enactor the enactor.
+ * \param noparent if true, parents of executor are not checked for atrname.
+ * \retval 0 failure.
+ * \retval 1 success.
  */
 int
 queue_attribute_base(dbref executor, const char *atrname, dbref enactor,
@@ -274,7 +283,7 @@ queue_attribute_base(dbref executor, const char *atrname, dbref enactor,
        atr_get(executor, strupper(atrname)));
   if (!a)
     return 0;
-  start = safe_uncompress(AL_STR(a));
+  start = safe_atr_value(a);
   command = start;
   /* Trim off $-command or ^-command prefix */
   if (*command == '$' || *command == '^') {
@@ -293,16 +302,22 @@ queue_attribute_base(dbref executor, const char *atrname, dbref enactor,
   return 1;
 }
 
-
+/** Queue an entry on the wait or semaphore queues.
+ * This function creates and adds a queue entry to the wait queue
+ * or the semaphore queue. Wait queue entries are sorted by when
+ * they're due to expire; semaphore queue entries are just added
+ * to the back of the queue.
+ * \param player the enqueuing object.
+ * \param wait time to wait, or 0.
+ * \param command command to enqueue.
+ * \param cause object that caused command to be enqueued.
+ * \param sem object to serve as a semaphore, or NOTHING.
+ * \param semattr attribute to serve as a semaphore, or NULL (to use SEMAPHORE).
+ * \param until 1 if we wait until an absolute time.
+ */
 void
-wait_que(player, wait, command, cause, sem, semattr, until)
-    dbref player;
-    int wait;
-    char *command;
-    dbref cause;
-    dbref sem;
-    const char *semattr;
-    int until;
+wait_que(dbref player, int wait, char *command, dbref cause, dbref sem,
+	 const char *semattr, int until)
 {
   BQUE *tmp;
   int a;
@@ -322,17 +337,17 @@ wait_que(player, wait, command, cause, sem, semattr, until)
   tmp->semattr = NULL;
   tmp->next = NULL;
   for (a = 0; a < 10; a++) {
-    if (!wnxt[a])
+    if (!global_eval_context.wnxt[a])
       tmp->env[a] = NULL;
     else {
-      tmp->env[a] = mush_strdup(wnxt[a], "bqueue_env");
+      tmp->env[a] = mush_strdup(global_eval_context.wnxt[a], "bqueue_env");
     }
   }
   for (a = 0; a < NUMQ; a++) {
-    if (!rnxt[a] || !rnxt[a][0])
+    if (!global_eval_context.rnxt[a] || !global_eval_context.rnxt[a][0])
       tmp->rval[a] = NULL;
     else {
-      tmp->rval[a] = mush_strdup(rnxt[a], "bqueue_rval");
+      tmp->rval[a] = mush_strdup(global_eval_context.rnxt[a], "bqueue_rval");
     }
   }
 
@@ -372,11 +387,15 @@ wait_que(player, wait, command, cause, sem, semattr, until)
   }
 }
 
+/** Once-a-second check for queued commands.
+ * This function is called every second to check for commands
+ * on the wait queue or semaphore queue, and to move a command
+ * off the low priority object queue and onto the normal priority
+ * player queue.
+ */
 void
-do_second()
+do_second(void)
 {
-  /* call every second to check for wait queue commands */
-
   BQUE *trail = NULL, *point, *next;
   /* move contents of low priority queue onto end of normal one 
    * this helps to keep objects from getting out of control since 
@@ -446,15 +465,19 @@ do_second()
   }
 }
 
+/** Execute some commands from the top of the queue.
+ * This function dequeues and executes commands on the normal
+ * priority (player) queue.
+ * \param ncom number of commands to execute.
+ * \return number of commands executed.
+ */
 int
-do_top(ncom)
-    int ncom;
+do_top(int ncom)
 {
-  /* execute ncom commands off the top of the queue */
-
   int a, i;
   BQUE *entry;
   char tbuf[BUFFER_LEN];
+  int break_count;
   char *r;
   char const *s;
 
@@ -468,33 +491,47 @@ do_top(ncom)
     entry = qfirst;
     if (!(qfirst = entry->next))
       qlast = NULL;
-    if (GoodObject(entry->player)
-	&& !IsGarbage(entry->player)) {
-      cplr = entry->player;
-      giveto(cplr, QUEUE_COST);
+    if (GoodObject(entry->player) && !IsGarbage(entry->player)) {
+      global_eval_context.cplr = entry->player;
+      giveto(global_eval_context.cplr, QUEUE_COST);
       add_to(entry->queued, -1);
       entry->player = 0;
-      if (IsPlayer(cplr) || !Halted(cplr)) {
+      if (IsPlayer(global_eval_context.cplr)
+	  || !Halted(global_eval_context.cplr)) {
 	for (a = 0; a < 10; a++)
-	  wenv[a] = entry->env[a];
+	  global_eval_context.wenv[a] = entry->env[a];
 	for (a = 0; a < NUMQ; a++) {
 	  if (entry->rval[a])
-	    strcpy(renv[a], entry->rval[a]);
+	    strcpy(global_eval_context.renv[a], entry->rval[a]);
 	  else
-	    renv[a][0] = '\0';
+	    global_eval_context.renv[a][0] = '\0';
 	}
 	s = entry->comm;
-	break_called = 0;
+	global_eval_context.break_called = 0;
+	break_count = 100;
+	*(global_eval_context.break_replace) = '\0';
 	start_cpu_timer();
-	while (!break_called && !cpu_time_limit_hit && *s) {
-	  r = ccom;
-	  process_expression(ccom, &r, &s, cplr, entry->cause,
+	while (!cpu_time_limit_hit && *s) {
+	  r = global_eval_context.ccom;
+	  process_expression(global_eval_context.ccom, &r, &s,
+			     global_eval_context.cplr, entry->cause,
 			     entry->cause, PE_NOTHING, PT_SEMI, NULL);
 	  *r = '\0';
 	  if (*s == ';')
 	    s++;
-	  strcpy(tbuf, ccom);
-	  process_command(cplr, tbuf, entry->cause, 0);
+	  strcpy(tbuf, global_eval_context.ccom);
+	  process_command(global_eval_context.cplr, tbuf, entry->cause, 0);
+	  if (global_eval_context.break_called) {
+	    global_eval_context.break_called = 0;
+	    s = global_eval_context.break_replace;
+	    if (!*global_eval_context.break_replace)
+	      break;
+	    break_count--;
+	    if (!break_count) {
+	      notify(global_eval_context.cplr, T("@break recursion exceeded."));
+	      break;
+	    }
+	  }
 	}
 	reset_cpu_timer();
       }
@@ -505,8 +542,16 @@ do_top(ncom)
   return i;
 }
 
+/** Determine whether it's time to run a queued command.
+ * This function returns the number of seconds we expect to wait
+ * before it's time to run a queued command.
+ * If there are commands in the player queue, that's 0.
+ * If there are commands in the object queue, that's 1.
+ * Otherwise, we check wait and semaphore queues to see what's next.
+ * \return seconds left before a queue entry will be ready.
+ */
 int
-que_next()
+que_next(void)
 {
   int min, curr;
   BQUE *point;
@@ -552,6 +597,7 @@ que_next()
 
 static int
 drain_helper(dbref player __attribute__ ((__unused__)), dbref thing,
+	     dbref parent __attribute__ ((__unused__)),
 	     char const *pattern __attribute__ ((__unused__)), ATTR *atr,
 	     void *args __attribute__ ((__unused__)))
 {
@@ -560,6 +606,15 @@ drain_helper(dbref player __attribute__ ((__unused__)), dbref thing,
   return 0;
 }
 
+/** Drain or notify a semaphore.
+ * This function dequeues an entry in the semaphore queue and either
+ * discards it (drain) or executes it (notify). Maybe more than one.
+ * \param thing object serving as semaphore.
+ * \param aname attribute serving as semaphore.
+ * \param count number of entries to dequeue.
+ * \param all if 1, dequeue all entries.
+ * \param drain if 1, drain rather than notify the entries.
+ */
 void
 dequeue_semaphores(dbref thing, char const *aname, int count, int all,
 		   int drain)
@@ -621,7 +676,7 @@ dequeue_semaphores(dbref thing, char const *aname, int count, int all,
     if (aname)
       (void) atr_clr(thing, aname, GOD);
     else
-      atr_iter_get(GOD, thing, "*", drain_helper, NULL);
+      atr_iter_get(GOD, thing, "**", 0, drain_helper, NULL);
   }
 
   /* If @notify and count was higher than the number of queue entries,
@@ -712,13 +767,18 @@ COMMAND (cmd_notify_drain) {
   }
 }
 
+/** Softcode interface to add a command to the wait or semaphore queue.
+ * \verbatim
+ * This is the top-level function for @wait.
+ * \endverbatim
+ * \param player the enactor
+ * \param cause the object causing the command to be added.
+ * \param arg1 the wait time, semaphore object/attribute, or both.
+ * \param cmd command to queue.
+ * \param until if 1, wait until an absolute time.
+ */
 void
-do_wait(player, cause, arg1, cmd, until)
-    dbref player;
-    dbref cause;
-    char *arg1;
-    char *cmd;
-    int until;
+do_wait(dbref player, dbref cause, char *arg1, char *cmd, int until)
 {
   dbref thing;
   char *tcount = NULL, *aname = NULL;
@@ -727,9 +787,9 @@ do_wait(player, cause, arg1, cmd, until)
   char *arg2;
   int j;
   for (j = 0; j < 10; j++)
-    wnxt[j] = wenv[j];
+    global_eval_context.wnxt[j] = global_eval_context.wenv[j];
   for (j = 0; j < NUMQ; j++)
-    rnxt[j] = renv[j];
+    global_eval_context.rnxt[j] = global_eval_context.renv[j];
 
   arg2 = strip_braces(cmd);
   if (is_strict_integer(arg1)) {
@@ -786,7 +846,7 @@ do_wait(player, cause, arg1, cmd, until)
   add_to_sem(thing, 1, aname);
   a = atr_get_noparent(thing, aname);
   if (a)
-    num = parse_integer(uncompress(a->value));
+    num = parse_integer(atr_value(a));
   else
     num = 0;
   if (num <= 0) {
@@ -798,16 +858,8 @@ do_wait(player, cause, arg1, cmd, until)
 }
 
 static void
-show_queue(player, victim, q_type, q_quiet, q_all, q_ptr, tot, self, del)
-    dbref player;
-    dbref victim;
-    int q_type;
-    int q_quiet;
-    int q_all;
-    BQUE *q_ptr;
-    int *tot;
-    int *self;
-    int *del;
+show_queue(dbref player, dbref victim, int q_type, int q_quiet, int q_all,
+	   BQUE *q_ptr, int *tot, int *self, int *del)
 {
   BQUE *tmp;
   for (tmp = q_ptr; tmp; tmp = tmp->next) {
@@ -844,23 +896,26 @@ show_queue(player, victim, q_type, q_quiet, q_all, q_ptr, tot, self, del)
   }
 }
 
+/** Display a player's queued commands.
+ * \verbatim
+ * This is the top-level function for @ps.
+ * \endverbatim
+ * \param player the enactor.
+ * \param what name of player whose queue is to be displayed.
+ * \param flag type of display. 0 - normal, 1 - all, 2 - summary, 3 - quick
+ */
 void
-do_queue(player, what, flag)
-    dbref player;
-    const char *what;
-    int flag;			/* 0 - normal, 1 - all, 2 - summary, 3 - quick */
+do_queue(dbref player, const char *what, enum queue_type flag)
 {
-  /* tell player what commands they have pending in the queue (@ps) */
-
   dbref victim = NOTHING;
   int all = 0;
   int quick = 0;
   int dpq = 0, doq = 0, dwq = 0, dsq = 0;
   int pq = 0, oq = 0, wq = 0, sq = 0;
   int tpq = 0, toq = 0, twq = 0, tsq = 0;
-  if (flag == 2 || flag == 3)
+  if (flag == QUEUE_SUMMARY || flag == QUEUE_QUICK)
     quick = 1;
-  if (flag == 1 || flag == 2) {
+  if (flag == QUEUE_ALL || flag == QUEUE_SUMMARY) {
     all = 1;
     victim = player;
   } else if (LookQueue(player)) {
@@ -884,7 +939,7 @@ do_queue(player, what, flag)
   default:
 
     if (!quick) {
-      if (all == 1)
+      if (all)
 	notify(player, T("Queue for : all"));
       else
 	notify_format(player, T("Queue for : %s"), Name(victim));
@@ -910,11 +965,15 @@ do_queue(player, what, flag)
   }
 }
 
+/** Halt an object, internal use.
+ * This function is used to halt objects by other hardcode.
+ * See do_halt1() for the softcode interface.
+ * \param owner the enactor.
+ * \param ncom command to queue after halting.
+ * \param victim object to halt.
+ */
 void
-do_halt(owner, ncom, victim)
-    dbref owner;
-    const char *ncom;
-    dbref victim;
+do_halt(dbref owner, const char *ncom, dbref victim)
 {
   BQUE *tmp, *trail = NULL, *point, *next;
   int num = 0;
@@ -980,25 +1039,31 @@ do_halt(owner, ncom, victim)
   if (ncom && *ncom) {
     int j;
     for (j = 0; j < 10; j++)
-      wnxt[j] = wenv[j];
+      global_eval_context.wnxt[j] = global_eval_context.wenv[j];
     for (j = 0; j < NUMQ; j++)
-      rnxt[j] = renv[j];
+      global_eval_context.rnxt[j] = global_eval_context.renv[j];
     parse_que(player, ncom, player);
   }
 }
 
+/** Halt an object, softcode interface.
+ * \verbatim
+ * This is the top-level function for @halt.
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg1 string representing object to halt.
+ * \param arg2 option string representing command to queue after halting.
+ */
 void
-do_halt1(player, arg1, arg2)
-    dbref player;
-    const char *arg1;
-    const char *arg2;
+do_halt1(dbref player, const char *arg1, const char *arg2)
 {
   dbref victim;
   if (*arg1 == '\0')
     do_halt(player, "", player);
   else {
     if ((victim =
-	 noisy_match_result(player, arg1, NOTYPE, MAT_OBJECTS)) == NOTHING)
+	 noisy_match_result(player, arg1, NOTYPE,
+			    MAT_OBJECTS | MAT_HERE)) == NOTHING)
       return;
     if (!Owns(player, victim) && !HaltAny(player)) {
       notify(player, T("Permission denied."));
@@ -1031,14 +1096,16 @@ do_halt1(player, arg1, arg2)
 		      Name(victim), unparse_dbref(victim), Name(player));
       }
       if (*arg2 == '\0')
-	Flags(victim) |= HALT;
+	set_flag_internal(victim, "HALT");
     }
   }
 }
 
+/** Halt all objects in the database.
+ * \param player the enactor.
+ */
 void
-do_allhalt(player)
-    dbref player;
+do_allhalt(dbref player)
 {
   dbref victim;
   if (!HaltAny(player)) {
@@ -1056,9 +1123,14 @@ do_allhalt(player)
   }
 }
 
+/** Restart all objects in the database.
+ * \verbatim
+ * A restart is a halt and then triggering the @startup.
+ * \endverbatim
+ * \param player the enactor.
+ */
 void
-do_allrestart(player)
-    dbref player;
+do_allrestart(dbref player)
 {
   dbref thing;
 
@@ -1068,8 +1140,10 @@ do_allrestart(player)
   }
   do_allhalt(player);
   for (thing = 0; thing < db_top; thing++) {
-    if (!IsGarbage(thing) && !(Halted(thing)))
+    if (!IsGarbage(thing) && !(Halted(thing))) {
       (void) queue_attribute_noparent(thing, "STARTUP", thing);
+      do_top(5);
+    }
     if (IsPlayer(thing)) {
       notify_format(thing,
 		    T("Your objects are being globally restarted by %s"),
@@ -1079,8 +1153,7 @@ do_allrestart(player)
 }
 
 static void
-do_raw_restart(victim)
-    dbref victim;
+do_raw_restart(dbref victim)
 {
   dbref thing;
 
@@ -1097,10 +1170,12 @@ do_raw_restart(victim)
   }
 }
 
+/** Restart an object.
+ * \param player the enactor.
+ * \param arg1 string representing the object to restart.
+ */
 void
-do_restart_com(player, arg1)
-    dbref player;
-    const char *arg1;
+do_restart_com(dbref player, const char *arg1)
 {
   dbref victim;
   if (*arg1 == '\0') {
@@ -1141,3 +1216,35 @@ do_restart_com(player, arg1)
     do_raw_restart(victim);
   }
 }
+
+/** Dequeue all queue entries, refunding deposits. 
+ * This function dequeues all entries in all queues, without executing
+ * them and refunds queue deposits. It's called at shutdown.
+ */
+void
+shutdown_queues(void)
+{
+  shutdown_a_queue(&qfirst, &qlast);
+  shutdown_a_queue(&qlfirst, &qllast);
+  shutdown_a_queue(&qsemfirst, &qsemlast);
+  shutdown_a_queue(&qwait, NULL);
+}
+
+
+static void
+shutdown_a_queue(BQUE **head, BQUE **tail)
+{
+  BQUE *entry;
+  /* Drain out a queue */
+  while (*head) {
+    entry = *head;
+    if (!(*head = entry->next) && tail)
+      *tail = NULL;
+    if (GoodObject(entry->player) && !IsGarbage(entry->player)) {
+      global_eval_context.cplr = entry->player;
+      giveto(global_eval_context.cplr, QUEUE_COST);
+      add_to(entry->queued, -1);
+    }
+    free_qentry(entry);
+  }
+}
