@@ -2,14 +2,8 @@
 
 #include "config.h"
 #include <stdio.h>
-#ifdef I_STDDEF
 #include <stddef.h>
-#endif
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #ifdef I_FCNTL
 #include <fcntl.h>
 #endif
@@ -60,16 +54,7 @@ extern char *strdup _((const char *));
  *  abstracted away from the source code.
  */
 
-#ifdef I_ERRNO
 #include <errno.h>
-#else
-#ifdef I_SYS_ERRNO
-#include <sys/errno.h>
-#endif
-#endif
-#ifndef errno
-extern int errno;		/* Some errno.h don't declare this */
-#endif
 
 #ifndef REGISTER
 #ifdef __GNUC__
@@ -97,9 +82,9 @@ extern int errno;		/* Some errno.h don't declare this */
  *  Malloc definitions from General Utilities <stdlib.h>. Note that we
  *  disagree with Berkeley Unix on the return type of free/cfree.
  */
-extern univptr_t malloc proto((Size_t));
-extern univptr_t calloc proto((Size_t, Size_t));
-extern univptr_t realloc proto((univptr_t, Size_t));
+extern univptr_t malloc proto((size_t));
+extern univptr_t calloc proto((size_t, size_t));
+extern univptr_t realloc proto((univptr_t, size_t));
 extern Free_t free proto((univptr_t));
 
 /* General Utilities <stdlib.h> */
@@ -120,24 +105,24 @@ extern int sprintf proto((char *, const char *, ...));
 extern int fputs proto((const char *, FILE *));
 extern int fflush proto((FILE *));
 #if 0
-extern int setvbuf proto((FILE *, char *, int, memSize_t));
+extern int setvbuf proto((FILE *, char *, int, memsize_t));
 #endif
 
 
 /* Character Handling: <string.h> */
 
 #ifndef HAS_MEMSET
-extern univptr_t memset proto((univptr_t, int, memSize_t));
+extern univptr_t memset proto((univptr_t, int, memsize_t));
 #endif
 
 #ifndef HAS_MEMCPY
-extern univptr_t memcpy proto((univptr_t, const univptr_t, memSize_t));
+extern univptr_t memcpy proto((univptr_t, const univptr_t, memsize_t));
 #endif
 
 #if 0
 /* We'd better not have to do this - Javelin */
 extern char *strcpy proto((char *, const char *));
-extern memSize_t strlen proto((const char *));
+extern memsize_t strlen proto((const char *));
 #endif
 
 /* UNIX -- unistd.h */
@@ -165,14 +150,14 @@ extern caddr_t sbrk proto((int));
 #endif
 
 /* Backwards compatibility with BSD/Sun -- these are going to vanish one day */
-extern univptr_t valloc proto((Size_t));
-extern univptr_t memalign proto((Size_t, Size_t));
+extern univptr_t valloc proto((size_t));
+extern univptr_t memalign proto((size_t, size_t));
 extern Free_t cfree proto((univptr_t));
 
 /* Malloc definitions - my additions.  Yuk, should use malloc.h properly!!  */
-extern univptr_t emalloc proto((Size_t));
-extern univptr_t ecalloc proto((Size_t, Size_t));
-extern univptr_t erealloc proto((univptr_t, Size_t));
+extern univptr_t emalloc proto((size_t));
+extern univptr_t ecalloc proto((size_t, size_t));
+extern univptr_t erealloc proto((univptr_t, size_t));
 extern char *strsave proto((const char *));
 #ifdef CSRI_DEBUG
 extern void mal_debug proto((int));
@@ -191,17 +176,17 @@ extern void mal_trace proto((int));
 
 /* Internal definitions */
 extern int __nothing proto((void));
-extern univptr_t _mal_sbrk proto((Size_t));
-extern univptr_t _mal_mmap proto((Size_t));
+extern univptr_t _mal_sbrk proto((size_t));
+extern univptr_t _mal_mmap proto((size_t));
 
 #ifdef HAVE_MMAP
-extern int madvise proto((caddr_t, Size_t, int));
-extern caddr_t mmap proto((caddr_t, Size_t, int, int, int, off_t));
+extern int madvise proto((caddr_t, size_t, int));
+extern caddr_t mmap proto((caddr_t, size_t, int, int, int, off_t));
 #endif
 
 #endif	/* EXTERNS_H__ */			/* Do not add anything after this line */
 
-/* $Id: csrimalloc.c 1.23.1.3 Sat, 20 Jul 2002 12:00:21 -0500 dunemush $ */
+/* $Id: csrimalloc.c 1.23.1.8 Wed, 26 May 2004 09:52:42 -0500 dunemush $ */
 #ifndef __ASSERT_H__
 #define __ASSERT_H__
 #ifdef CSRI_DEBUG
@@ -233,7 +218,7 @@ proto((const char *, const char *, univptr_t, int, const char *, int));
 #include <values.h>
 #endif
 
-/* $Id: csrimalloc.c 1.23.1.3 Sat, 20 Jul 2002 12:00:21 -0500 dunemush $ */
+/* $Id: csrimalloc.c 1.23.1.8 Wed, 26 May 2004 09:52:42 -0500 dunemush $ */
 #ifndef __ALIGN_H__
 #define __ALIGN_H__
 /* 
@@ -276,15 +261,15 @@ proto((const char *, const char *, univptr_t, int, const char *, int));
 #endif
 
 /* Align with power of 2 */
-#define SIMPLEALIGN(X, N) (univptr_t)(((Size_t)((char *)(X) + (N-1))) & ~(N-1))
+#define SIMPLEALIGN(X, N) (univptr_t)(((size_t)((char *)(X) + (N-1))) & ~(N-1))
 
 #if     NALIGN == 2 || NALIGN == 4 || NALIGN == 8 || NALIGN == 16
   /* if NALIGN is a power of 2, the next line will do ok */
 #define ALIGNPTR(X) SIMPLEALIGN(X, NALIGN)
 #else
   /* otherwise we need the generic version; hope the compiler isn't too smart */
-static Size_t _N = NALIGN;
-#define ALIGNPTR(X) (univptr_t)((((Size_t)((univptr_t)(X)+(NALIGN-1)))/_N)*_N)
+static size_t _N = NALIGN;
+#define ALIGNPTR(X) (univptr_t)((((size_t)((univptr_t)(X)+(NALIGN-1)))/_N)*_N)
 #endif
 
 /*
@@ -323,8 +308,8 @@ static Size_t _N = NALIGN;
 #define BITS(type)      (BITSPERBYTE * (int) sizeof(type))
 #endif				/* BITS */
 
-/* Size_t with only the high-order bit turned on */
-#define HIBITSZ (((Size_t) 1) << (BITS(Size_t) - 1))
+/* size_t with only the high-order bit turned on */
+#define HIBITSZ (((size_t) 1) << (BITS(size_t) - 1))
 
 #endif	/* __ALIGN_H__ */			/* Do not add anything after this line */
 
@@ -334,7 +319,7 @@ static Size_t _N = NALIGN;
  * should turn it off.
  */
 
-#define FREE            ((Size_t) 0)
+#define FREE            ((size_t) 0)
 #define ALLOCED         (HIBITSZ)
 #define SIZEMASK        (~HIBITSZ)
 
@@ -343,7 +328,7 @@ static Size_t _N = NALIGN;
 #define DEF_SBRKUNITS   1024
 
 union word {			/* basic unit of storage */
-  Size_t size;			/* size of this block + 1 bit status */
+  size_t size;			/* size of this block + 1 bit status */
   union word *next;		/* next free block */
   union word *prev;		/* prev free block */
   univptr_t ptr;		/* stops lint complaining, keeps alignment */
@@ -407,7 +392,7 @@ typedef union word Word;
  *  a block is free. We take advantage of that as a minor optimization.
  */
 #define SIZEFIELD(p)            ((p)->size)
-#define SIZE(p)                 ((Size_t) (SIZEFIELD(p) & SIZEMASK))
+#define SIZE(p)                 ((size_t) (SIZEFIELD(p) & SIZEMASK))
 #define ALLOCMASK(n)            ((n) | ALLOCED)
 #define FREESIZE(p)             SIZEFIELD(p)
 /*
@@ -647,10 +632,10 @@ typedef union word Word;
  *  package/C News src
  */
 #define MEMSET(p, c, n) \
-        (void) memset((univptr_t) (p), (c), (memSize_t) ((n) * sizeof(Word)))
+        (void) memset((univptr_t) (p), (c), (memsize_t) ((n) * sizeof(Word)))
 #define MEMCPY(p1, p2, n) \
         (void) memcpy((univptr_t) (p1), (univptr_t) (p2), \
-                                  (memSize_t) ((n) * sizeof(Word)))
+                                  (memsize_t) ((n) * sizeof(Word)))
 
 
 #ifdef CSRI_DEBUG
@@ -684,7 +669,7 @@ typedef union word Word;
  *  Remember to initialize the variable in globals.c if you want, and
  *  provide an alternative short name in globrename.h
  */
-/* $Id: csrimalloc.c 1.23.1.3 Sat, 20 Jul 2002 12:00:21 -0500 dunemush $ */
+/* $Id: csrimalloc.c 1.23.1.8 Wed, 26 May 2004 09:52:42 -0500 dunemush $ */
 #ifndef __GLOBALRENAME_H__
 #define __GLOBALRENAME_H__
 /*
@@ -744,7 +729,7 @@ const char *_malloc_version =
  *  boundary tags, one for the next and one for the prev pointers in a
  *  free block.
  */
-Size_t _malloc_minchunk = FIXEDOVERHEAD;
+size_t _malloc_minchunk = FIXEDOVERHEAD;
 
 /*
  * _malloc_rovers is the array of pointers into that each point 'someplace'
@@ -754,7 +739,7 @@ Size_t _malloc_minchunk = FIXEDOVERHEAD;
  * improves performance considerably, sez Knuth
  */
 Word *_malloc_rovers[MAXBINS + 1] = { NULL };
-const Size_t _malloc_binmax[MAXBINS] = {
+const size_t _malloc_binmax[MAXBINS] = {
   8, 16, 32, 64, 128, 256, 512, 1024
 };
 int _malloc_firstbin = 0;
@@ -772,7 +757,7 @@ Word *_malloc_loword = NULL;
  *  modern programs that malloc heavily. For small programs, you may
  *  want to set it to a lower number.
  */
-Size_t _malloc_sbrkunits = DEF_SBRKUNITS;
+size_t _malloc_sbrkunits = DEF_SBRKUNITS;
 
 Word *_malloc_mem = NULL;
 
@@ -799,8 +784,8 @@ int _malloc_scount[MAXPROFILESIZE];
 int _malloc_debugging = 0;
 #endif				/* CSRI_DEBUG */
 
-univptr_t (*_malloc_memfunc) proto((Size_t)) = _mal_sbrk;
-/* $Id: csrimalloc.c 1.23.1.3 Sat, 20 Jul 2002 12:00:21 -0500 dunemush $ */
+univptr_t (*_malloc_memfunc) proto((size_t)) = _mal_sbrk;
+
 #ifndef __GLOBALS_H__
 #define __GLOBALS_H__
 /*
@@ -809,11 +794,11 @@ univptr_t (*_malloc_memfunc) proto((Size_t)) = _mal_sbrk;
  */
 
 extern
-Size_t _malloc_minchunk;
+size_t _malloc_minchunk;
 
 extern Word *_malloc_rovers[];
 extern const
-Size_t _malloc_binmax[];
+size_t _malloc_binmax[];
 extern int
  _malloc_firstbin;
 extern int
@@ -822,7 +807,7 @@ extern Word *_malloc_hiword;
 extern Word *_malloc_loword;
 
 extern
-Size_t _malloc_sbrkunits;
+size_t _malloc_sbrkunits;
 
 extern Word *_malloc_mem;
 
@@ -850,7 +835,7 @@ extern int
 #endif				/* CSRI_DEBUG */
 
 extern
-univptr_t (*_malloc_memfunc) proto((Size_t));
+univptr_t (*_malloc_memfunc) proto((size_t));
 
 extern int
 __m_prblock proto((univptr_t, int, FILE *));
@@ -959,42 +944,44 @@ __m_delete_record proto((univptr_t));
 #ifdef CSRI_TRACE
 /* Tracing malloc definitions - helps find leaks */
 univptr_t
-__malloc _((Size_t nbytes, const char *fname, int linenum));
+trace__malloc _((size_t nbytes, const char *fname, int linenum));
 univptr_t
-__calloc _((Size_t nelem, Size_t elsize, const char *fname, int linenum));
+trace__calloc _((size_t nelem, size_t elsize, const char *fname, int linenum));
 univptr_t
-__realloc _((univptr_t cp, Size_t nbytes, const char *fname, int linenum));
-univptr_t __valloc _((Size_t size, const char *fname, int linenum));
+trace__realloc _((univptr_t cp, size_t nbytes, const char *fname, int linenum));
+univptr_t trace__valloc _((size_t size, const char *fname, int linenum));
 univptr_t
-__memalign _((Size_t alignment, Size_t size, const char *fname, int linenum));
-univptr_t __emalloc _((Size_t nbytes, const char *fname, int linenum));
+ trace__memalign
+_((size_t alignment, size_t size, const char *fname, int linenum));
+univptr_t trace__emalloc _((size_t nbytes, const char *fname, int linenum));
 univptr_t
-__ecalloc _((Size_t nelem, Size_t sz, const char *fname, int linenum));
+trace__ecalloc _((size_t nelem, size_t sz, const char *fname, int linenum));
 univptr_t
-__erealloc _((univptr_t ptr, Size_t nbytes, const char *fname, int linenum));
-char *__strdup _((const char *s, const char *fname, int linenum));
-char *__strsave _((const char *s, const char *fname, int linenum));
+ trace__erealloc
+_((univptr_t ptr, size_t nbytes, const char *fname, int linenum));
+char *trace__strdup _((const char *s, const char *fname, int linenum));
+char *trace__strsave _((const char *s, const char *fname, int linenum));
 void
-__free _((univptr_t cp, const char *fname, int linenum));
+trace__free _((univptr_t cp, const char *fname, int linenum));
 void
-__cfree _((univptr_t cp, const char *fname, int linenum));
+trace__cfree _((univptr_t cp, const char *fname, int linenum));
 #else				/* CSRI_TRACE */
 univptr_t
-malloc _((Size_t nbytes));
+malloc _((size_t nbytes));
 univptr_t
-calloc _((Size_t nelem, Size_t elsize));
+calloc _((size_t nelem, size_t elsize));
 univptr_t
-realloc _((univptr_t cp, Size_t nbytes));
+realloc _((univptr_t cp, size_t nbytes));
 univptr_t
-valloc _((Size_t size));
+valloc _((size_t size));
 univptr_t
-memalign _((Size_t alignment, Size_t size));
+memalign _((size_t alignment, size_t size));
 univptr_t
-emalloc _((Size_t nbytes));
+emalloc _((size_t nbytes));
 univptr_t
-ecalloc _((Size_t nelem, Size_t sz));
+ecalloc _((size_t nelem, size_t sz));
 univptr_t
-erealloc _((univptr_t ptr, Size_t nbytes));
+erealloc _((univptr_t ptr, size_t nbytes));
 Free_t free _((univptr_t cp));
 Free_t cfree _((univptr_t cp));
 #endif				/* CSRI_TRACE */
@@ -1035,8 +1022,8 @@ mal_mmap _((char *fname));
 #ifdef CSRI_TRACE
 
 univptr_t
-__emalloc(nbytes, fname, linenum)
-    Size_t nbytes;
+trace__emalloc(nbytes, fname, linenum)
+    size_t nbytes;
     const char *fname;
     int
      linenum;
@@ -1051,10 +1038,10 @@ __emalloc(nbytes, fname, linenum)
 
 
 univptr_t
-__erealloc(ptr, nbytes, fname, linenum)
+trace__erealloc(ptr, nbytes, fname, linenum)
     univptr_t
      ptr;
-    Size_t nbytes;
+    size_t nbytes;
     const char *fname;
     int
      linenum;
@@ -1068,8 +1055,8 @@ __erealloc(ptr, nbytes, fname, linenum)
 }
 
 univptr_t
-__ecalloc(nelem, sz, fname, linenum)
-    Size_t nelem, sz;
+trace__ecalloc(nelem, sz, fname, linenum)
+    size_t nelem, sz;
     const char *fname;
     int
      linenum;
@@ -1095,8 +1082,8 @@ __ecalloc(nelem, sz, fname, linenum)
 /*LINTLIBRARY */
 
 univptr_t
-__malloc(nbytes, fname, linenum)
-    Size_t nbytes;
+trace__malloc(nbytes, fname, linenum)
+    size_t nbytes;
     const char *fname;
     int
      linenum;
@@ -1111,7 +1098,7 @@ __malloc(nbytes, fname, linenum)
 
 /* ARGSUSED if CSRI_TRACE is not defined */
 void
-__free(cp, fname, linenum)
+trace__free(cp, fname, linenum)
     univptr_t cp;
     const char *fname;
     int linenum;
@@ -1122,10 +1109,10 @@ __free(cp, fname, linenum)
 }
 
 univptr_t
-__realloc(cp, nbytes, fname, linenum)
+trace__realloc(cp, nbytes, fname, linenum)
     univptr_t
      cp;
-    Size_t nbytes;
+    size_t nbytes;
     const char *fname;
     int
      linenum;
@@ -1143,8 +1130,8 @@ __realloc(cp, nbytes, fname, linenum)
 }
 
 univptr_t
-__calloc(nelem, elsize, fname, linenum)
-    Size_t nelem, elsize;
+trace__calloc(nelem, elsize, fname, linenum)
+    size_t nelem, elsize;
     const char *fname;
     int
      linenum;
@@ -1159,7 +1146,7 @@ __calloc(nelem, elsize, fname, linenum)
 
 /* ARGSUSED if CSRI_TRACE is not defined */
 void
-__cfree(cp, fname, linenum)
+trace__cfree(cp, fname, linenum)
     univptr_t cp;
     const char *fname;
     int linenum;
@@ -1177,8 +1164,8 @@ __cfree(cp, fname, linenum)
 
 
 univptr_t
-__memalign(alignment, size, fname, linenum)
-    Size_t alignment, size;
+trace__memalign(alignment, size, fname, linenum)
+    size_t alignment, size;
     const char *fname;
     int
      linenum;
@@ -1192,8 +1179,8 @@ __memalign(alignment, size, fname, linenum)
 }
 
 univptr_t
-__valloc(size, fname, linenum)
-    Size_t size;
+trace__valloc(size, fname, linenum)
+    size_t size;
     const char *fname;
     int
      linenum;
@@ -1211,7 +1198,7 @@ __valloc(size, fname, linenum)
 /*LINTLIBRARY */
 
 char *
-__strdup(s, fname, linenum)
+trace__strdup(s, fname, linenum)
     const char *s;
     const char *fname;
     int linenum;
@@ -1225,7 +1212,7 @@ __strdup(s, fname, linenum)
 }
 
 char *
-__strsave(s, fname, linenum)
+trace__strsave(s, fname, linenum)
     const char *s;
     const char *fname;
     int linenum;
@@ -1461,7 +1448,7 @@ mal_heapdump(fp)
  */
 univptr_t
 emalloc(nbytes)
-    Size_t nbytes;
+    size_t nbytes;
 {
   univptr_t cp = malloc(nbytes);
 
@@ -1484,7 +1471,7 @@ univptr_t
 erealloc(ptr, nbytes)
     univptr_t
      ptr;
-    Size_t nbytes;
+    size_t nbytes;
 {
   univptr_t cp = realloc(ptr, nbytes);
 
@@ -1505,12 +1492,12 @@ erealloc(ptr, nbytes)
  */
 univptr_t
 ecalloc(nelem, sz)
-    Size_t nelem, sz;
+    size_t nelem, sz;
 {
-  Size_t nbytes = nelem * sz;
+  size_t nbytes = nelem * sz;
   univptr_t cp = emalloc(nbytes);
 
-  (void) memset((univptr_t) cp, 0, (memSize_t) nbytes);
+  (void) memset((univptr_t) cp, 0, (memsize_t) nbytes);
   return (cp);
 }
 
@@ -1523,7 +1510,7 @@ ecalloc(nelem, sz)
 /* gets memory from the system via the sbrk() system call.  Most Un*xes */
 univptr_t
 _mal_sbrk(nbytes)
-    Size_t nbytes;
+    size_t nbytes;
 {
   static char *lastsbrk;
   char *p;
@@ -1553,21 +1540,21 @@ _mal_sbrk(nbytes)
 static char *mmap_filename;
 
 #ifdef HAVE_MMAP
-/* Sun gets Size_t wrong, and these follow, thanks to my #defines! */
+/* Sun gets size_t wrong, and these follow, thanks to my #defines! */
 #undef caddr_t
-#undef Size_t
+#undef size_t
 #undef u_char
 
 univptr_t
 _mal_mmap(nbytes)
-    Size_t nbytes;
+    size_t nbytes;
 {
   static struct {
     int i_fd;
     caddr_t i_data;
     caddr_t i_end;
-    Size_t i_size;
-    Size_t i_alloced;
+    size_t i_size;
+    size_t i_alloced;
   } mmf;
   struct stat stbuf;
 
@@ -1614,7 +1601,7 @@ _mal_mmap(nbytes)
 #else				/* !HAVE_MMAP */
 univptr_t
 _mal_mmap(nbytes)
-    Size_t nbytes __attribute__ ((__unused__));
+    size_t nbytes __attribute__ ((__unused__));
 {
   return (univptr_t) -1;
 }
@@ -1852,7 +1839,7 @@ mal_contents(fp)
 
 #define CARVE(ep, epsize, epbin, reqsize) \
          { \
-                REGISTER Size_t eprest = epsize - reqsize; \
+                REGISTER size_t eprest = epsize - reqsize; \
                 int newepbin; \
                 \
                 if (eprest >= _malloc_minchunk) { \
@@ -1871,17 +1858,17 @@ mal_contents(fp)
                 } \
          }
 
-static int grabhunk _((Size_t));
+static int grabhunk _((size_t));
 
 static int
 grabhunk(nwords)
-    Size_t nwords;
+    size_t nwords;
 {
   univptr_t cp;
-  Size_t morecore;
+  size_t morecore;
   Word *ptr;
-  Size_t sbrkwords;
-  Size_t blksize;
+  size_t sbrkwords;
+  size_t blksize;
   static char *spare;
   static int nspare;
 
@@ -1890,7 +1877,7 @@ grabhunk(nwords)
    *  for the next ptr of the block.
    */
 #define EXCESS 3
-  sbrkwords = (Size_t) (((nwords + EXCESS) / _malloc_sbrkunits + 1) *
+  sbrkwords = (size_t) (((nwords + EXCESS) / _malloc_sbrkunits + 1) *
 			_malloc_sbrkunits);
   morecore = sbrkwords * sizeof(Word) + SBRKEXTRA;
   if ((cp = (*_malloc_memfunc) (morecore)) == (univptr_t) -1)
@@ -1995,12 +1982,12 @@ grabhunk(nwords)
 
 univptr_t
 malloc(nbytes)
-    Size_t nbytes;
+    size_t nbytes;
 {
   REGISTER Word *start, *search = NULL;
   REGISTER Word *p;
-  REGISTER Size_t required;
-  REGISTER Size_t searchsize;
+  REGISTER size_t required;
+  REGISTER size_t searchsize;
   int bin;
 
 #ifdef SVID_MALLOC_0
@@ -2012,7 +1999,7 @@ malloc(nbytes)
 #endif				/* SVID_MALLOC_0 */
 
   required = ALLOC_OVERHEAD + (nbytes + sizeof(Word) - 1) / sizeof(Word);
-  if (required < (Size_t) _malloc_minchunk)
+  if (required < (size_t) _malloc_minchunk)
     required = _malloc_minchunk;
 
   searchsize = 0;
@@ -2088,7 +2075,7 @@ free(cp)
    *  We can either coalesce with P1, P2, both, or neither
    */
   REGISTER Word *p0, *p1, *p2;
-  REGISTER Size_t sizep0;
+  REGISTER size_t sizep0;
   int bin, oldbin = -1;
 
   if (cp == NULL)
@@ -2212,13 +2199,13 @@ univptr_t
 realloc(cp, nbytes)
     univptr_t
      cp;
-    Size_t nbytes;
+    size_t nbytes;
 {
   REGISTER Word *p0 = (Word *) cp;
   REGISTER Word *p1;
   univptr_t tmp;
-  REGISTER Size_t required;
-  REGISTER Size_t sizep0;
+  REGISTER size_t required;
+  REGISTER size_t sizep0;
   int bin;
 
   if (p0 == NULL)
@@ -2229,7 +2216,7 @@ realloc(cp, nbytes)
     return (NULL);
   }
   required = ALLOC_OVERHEAD + (nbytes + sizeof(Word) - 1) / sizeof(Word);
-  if (required < (Size_t) _malloc_minchunk)
+  if (required < (size_t) _malloc_minchunk)
     required = _malloc_minchunk;
 
   p0 -= HEADERWORDS;
@@ -2243,7 +2230,7 @@ realloc(cp, nbytes)
   sizep0 = SIZE(p0);
   if (sizep0 >= required) {
     /* Shrinking the block */
-    Size_t after = sizep0 - required;
+    size_t after = sizep0 - required;
 
     SET_REALSIZE(p0, nbytes);
     if (after < _malloc_minchunk) {
@@ -2309,13 +2296,13 @@ realloc(cp, nbytes)
  */
 univptr_t
 calloc(nelem, elsize)
-    Size_t nelem, elsize;
+    size_t nelem, elsize;
 {
-  REGISTER Size_t nbytes = nelem * elsize;
+  REGISTER size_t nbytes = nelem * elsize;
   REGISTER univptr_t cp = malloc(nbytes);
 
   if (cp)
-    (void) memset((univptr_t) cp, 0, (memSize_t) nbytes);
+    (void) memset((univptr_t) cp, 0, (memsize_t) nbytes);
   return (cp);
 }
 
@@ -2360,13 +2347,13 @@ cfree(cp)
  */
 univptr_t
 memalign(alignment, size)
-    Size_t alignment, size;
+    size_t alignment, size;
 {
   univptr_t cp;
   univptr_t addr;
   REGISTER Word *p0, *p1;
-  REGISTER Size_t before, after;
-  Size_t blksize;
+  REGISTER size_t before, after;
+  size_t blksize;
 #ifdef CSRI_DEBUG
   int tmp_debugging = _malloc_debugging;
 #endif				/* CSRI_DEBUG */
@@ -2382,7 +2369,7 @@ memalign(alignment, size)
    * enough to hold a block 'size' long, starting on an 'alignment'
    * boundary
    */
-  if ((cp = malloc((Size_t) (size + alignment - 1))) == NULL)
+  if ((cp = malloc((size_t) (size + alignment - 1))) == NULL)
     return (NULL);
   addr = SIMPLEALIGN(cp, alignment);
   /*
@@ -2493,12 +2480,12 @@ memalign(alignment, size)
 /* Just following the Sun manual page here */
 univptr_t
 valloc(size)
-    Size_t size;
+    size_t size;
 {
-  static Size_t pagesz = 0;
+  static size_t pagesz = 0;
 
   if (pagesz == 0)
-    pagesz = (Size_t) getpagesize();
+    pagesz = (size_t) getpagesize();
   return (memalign(pagesz, size));
 }
 
@@ -2650,7 +2637,7 @@ strsave(s)
     const char *s;
 {
   if (s)
-    return (strcpy((char *) emalloc((Size_t) (strlen(s) + 1)), s));
+    return (strcpy((char *) emalloc((size_t) (strlen(s) + 1)), s));
   else
     return ((char *) NULL);
 }
@@ -2776,7 +2763,7 @@ static SPBLK *spfnext proto((SPBLK *));
 
 /* USER SUPPLIED! */
 
-extern univptr_t emalloc proto((Size_t));
+extern univptr_t emalloc proto((size_t));
 
 #if 0
 static SPBLK *
