@@ -36,6 +36,10 @@
 #include "extmail.h"
 #include "confmagic.h"
 
+#ifdef WIN32
+#pragma warning( disable : 4761)	/* disable warning re conversion */
+#endif
+
 #ifdef WIN32SERVICES
 void shutdown_checkpoint(void);
 #endif
