@@ -1,33 +1,19 @@
-/* destroy.c */
-#include "config.h"
-
-#include <ctype.h>
-#include <assert.h>
-#include <limits.h>
-#include <stdlib.h>
-#ifdef I_STRING
-#include <string.h>
-#else
-#include <strings.h>
-#endif
-
-#include "copyrite.h"
-#include "conf.h"
-#include "mushdb.h"
-#include "match.h"
-#include "externs.h"
-#include "log.h"
-#include "game.h"
-#include "extmail.h"
-#include "malias.h"
-#include "attrib.h"
-#include "dbdefs.h"
-#include "flags.h"
-#include "lock.h"
-#include "confmagic.h"
-
-/* These lengthy comments are by Ralph Melton, December 1995. */
-/*
+/**
+ * \file destroy.c
+ *
+ * \brief Destroying objects and consistency checking.
+ *
+ * This file has two main parts. One part is the functions for destroying
+ * objects and getting objects off of the free list. The major public
+ * functions here are do_destroy(), free_get(), and purge().
+ *
+ * The other part is functions for checking the consistency of the
+ * database, and repairing any inconsistencies that are found. The
+ * major function in this group is dbck().
+ * 
+ *
+ * These lengthy comments are by Ralph Melton, December 1995.
+ *
  * First, a discourse on the theory of how we handle destruction.
  *
  * We want to maintain the following invariants:
@@ -62,21 +48,37 @@
  * Note that phases 2 and 3 do not have to happen immediately after Phase 1.
  * To allow some delay, we set the object GOING, and then process it on
  * the check that happens every ten minutes. 
- */
-
-/*
- * This file has two main parts. One part is the functions for destroying
- * objects and getting objects off of the free list. The major public
- * functions here are do_destroy(), free_get(), and purge().
  *
- * The other part is functions for checking the consistency of the
- * database, and repairing any inconsistencies that are found. The
- * major function in this group is dbck().
  */
 
-dbref first_free = NOTHING;
-static dbref what_to_destroy(dbref player, char *name, int confirm);
+#include "config.h"
 
+#include <ctype.h>
+#include <assert.h>
+#include <limits.h>
+#include <stdlib.h>
+#include <string.h>
+
+#include "copyrite.h"
+#include "conf.h"
+#include "mushdb.h"
+#include "match.h"
+#include "externs.h"
+#include "log.h"
+#include "game.h"
+#include "extmail.h"
+#include "malias.h"
+#include "attrib.h"
+#include "dbdefs.h"
+#include "flags.h"
+#include "lock.h"
+#include "confmagic.h"
+
+
+
+dbref first_free = NOTHING;   /**< Object at top of free list */
+
+static dbref what_to_destroy(dbref player, char *name, int confirm);
 static void pre_destroy(dbref player, dbref thing);
 static void free_object(dbref thing);
 static void empty_contents(dbref thing);
@@ -94,13 +96,20 @@ static void check_contents(void);
 static void check_locations(void);
 static void check_zones(void);
 static int attribute_owner_helper
-  (dbref player, dbref thing, char const *pattern, ATTR *atr, void *args);
+  (dbref player, dbref thing, dbref parent, char const *pattern, ATTR *atr,
+   void *args);
 
 extern void remove_all_obj_chan(dbref thing);
 extern void chan_chownall(dbref old, dbref new);
 
 extern struct db_stat_info current_state;
 
+/** Mark an object */
+#define SetMarked(x)    Type(x) |= TYPE_MARKED
+/** Unmark an object */
+#define ClearMarked(x)  Type(x) &= ~TYPE_MARKED
+
+
 /* Section I: do_destroy() and related functions. This section is where
  * the human interface of do_destroy() should largely be determined.
  */
@@ -190,16 +199,13 @@ extern struct db_stat_info current_state;
  */
 
 
-static dbref
-what_to_destroy(player, name, confirm)
-    dbref player;
-    char *name;
-    int confirm;
-
-/* Do all matching and permissions checking. Returns the object to be
+/** Determine what object to destroy and if we're allowed.
+ * Do all matching and permissions checking. Returns the object to be
  * destroyed if all the permissions checks are successful, otherwise
  * return NOTHING.
  */
+static dbref
+what_to_destroy(dbref player, char *name, int confirm)
 {
   dbref thing;
 
@@ -228,8 +234,8 @@ what_to_destroy(player, name, confirm)
     notify(player, T("Permission denied."));
     return NOTHING;
   }
-  if (thing == PLAYER_START || thing == MASTER_ROOM || thing == BASE_ROOM
-      || God(thing)) {
+  if (thing == PLAYER_START || thing == MASTER_ROOM || thing == BASE_ROOM ||
+      thing == DEFAULT_HOME || God(thing)) {
     notify(player, T("That is too special to be destroyed."));
     return NOTHING;
   }
@@ -289,16 +295,8 @@ what_to_destroy(player, name, confirm)
 	     T("That object is set WIZARD. You must use @nuke to destroy it."));
       return NOTHING;
     }
-    if (thing == Home(player) && !IsExit(player)) {
-      notify(player, T("No home-wrecking allowed! Relink first."));
-      return NOTHING;
-    }
     break;
   case TYPE_ROOM:
-    if (thing == Home(player) && !IsExit(player)) {
-      notify(player, T("No home-wrecking allowed! Relink first."));
-      return NOTHING;
-    }
     break;
   case TYPE_EXIT:
     break;
@@ -308,11 +306,16 @@ what_to_destroy(player, name, confirm)
 }
 
 
+/** User interface to destroy an object.
+ * \verbatim
+ * This is the top-level function for @destroy.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of object to destroy.
+ * \param confirm if 1, called with /override (or nuke).
+ */
 void
-do_destroy(player, name, confirm)
-    dbref player;
-    char *name;
-    int confirm;
+do_destroy(dbref player, char *name, int confirm)
 {
   dbref thing;
   thing = what_to_destroy(player, name, confirm);
@@ -336,7 +339,7 @@ do_destroy(player, name, confirm)
   case TYPE_ROOM:
     /* wait until dbck */
     notify_except(Contents(thing), NOTHING,
-		  T("The room shakes and begins to crumble."));
+		  T("The room shakes and begins to crumble."), 0);
     if (Owns(player, thing))
       notify_format(player,
 		    T("You will be rewarded shortly for %s."),
@@ -400,13 +403,17 @@ do_destroy(player, name, confirm)
 }
 
 
-/* Not undestroy, quite--it's actually 'remove it from its status as about
+/** Spare an object from slated destruction.
+ * \verbatim
+ * This is the top-level function for @undestroy.
+ * Not undestroy, quite--it's actually 'remove it from its status as about
  * to be destroyed.'
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of object to be spared.
  */
 void
-do_undestroy(player, name)
-    dbref player;
-    char *name;
+do_undestroy(dbref player, char *name)
 {
   dbref thing;
   thing = noisy_match_result(player, name, NOTYPE, MAT_EVERYTHING);
@@ -417,14 +424,17 @@ do_undestroy(player, name)
     notify(player, T("Alas, your efforts of mercy are in vain."));
     return;
   }
-  undestroy(player, thing);
-  notify_format(Owner(thing),
-		T("Your %s has been spared from destruction."),
-		object_header(Owner(thing), thing));
-  if (player != Owner(thing)) {
-    notify_format(player,
-		  T("%s's %s has been spared from destruction."),
-		  Name(Owner(thing)), object_header(player, thing));
+  if (undestroy(player, thing)) {
+    notify_format(Owner(thing),
+		  T("Your %s has been spared from destruction."),
+		  object_header(Owner(thing), thing));
+    if (player != Owner(thing)) {
+      notify_format(player,
+		    T("%s's %s has been spared from destruction."),
+		    Name(Owner(thing)), object_header(player, thing));
+    }
+  } else {
+    notify(player, T("That can't be undestroyed."));
   }
 }
 
@@ -436,17 +446,15 @@ do_undestroy(player, name)
 
 /* Schedule something to be destroyed, run @adestroy, etc. */
 static void
-pre_destroy(player, thing)
-    dbref player;
-    dbref thing;
+pre_destroy(dbref player, dbref thing)
 {
   dbref tmp;
   if (Going(thing) || IsGarbage(thing)) {
     /* we've already covered this thing. No need to do so again. */
     return;
   }
-  Flags(thing) |= GOING;
-  Flags(thing) &= ~GOING_TWICE;
+  set_flag_internal(thing, "GOING");
+  clear_flag_internal(thing, "GOING_TWICE");
 
   /* Present informative messages, and do recursive destruction. */
   switch (Typeof(thing)) {
@@ -491,20 +499,23 @@ pre_destroy(player, thing)
 }
 
 
-/* Not undestroy, quite--it's actually 'remove it from its status as about
- * to be destroyed.'
+/** Spare an object from destruction.
+ * Not undestroy, quite--it's actually 'remove it from its status as about
+ * to be destroyed.' This is the internal function used in hardcode.
+ * \param player the enactor.
+ * \param thing dbref of object to be spared.
+ * \return 1 successful undestruction.
+ * \return 0 thing is not a valid object to undestroy.
  */
-void
-undestroy(player, thing)
-    dbref player;
-    dbref thing;
+int
+undestroy(dbref player, dbref thing)
 {
   dbref tmp;
-  if (!Going(thing)) {
-    return;
+  if (!Going(thing) || IsGarbage(thing)) {
+    return 0;
   }
-  Flags(thing) &= ~GOING;
-  Flags(thing) &= ~GOING_TWICE;
+  clear_flag_internal(thing, "GOING");
+  clear_flag_internal(thing, "GOING_TWICE");
   if (!Halted(thing))
     (void) queue_attribute_noparent(thing, "STARTUP", thing);
   /* undestroy owner, if need be. */
@@ -519,7 +530,7 @@ undestroy(player, thing)
     } else {
       notify(player, T("You have been spared from destruction."));
     }
-    undestroy(player, Owner(thing));
+    (void) undestroy(player, Owner(thing));
   }
   switch (Typeof(thing)) {
   case TYPE_PLAYER:
@@ -532,7 +543,7 @@ undestroy(player, thing)
 	if (Owns(thing, tmp) &&
 	    (tmp != thing) &&
 	    !(IsExit(tmp) && !Owns(thing, Source(tmp)) && Going(Source(tmp)))) {
-	  undestroy(player, tmp);
+	  (void) undestroy(player, tmp);
 	}
       }
     break;
@@ -541,7 +552,7 @@ undestroy(player, thing)
   case TYPE_EXIT:
     /* undestroy containing room. */
     if (Going(Source(thing))) {
-      undestroy(player, Source(thing));
+      (void) undestroy(player, Source(thing));
       notify_format(player,
 		    T("The room %s has been spared from destruction."),
 		    object_header(player, Source(thing)));
@@ -559,31 +570,29 @@ undestroy(player, thing)
      */
     DOLIST(tmp, Exits(thing)) {
       if (DESTROY_POSSESSIONS ? (!Going(Owner(tmp)) || Safe(tmp)) : 1) {
-	undestroy(player, tmp);
+	(void) undestroy(player, tmp);
       }
     }
     break;
   default:
     do_log(LT_ERR, NOTHING, NOTHING, T("Surprising type in un_destroy."));
-    return;
+    return 0;
   }
+  return 1;
 }
 
 
-static void
-free_object(thing)
-    dbref thing;
-/* Does the real work of freeing all the memory and unlinking an object. */
-/* This is going to have to be very tightly coupled with the implementation;
+/* Does the real work of freeing all the memory and unlinking an object.
+ * This is going to have to be very tightly coupled with the implementation;
  * if the database format changes, this will likely have to change too.
  */
+static void
+free_object(dbref thing)
 {
   dbref i, loc;
   if (!GoodObject(thing))
     return;
-#ifdef LOCAL_DATA
   local_data_free(thing);
-#endif
   switch (Typeof(thing)) {
   case TYPE_THING:
     clear_thing(thing);
@@ -602,9 +611,7 @@ free_object(thing)
 	   thing);
     return;
   }
-#ifdef QUOTA
   change_quota(Owner(thing), QUOTA_COST);
-#endif				/* QUOTA */
   do_halt(thing, "", thing);
   /* The equivalent of an @drain/any/all: */
   dequeue_semaphores(thing, NULL, INT_MAX, 1, 1);
@@ -622,7 +629,7 @@ free_object(thing)
       switch (Typeof(i)) {
       case TYPE_PLAYER:
       case TYPE_THING:
-	Home(i) = PLAYER_START;
+	Home(i) = DEFAULT_HOME;
 	break;
       case TYPE_EXIT:
 	/* Huh.  An exit that claims to be from here, but wasn't linked
@@ -680,15 +687,13 @@ free_object(thing)
   Owner(thing) = GOD;
   Parent(thing) = NOTHING;
   Zone(thing) = NOTHING;
-#ifdef CHAT_SYSTEM
   remove_all_obj_chan(thing);
-#endif
 
   switch (Typeof(thing)) {
     /* Make absolutely sure we are removed from Location's content or
        exit list. If we are in a room we own and destroy_possessions
        is yes, this can happen, causing much ickyness: All garbage
-       items would be in PLAYER_START. */
+       items would be in DEFAULT_HOME. */
   case TYPE_PLAYER:
   case TYPE_THING:
     loc = Location(thing);
@@ -713,12 +718,18 @@ free_object(thing)
     break;
   }
 
-  Flags(thing) = TYPE_GARBAGE;
-  Toggles(thing) = Powers(thing) = 0;
+  Type(thing) = TYPE_GARBAGE;
+  destroy_flag_bitmask(Flags(thing));
+  Flags(thing) = NULL;
+  destroy_flag_bitmask(Powers(thing));
+  Powers(thing) = NULL;
   Location(thing) = NOTHING;
   set_name(thing, "Garbage");
   Exits(thing) = NOTHING;
   Home(thing) = NOTHING;
+  CreTime(thing) = 0;		/* Prevents it from matching objids */
+
+  clear_objdata(thing);
 
   Next(thing) = first_free;
   first_free = thing;
@@ -731,8 +742,7 @@ free_object(thing)
 }
 
 static void
-empty_contents(thing)
-    dbref thing;
+empty_contents(dbref thing)
 {
   /* Destroy any exits they may be carrying, send everything else home. */
   dbref first;
@@ -740,7 +750,8 @@ empty_contents(thing)
   dbref target;
   notify_except(Contents(thing), NOTHING,
 		T
-		("The floor disappears under your feet, you fall through NOTHINGness and then:"));
+		("The floor disappears under your feet, you fall through NOTHINGness and then:"),
+		0);
   first = Contents(thing);
   Contents(thing) = NOTHING;
   /* send all objects to nowhere */
@@ -760,11 +771,11 @@ empty_contents(thing)
       /* Make sure the home is a reasonable object. */
       if (!GoodObject(Home(first)) || IsExit(Home(first)) ||
 	  Home(first) == thing)
-	Home(first) = PLAYER_START;
+	Home(first) = DEFAULT_HOME;
       target = Home(first);
-      /* If home isn't a good place to send it, send it to PLAYER_START. */
+      /* If home isn't a good place to send it, send it to DEFAULT_HOME. */
       if (!GoodObject(target) || recursive_member(target, first, 0))
-	target = PLAYER_START;
+	target = DEFAULT_HOME;
       if (target != NOTHING) {
 	/* Use enter_room() on everything so that AENTER and such
 	 * are all triggered properly. */
@@ -777,8 +788,7 @@ empty_contents(thing)
 }
 
 static void
-clear_thing(thing)
-    dbref thing;
+clear_thing(dbref thing)
 {
   dbref loc;
   int a;
@@ -793,8 +803,7 @@ clear_thing(thing)
   /* give player money back */
   giveto(Owner(thing), (a = Pennies(thing)));
   empty_contents(thing);
-  if (Toggles(thing) & THING_PUPPET)
-    Toggles(thing) &= ~THING_PUPPET;
+  clear_flag_internal(thing, "PUPPET");
   if (!Quiet(thing) && !Quiet(Owner(thing)))
     notify_format(Owner(thing),
 		  T("You get your %d %s deposit back for %s."),
@@ -803,31 +812,24 @@ clear_thing(thing)
 }
 
 static void
-clear_player(thing)
-    dbref thing;
+clear_player(dbref thing)
 {
   dbref i;
   ATTR *atemp;
   char alias[BUFFER_LEN + 1];
 
   /* Clear out mail. */
-#ifdef USE_MAILER
   do_mail_clear(thing, NULL);
   do_mail_purge(thing);
-#ifdef MAIL_ALIASES
   malias_cleanup(thing);
-#endif
-#endif
 
   /* Chown any chat channels they own to God */
-#ifdef CHAT_SYSTEM
   chan_chownall(thing, GOD);
-#endif
 
   /* Clear out names from the player list */
   delete_player(thing, NULL);
   if ((atemp = atr_get_noparent(thing, "ALIAS")) != NULL) {
-    strcpy(alias, uncompress(atemp->value));
+    strcpy(alias, atr_value(atemp));
     delete_player(thing, alias);
   }
   /* Do all the thing-esque manipulations. */
@@ -846,8 +848,7 @@ clear_player(thing)
 }
 
 static void
-clear_room(thing)
-    dbref thing;
+clear_room(dbref thing)
 {
   dbref first, rest;
   /* give player money back */
@@ -872,8 +873,7 @@ clear_room(thing)
 
 
 static void
-clear_exit(thing)
-    dbref thing;
+clear_exit(dbref thing)
 {
   dbref loc;
   loc = Source(thing);
@@ -884,9 +884,11 @@ clear_exit(thing)
 }
 
 
-/* return a cleaned up object off the free list or NOTHING */
+/** Return a cleaned up object off the free list or NOTHING.
+ * \return a garbage object or NOTHING.
+ */
 dbref
-free_get()
+free_get(void)
 {
   dbref newobj;
   if (first_free == NOTHING)
@@ -916,14 +918,14 @@ free_get()
   return (newobj);
 }
 
-/* Build the free list with a sledgehammer. Only do this when it's actually
- * necessary.
+/** Build the free list with a sledgehammer. 
+ * Only do this when it's actually necessary.
  * Since we only do it if things are corrupted, we do not free any memory.
  * Presumably, this will only waste a reasonable amount of memory, since
  * it's only called in exceptional cases.
  */
 void
-fix_free_list()
+fix_free_list(void)
 {
   dbref thing;
   first_free = NOTHING;
@@ -937,9 +939,9 @@ fix_free_list()
 
 
 
-/* Destroy all the objects we said we would destroy later. */
+/** Destroy all the objects we said we would destroy later. */
 void
-purge()
+purge(void)
 {
   dbref thing;
   for (thing = 0; thing < db_top; thing++) {
@@ -949,7 +951,7 @@ purge()
       if (Going_Twice(thing)) {
 	free_object(thing);
       } else {
-	Flags(thing) |= GOING_TWICE;
+	set_flag_internal(thing, "GOING_TWICE");
       }
     } else {
       continue;
@@ -958,10 +960,14 @@ purge()
 }
 
 
-/* Destroy objects slated for destruction. */
+/** Destroy objects slated for destruction.
+ * \verbatim
+ * This is the top-level function for @purge.
+ * \endverbatim
+ * \param player the enactor.
+ */
 void
-do_purge(player)
-    dbref player;
+do_purge(dbref player)
 {
   if (Wizard(player)) {
     purge();
@@ -974,9 +980,10 @@ do_purge(player)
 
 /* Section III: dbck() and related functions. */
 
-/* The regular db checkup. */
+/** The complete db checkup.
+ */
 void
-dbck()
+dbck(void)
 {
   check_fields();
   check_contents();
@@ -988,7 +995,7 @@ dbck()
 
 /* Do sanity checks on non-destroyed objects. */
 static void
-check_fields()
+check_fields(void)
 {
   dbref thing;
   for (thing = 0; thing < db_top; thing++) {
@@ -1036,7 +1043,7 @@ check_fields()
       case TYPE_PLAYER:
       case TYPE_THING:
 	if (!GoodObject(home) || IsGarbage(home) || IsExit(home))
-	  Home(thing) = PLAYER_START;
+	  Home(thing) = DEFAULT_HOME;
 	if (!GoodObject(loc) || IsGarbage(loc) || IsExit(loc))
 	  enter_room(thing, Home(thing), 0);
 	break;
@@ -1097,18 +1104,18 @@ check_fields()
        * an invalid dbref, change its ownership to God.
        */
       if (!IsGarbage(thing))
-	atr_iter_get(GOD, thing, NULL, attribute_owner_helper, NULL);
+	atr_iter_get(GOD, thing, "**", 0, attribute_owner_helper, NULL);
     }
   }
 }
 
 static int
-attribute_owner_helper(player, thing, pattern, atr, args)
-    dbref player __attribute__ ((__unused__));
-    dbref thing __attribute__ ((__unused__));
-    char const *pattern __attribute__ ((__unused__));
-    ATTR *atr;
-    void *args __attribute__ ((__unused__));
+attribute_owner_helper(dbref player __attribute__ ((__unused__)),
+		       dbref thing __attribute__ ((__unused__)),
+		       dbref parent __attribute__ ((__unused__)),
+		       char const *pattern
+		       __attribute__ ((__unused__)), ATTR *atr, void *args
+		       __attribute__ ((__unused__)))
 {
   if (!GoodObject(AL_CREATOR(atr)))
     AL_CREATOR(atr) = GOD;
@@ -1116,39 +1123,38 @@ attribute_owner_helper(player, thing, pattern, atr, args)
 }
 
 static void
-check_connected_rooms()
+check_connected_rooms(void)
 {
   mark_connected(BASE_ROOM);
   check_connected_marks();
 }
 
 static void
-mark_connected(loc)
-    dbref loc;
+mark_connected(dbref loc)
 {
   dbref thing;
-  if (!GoodObject(loc) || Marked(loc) || !IsRoom(loc))
+  if (!GoodObject(loc) || !IsRoom(loc) || Marked(loc))
     return;
-  Flags(loc) |= MARKED;
+  SetMarked(loc);
   /* recursively trace */
   for (thing = Exits(loc); thing != NOTHING; thing = Next(thing))
     mark_connected(Destination(thing));
 }
 
 static void
-check_connected_marks()
+check_connected_marks(void)
 {
   dbref loc;
   for (loc = 0; loc < db_top; loc++)
-    if (Marked(loc))
-      Flags(loc) &= ~MARKED;
+    if (!IsGarbage(loc) && Marked(loc))
+      ClearMarked(loc);
     else if (IsRoom(loc)) {
       if (!Name(loc)) {
 	do_log(LT_ERR, NOTHING, NOTHING, T("ERROR: no name for room #%d."),
 	       loc);
 	set_name(loc, "XXXX");
       }
-      if (!Going(loc) && !Floating(loc) &&
+      if (!Going(loc) && !Floating(loc) && !NoWarnable(loc) &&
 	  (!EXITS_CONNECT_ROOMS || (Exits(loc) == NOTHING))) {
 	notify_format(Owner(loc), T("You own a disconnected room, %s"),
 		      object_header(Owner(loc), loc));
@@ -1160,7 +1166,8 @@ check_connected_marks()
 static void
 check_zones(void)
 {
-  dbref n, zone, tmp;
+  dbref n, zone = NOTHING, tmp;
+  int zone_depth;
 
   for (n = 0; n < db_top; n++) {
     if (IsGarbage(n))
@@ -1171,7 +1178,8 @@ check_zones(void)
     if (ZONE_CONTROL_ZMP && !IsPlayer(zone))
       continue;
     if (zone != n)		/* Objects can be zoned to themselves */
-      for (tmp = Zone(zone); GoodObject(tmp); tmp = Zone(tmp)) {
+      for (zone_depth = MAX_ZONES, tmp = Zone(zone);
+	   zone_depth-- && GoodObject(tmp); tmp = Zone(tmp)) {
 	if (tmp == n) {
 	  notify_format(Owner(n),
 			T("You own an object in a circular zone chain: %s"),
@@ -1185,12 +1193,12 @@ check_zones(void)
     if (Marked(zone))
       continue;
     if (getlock(zone, Zone_Lock) == TRUE_BOOLEXP)
-      Flags(zone) |= MARKED;
+      SetMarked(zone);
   }
 
   for (n = 0; n < db_top; n++) {
-    if (Marked(n)) {
-      Flags(n) &= ~MARKED;
+    if (!IsGarbage(n) && Marked(n)) {
+      ClearMarked(n);
       notify_format(Owner(n),
 		    T
 		    ("You own an object without a @lock/zone being used as a zone: %s"),
@@ -1199,14 +1207,10 @@ check_zones(void)
   }
 }
 
-/* An auxiliary function for check_contents. */
-static void
-mark_contents(thing)
-    dbref thing;
-{
-  /* In this next macro, field must be an lvalue whose evaluation has
-   * no side effects.  All hell will break loose if this is not so.
-   */
+/** In this macro, field must be an lvalue whose evaluation has
+ * no side effects and results in a dbref to be checked.
+ * All hell will break loose if this is not so.
+ */
 #define CHECK(field)            \
   if ((field) != NOTHING) { \
      if (!GoodObject(field) || IsGarbage(field)) { \
@@ -1222,15 +1226,19 @@ mark_contents(thing)
                  unparse_object(GOD, (field)), thing); \
        (field) = NOTHING; \
      } else { \
-       Flags(field) |= MARKED; \
+       SetMarked(field); \
        mark_contents(field); \
      } \
   }
 
-  if (!GoodObject(thing))
+/* An auxiliary function for check_contents. */
+static void
+mark_contents(dbref thing)
+{
+  if (!GoodObject(thing) || IsGarbage(thing))
     return;
 
-  Flags(thing) |= MARKED;
+  SetMarked(thing);
   switch (Typeof(thing)) {
   case TYPE_ROOM:
     CHECK(Exits(thing));
@@ -1249,15 +1257,16 @@ mark_contents(thing)
 	      unparse_object(GOD, thing));
     break;
   }
-#undef CHECK
 }
 
+#undef CHECK
+
 /* Check that for every thing, player, and exit, you can trace exactly one
  * path to that object from a room by following the exits field of rooms,
  * the next field of non-rooms, and the contents field of non-exits.
  */
 static void
-check_contents()
+check_contents(void)
 {
   dbref thing;
   for (thing = 0; thing < db_top; thing++) {
@@ -1266,14 +1275,14 @@ check_contents()
     }
   }
   for (thing = 0; thing < db_top; thing++) {
-    if (!Marked(thing) && !IsRoom(thing) && !IsGarbage(thing)) {
+    if (!IsRoom(thing) && !IsGarbage(thing) && !Marked(thing)) {
       do_rawlog(LT_ERR, T("Object %s not pointed to by anything."),
 		unparse_object(GOD, thing));
       notify_format(Owner(thing),
 		    T("You own an object %s that was \'orphaned\'."),
 		    object_header(Owner(thing), thing));
       /* We try to fix this by trying to send players and things to
-       * their current location, to their home, or to PLAYER_START, in
+       * their current location, to their home, or to DEFAULT_HOME, in
        * that order, and relinking exits to their source.
        */
       Next(thing) = NOTHING;
@@ -1292,8 +1301,8 @@ check_contents()
 	} else {
 	  Contents(Location(thing)) =
 	    remove_first(Contents(Location(thing)), thing);
-	  PUSH(thing, Contents(PLAYER_START));
-	  Location(thing) = PLAYER_START;
+	  PUSH(thing, Contents(DEFAULT_HOME));
+	  Location(thing) = DEFAULT_HOME;
 	}
 	enter_room(thing, Location(thing), 0);
 	/* If we've managed to reconnect it, then we've reconnected
@@ -1331,7 +1340,8 @@ check_contents()
     }
   }
   for (thing = 0; thing < db_top; thing++) {
-    Flags(thing) &= ~MARKED;
+    if (!IsGarbage(thing))
+      ClearMarked(thing);
   }
 }
 
@@ -1340,7 +1350,7 @@ check_contents()
  * location, and that every exit occurs in the exit list of its source.
  */
 static void
-check_locations()
+check_locations(void)
 {
   dbref thing;
   dbref loc;
@@ -1366,7 +1376,7 @@ check_locations()
 		    unparse_object(GOD, thing), loc);
 	  Location(thing) = loc;
 	}
-	Flags(thing) |= MARKED;
+	SetMarked(thing);
       }
     }
     if (IsRoom(loc)) {
@@ -1390,23 +1400,24 @@ check_locations()
 
 
   for (thing = 0; thing < db_top; thing++)
-    if (Marked(thing))
-      Flags(thing) &= ~MARKED;
+    if (!IsGarbage(thing) && Marked(thing))
+      ClearMarked(thing);
     else if (Mobile(thing)) {
       do_rawlog(LT_ERR, T("ERROR DBCK: Moved object %d"), thing);
-      moveto(thing, PLAYER_START);
+      moveto(thing, DEFAULT_HOME);
     }
 }
 
 
-/* Do database checkup. */
-/* N.B. Unlike before, this is only a user command. The real work is
- * farmed out to the dbck() routine. When a dbck is done automatically,
- * call dbck(), not do_dbck()!
+/** Database checkup, user interface.
+ * \verbatim
+ * This is the top-level function for @dbck. Automatic checks should
+ * call dbck(), not this.
+ * \endverbatim
+ * \param player the enactor.
  */
 void
-do_dbck(player)
-    dbref player;
+do_dbck(dbref player)
 {
   if (!Wizard(player)) {
     notify(player, T("Silly mortal chicks are for kids!"));
