@@ -2099,6 +2099,12 @@ list_cflags(CHAN *c)
     safe_chr('H', tbuf1, &bp);
   if (Channel_Open(c))
     safe_chr('o', tbuf1, &bp);
+  if (Channel_NoTitles(c))
+    safe_chr('T', tbuf1, &bp);
+  if (Channel_NoNames(c))
+    safe_chr('N', tbuf1, &bp);
+  if (Channel_NoCemit(c))
+    safe_chr('C', tbuf1, &bp);
   *bp = '\0';
   return tbuf1;
 }
