@@ -1,4 +1,10 @@
-/* Functions for encrypt/decrypt, and password checking */
+/**
+ * \file funcrypt.c
+ *
+ * \brief Functions for cryptographic stuff in softcode
+ *
+ *
+ */
 #include "copyrite.h"
 
 #include "config.h"
@@ -20,12 +26,23 @@
 #include "attrib.h"
 #include "ansi.h"
 #include "match.h"
+#ifdef HAS_OPENSSL
+#include <openssl/sha.h>
+#include <openssl/evp.h>
+#else
 #include "shs.h"
+#endif
 #include "confmagic.h"
 
 
-static char *crunch_code _((char *code));
-static char *crypt_code _((char *code, char *text, int type));
+static char *crunch_code(char *code);
+static char *crypt_code(char *code, char *text, int type);
+
+#ifdef HAS_OPENSSL
+static void safe_hexchar(unsigned char c, char *buff, char **bp);
+#endif
+
+static void safe_sha0(const char *text, size_t len, char *buff, char **bp);
 
 /* Copy over only alphanumeric chars */
 static char *
@@ -91,6 +108,29 @@ crypt_code(char *code, char *text, int type)
   return textbuff;
 }
 
+static void
+safe_sha0(const char *text, size_t len, char *buff, char **bp)
+{
+#ifdef HAS_OPENSSL
+  unsigned char hash[SHA_DIGEST_LENGTH];
+  int n;
+
+  SHA(text, len, hash);
+
+  for (n = 0; n < SHA_DIGEST_LENGTH; n++)
+    safe_hexchar(hash[n], buff, bp);
+#else
+  SHS_INFO shsInfo;
+  shsInfo.reverse_wanted = (BYTE) options.reverse_shs;
+  shsInit(&shsInfo);
+  shsUpdate(&shsInfo, (const BYTE *) text, len);
+  shsFinal(&shsInfo);
+  safe_format(buff, bp, "%0lx%0lx%0lx%0lx%0lx", shsInfo.digest[0],
+	      shsInfo.digest[1], shsInfo.digest[2], shsInfo.digest[3],
+	      shsInfo.digest[4]);
+#endif
+}
+
 FUNCTION(fun_encrypt)
 {
   safe_str(crypt_code(args[1], args[0], 1), buff, bp);
@@ -111,14 +151,53 @@ FUNCTION(fun_checkpass)
   safe_boolean(password_check(it, args[1]), buff, bp);
 }
 
-FUNCTION(fun_sha1)
+FUNCTION(fun_sha0)
 {
-  SHS_INFO shsInfo;
-  shsInfo.reverse_wanted = (BYTE) options.reverse_shs;
-  shsInit(&shsInfo);
-  shsUpdate(&shsInfo, (const BYTE *) args[0], arglens[0]);
-  shsFinal(&shsInfo);
-  safe_format(buff, bp, "%0lx%0lx%0lx%0lx%0lx", shsInfo.digest[0],
-	      shsInfo.digest[1], shsInfo.digest[2], shsInfo.digest[3],
-	      shsInfo.digest[4]);
+  safe_sha0(args[0], arglens[0], buff, bp);
 }
+
+FUNCTION(fun_digest)
+{
+#ifdef HAS_OPENSSL
+  EVP_MD_CTX ctx;
+  const EVP_MD *mp;
+  unsigned char md[EVP_MAX_MD_SIZE];
+  unsigned int n, len = 0;
+
+  if ((mp = EVP_get_digestbyname(args[0])) == NULL) {
+    safe_str(T("#-1 UNSUPPORTED DIGEST TYPE"), buff, bp);
+    return;
+  }
+
+  EVP_DigestInit(&ctx, mp);
+  EVP_DigestUpdate(&ctx, args[1], arglens[1]);
+  EVP_DigestFinal(&ctx, md, &len);
+
+  for (n = 0; n < len; n++) {
+    safe_hexchar(md[n], buff, bp);
+  }
+
+#else
+  if (strcmp(args[0], "sha") == 0) {
+    safe_sha0(args[1], arglens[1], buff, bp);
+  } else {
+    safe_str(T("#-1 UNSUPPORTED DIGEST TYPE"), buff, bp);
+  }
+#endif
+}
+
+#ifdef HAS_OPENSSL
+static void
+safe_hexchar(unsigned char c, char *buff, char **bp)
+{
+  const char *digits = "0123456789abcdef";
+  if (*bp - buff < BUFFER_LEN - 1) {
+    **bp = digits[c >> 4];
+    (*bp)++;
+  }
+  if (*bp - buff < BUFFER_LEN - 1) {
+    **bp = digits[c & 0x0F];
+    (*bp)++;
+  }
+}
+#endif
