@@ -753,8 +753,8 @@ FUNCTION(fun_con)
 {
   dbref loc = match_thing(executor, args[0]);
   safe_dbref(dbwalk
-	     (NULL, NULL, executor, enactor, TYPE_THING, loc, NOTHING, 0, 0, 0,
-	      NULL), buff, bp);
+	     (NULL, NULL, executor, enactor, TYPE_THING | TYPE_PLAYER, loc,
+	      NOTHING, 0, 0, 0, NULL), buff, bp);
 }
 
 /* ARGSUSED */
@@ -781,8 +781,8 @@ FUNCTION(fun_next)
     case TYPE_THING:
     case TYPE_PLAYER:
       safe_dbref(dbwalk
-		 (NULL, NULL, executor, enactor, TYPE_THING, Location(it), it,
-		  0, 0, 0, NULL), buff, bp);
+		 (NULL, NULL, executor, enactor, TYPE_THING | TYPE_PLAYER,
+		  Location(it), it, 0, 0, 0, NULL), buff, bp);
       break;
     default:
       safe_str("#-1", buff, bp);
@@ -1227,14 +1227,17 @@ FUNCTION(fun_lset)
 FUNCTION(fun_lock)
 {
   dbref it;
-  char *p;
-  lock_type ltype;
+  char *ltype = NULL;
+  lock_type real_ltype;
 
-  if ((p = strchr(args[0], '/')))
-    *p++ = '\0';
+  if ((ltype = strchr(args[0], '/'))) {
+    *ltype++ = '\0';
+    upcasestr(ltype);
+  }
+
+  real_ltype = get_locktype(ltype);
 
   it = match_thing(executor, args[0]);
-  ltype = get_locktype(p);
 
   if (nargs == 2) {
     if (!command_check_byname(executor, "@lock") || fun->flags & FN_NOSIDEFX) {
@@ -1248,9 +1251,10 @@ FUNCTION(fun_lock)
       return;
     }
   }
-  if (GoodObject(it) && (ltype != NULL)
-      && Can_Read_Lock(executor, it, ltype)) {
-    safe_str(unparse_boolexp(executor, getlock(it, ltype), UB_DBREF), buff, bp);
+  if (GoodObject(it) && (real_ltype != NULL)
+      && Can_Read_Lock(executor, it, real_ltype)) {
+    safe_str(unparse_boolexp(executor, getlock(it, real_ltype), UB_DBREF),
+	     buff, bp);
     return;
   }
   safe_str("#-1", buff, bp);
@@ -1327,7 +1331,7 @@ FUNCTION(fun_ctime)
 {
   dbref it = match_thing(executor, args[0]);
 
-  if (GoodObject(it))
+  if (GoodObject(it) && !IsGarbage(it))
     safe_str(show_time(CreTime(it), 0), buff, bp);
   else
     safe_str(T(e_notvis), buff, bp);
@@ -1337,7 +1341,7 @@ FUNCTION(fun_ctime)
 FUNCTION(fun_mtime)
 {
   dbref it = match_thing(executor, args[0]);
-  if (!GoodObject(it))
+  if (!GoodObject(it) || IsGarbage(it))
     safe_str(T(e_notvis), buff, bp);
   else if (!Can_Examine(executor, it) || IsPlayer(it))
     safe_str(T(e_perm), buff, bp);
