@@ -10,6 +10,7 @@
 #include "config.h"
 #include <string.h>
 #include <ctype.h>
+#include "ansi.h"
 #include "conf.h"
 #include "case.h"
 #include "externs.h"
@@ -33,7 +34,7 @@ static char *autodetect_list(char **ptrs, int nptrs);
 static char *get_list_type(char **args, int nargs,
 			   int type_pos, char **ptrs, int nptrs);
 static char *get_list_type_noauto(char **args, int nargs, int type_pos);
-int i_comp(const void *s1, const void *s2);
+static int i_comp(const void *s1, const void *s2);
 static int f_comp(const void *s1, const void *s2);
 static int u_comp(const void *s1, const void *s2);
 static int regrep_helper(dbref who, dbref what, dbref parent,
@@ -767,7 +768,7 @@ si_comp(const void *s1, const void *s2)
   return Compare(res, sr1, sr2);
 }
 
-int
+static int
 i_comp(const void *s1, const void *s2)
 {
   const s_rec *sr1 = (const s_rec *) s1;
@@ -789,7 +790,13 @@ f_comp(const void *s1, const void *s2)
 
 GENRECORD(gen_alphanum)
 {
-  rec->str = rec->val;
+  int len;
+  if (strchr(rec->val, ESC_CHAR)) {
+    rec->str = mush_strdup(remove_markup(rec->val, &len), "genrecord");
+    rec->freestr = 1;
+  } else {
+    rec->str = rec->val;
+  }
 }
 
 GENRECORD(gen_dbref)
