@@ -1,3 +1,10 @@
+/**
+ * \file funmath.c
+ *
+ * \brief Mathematical functions for mushcode.
+ * 
+ *
+ */
 
 #include "copyrite.h"
 
@@ -17,26 +24,36 @@
 #endif
 
 #ifndef M_PI
+/** The ratio of the circumference of a circle to its denominator. */
 #define M_PI 3.14159265358979323846264338327
 #endif
 
 static void do_spellnum(char *num, unsigned int len, char **buff, char ***bp);
+static int nval_sort(const void *, const void *);
+static NVAL find_median(NVAL *, int);
 
+/** Declaration macro for math functions */
 #define MATH_FUNC(func) static void func(char **ptr, int nptr, char *buff, char **bp)
 
+/** Prototype macro for math functions */
 #define MATH_PROTO(func) static void func (char **ptr, int nptr, char *buff, char **bp)
 
-HASHTAB htab_math;
+HASHTAB htab_math;   /**< Math function hash table */
 
+/** A math function. */
 typedef struct {
-  const char *name;
-  void (*func) (char **, int, char *, char **);
+  const char *name;	/**< Name of the function. */
+  void (*func) (char **, int, char *, char **);	/**< Pointer to function code. */
 } MATH;
 
+static void math_hash_insert(const char *, MATH *);
 static MATH *math_hash_lookup(char *);
+static NVAL angle_to_rad(NVAL angle, const char *from);
+static NVAL rad_to_angle(NVAL angle, const char *to);
+static double frac(double v, int *RESTRICT n, int *RESTRICT d, double error);
+void init_math_hashtab(void);
 
-NVAL angle_to_rad(NVAL angle, const char *from);
-NVAL rad_to_angle(NVAL angle, const char *to);
+extern int format_long(long n, char *buff, char **bp, int maxlen, int base);
 
 MATH_PROTO(math_add);
 MATH_PROTO(math_sub);
@@ -242,9 +259,14 @@ FUNCTION(fun_inc)
     }
     p--;
   }
-  /* p now points to the last non-numeric character in the string
-   * Move it to the first numeric character
-   */
+  /* p now points to the last non-numeric character in the string */
+  if (p == args[0] && (isdigit((unsigned char) *p) || (*p == '-'))) {
+    /* Special case - it's all digits, but out of range. */
+    safe_str(T(e_range), buff, bp);
+    return;
+  }
+
+  /* Move it to the first numeric character */
   p++;
   num = parse_integer(p) + 1;
   *p = '\0';
@@ -284,9 +306,13 @@ FUNCTION(fun_dec)
     }
     p--;
   }
-  /* p now points to the last non-numeric character in the string
-   * Move it to the first numeric character
-   */
+  /* p now points to the last non-numeric character in the string */
+  if (p == args[0] && (isdigit((unsigned char) *p) || (*p == '-'))) {
+    /* Special case - it's all digits, but out of range. */
+    safe_str(T(e_range), buff, bp);
+    return;
+  }
+  /* Move it to the first numeric character */
   p++;
   num = parse_integer(p) - 1;
   *p = '\0';
@@ -727,6 +753,39 @@ FUNCTION(fun_vunit)
   }
 }
 
+FUNCTION(fun_vcross)
+{
+  char sep = ' ';
+  char *v1[BUFFER_LEN / 2], *v2[BUFFER_LEN / 2];
+  int v1len, v2len, n;
+  NVAL vec1[3], vec2[3], cross[3];
+
+  if (!delim_check(buff, bp, nargs, args, 3, &sep))
+    return;
+
+  v1len = list2arr(v1, BUFFER_LEN / 2, args[0], sep);
+  v2len = list2arr(v2, BUFFER_LEN / 2, args[1], sep);
+
+  if (v1len != 3 || v2len != 3) {
+    safe_str(T("#-1 VECTORS MUST BE THREE-DIMENSIONAL"), buff, bp);
+    return;
+  }
+
+  for (n = 0; n < 3; n++) {
+    vec1[n] = parse_number(v1[n]);
+    vec2[n] = parse_number(v2[n]);
+  }
+
+  cross[0] = vec1[1] * vec2[2] - vec2[1] * vec1[2];
+  cross[1] = vec1[2] * vec2[0] - vec2[2] * vec1[0];
+  cross[2] = vec1[0] * vec2[1] - vec2[0] * vec1[1];
+
+  safe_number(cross[0], buff, bp);
+  safe_chr(sep, buff, bp);
+  safe_number(cross[1], buff, bp);
+  safe_chr(sep, buff, bp);
+  safe_number(cross[2], buff, bp);
+}
 
 /* ARGSUSED */
 FUNCTION(fun_fdiv)
@@ -1015,12 +1074,174 @@ FUNCTION(fun_sqrt)
   }
   num = parse_number(args[0]);
   if (num < 0) {
-    safe_str(T("#-1 SQUARE ROOT OF NEGATIVE"), buff, bp);
+    safe_str(T("#-1 IMAGINARY NUMBER"), buff, bp);
     return;
   }
   safe_number(sqrt(num), buff, bp);
 }
 
+FUNCTION(fun_root)
+{
+  NVAL root, x;
+  int n;
+  int sign = 0;
+
+  if (!is_number(args[0])) {
+    safe_str(T(e_num), buff, bp);
+    return;
+  }
+  if (!is_integer(args[1])) {
+    safe_str(T(e_int), buff, bp);
+    return;
+  }
+  x = parse_number(args[0]);
+  n = parse_integer(args[1]);
+
+  if (n < 0) {
+    safe_str(T("#-1 ROOT OUT OF RANGE"), buff, bp);
+    return;
+  }
+
+  if (x < 0) {
+    if (n & 1) {		/* Odd root */
+      sign = 1;
+      x = fabs(x);
+    } else {			/* Even */
+      safe_str(T("#-1 IMAGINARY NUMBER"), buff, bp);
+      return;
+    }
+  }
+
+  switch (n) {
+  case 0:
+    root = 0;
+    break;
+  case 1:
+    root = x;
+    break;
+  case 2:
+    root = sqrt(x);
+    break;
+  case 3:
+    /* TO-DO: Configure check for cbrt()? */
+  default:
+    /* Spiffy logarithm trick */
+    root = exp(log(x) / n);
+  }
+
+  if (sign)
+    root = -root;
+
+  safe_number(root, buff, bp);
+}
+
+
+/** Calculates the numerator and denominator for a fraction representing
+ *  a floating point number. Only works for positive numbers!
+ * \param v the number 
+ * \param n pointer to the numerator
+ * \param d pointer to the denominator
+ * \param error accuracy to which the fraction should represent the original number
+ * \return -1.0 if (v < MIN || v > MAX || error < 0.0) | (v - n/d) / v | otherwise.
+ */
+static double
+frac(double v, int *RESTRICT n, int *RESTRICT d, double error)
+{
+
+/* Based on a routine found in netlib (http://www.netlib.org) by
+   
+                        Robert J. Craig
+                        AT&T Bell Laboratories
+                        1200 East Naperville Road
+                        Naperville, IL 60566-7045
+
+ though I have no idea if that address is still valid.
+
+ Rewritten by Raevnos to use modern C, and get rid of stupid gotos.
+
+    reference:  Jerome Spanier and Keith B. Oldham, "An Atlas
+    of Functions," Springer-Verlag, 1987, pp. 665-7.
+  */
+
+
+  int D, N, t;
+  int first = 1;
+  double epsilon, r = 0.0, m;
+
+  if (v < 0 || error < 0.0)
+    return -1.0;
+  *d = D = 1;
+  *n = (int) v;
+  N = *n + 1;
+
+  do {
+    if (!first) {
+      if (r <= 1.0)
+	r = 1.0 / r;
+      N += *n * (int) r;
+      D += *d * (int) r;
+      *n += N;
+      *d += D;
+    } else
+      first = 0;
+    r = 0.0;
+    if (v * (*d) != *n) {
+      r = (N - v * D) / (v * (*d) - *n);
+      if (r <= 1.0) {
+	t = N;
+	N = *n;
+	*n = t;
+	t = D;
+	D = *d;
+	*d = t;
+      }
+    }
+    epsilon = fabs(1.0 - *n / (v * (*d)));
+    if (epsilon <= error)
+      return epsilon;
+    m = 1.0;
+    do {
+      m *= 10.0;
+    } while (m * epsilon < 1.0);
+    epsilon = 1.0 / m * ((int) (0.5 + m * epsilon));
+    if (epsilon <= error)
+      return epsilon;
+  } while (r != 0.0);
+  return epsilon;
+}
+
+FUNCTION(fun_fraction)
+{
+  int num = 0, denom = 0;
+  NVAL n;
+  int sign = 0;
+
+  if (!is_number(args[0])) {
+    safe_str(T(e_num), buff, bp);
+    return;
+  }
+
+  n = parse_number(args[0]);
+
+  if (n < 0) {
+    n = fabs(n);
+    sign = 1;
+  } else if (n == 0) {
+    safe_chr('0', buff, bp);
+    return;
+  }
+
+  frac(n, &num, &denom, 1.0e-10);
+
+  if (sign)
+    safe_chr('-', buff, bp);
+
+  if (denom == 1)
+    safe_integer(num, buff, bp);
+  else
+    safe_format(buff, bp, "%d/%d", num, denom);
+}
+
 FUNCTION(fun_isint)
 {
   safe_boolean(is_strict_integer(args[0]), buff, bp);
@@ -1104,13 +1325,14 @@ FUNCTION(fun_xor)
   math_xor(args, nargs, buff, bp);
 }
 
+/** Return the spelled-out version of an integer.
+ * \param num string containing an integer to spell out.
+ * \param len length of num, which must be a multiple of 3, max 15.
+ * \param buff pointer to address of output buffer.
+ * \param bp pointer to pointer to insertion point in buff.
+ */
 static void
-do_spellnum(num, len, buff, bp)
-    char *num;			/* string containing an integer. */
-    unsigned int len;		/* The string length must be a multiple of 3, 
-				   with a maximum of 15. */
-    char **buff;		/* output is the integer spelled out */
-    char ***bp;
+do_spellnum(char *num, unsigned int len, char **buff, char ***bp)
 {
   static const char *bigones[] =
     { "", "thousand", "million", "billion", "trillion" };
@@ -1172,7 +1394,7 @@ do_spellnum(num, len, buff, bp)
   }
 }				/* do_spellnum */
 
-/* adds zeros to the beginning of the string, untill its length is 
+/** adds zeros to the beginning of the string, untill its length is 
  * a multiple of 3.
  */
 #define add_zeros(p) \
@@ -1400,6 +1622,40 @@ FUNCTION(fun_lmath)
   mush_free((Malloc_t) ptr, "string");
 }
 
+FUNCTION(fun_baseconv)
+{
+  long n;
+  int from, to;
+  char *end;
+
+  if (!(is_integer(args[1]) && is_integer(args[2]))) {
+    safe_str(T(e_ints), buff, bp);
+    return;
+  }
+
+  from = parse_integer(args[1]);
+  to = parse_integer(args[2]);
+
+  if (from < 2 || from > 36) {
+    safe_str(T("#-1 FROM BASE OUT OF RANGE"), buff, bp);
+    return;
+  }
+
+  if (to < 2 || to > 36) {
+    safe_str(T("#-1 TO BASE OUT OF RANGE"), buff, bp);
+    return;
+  }
+
+  n = strtol(trim_space_sep(args[0], ' '), &end, from);
+
+  if (*end != '\0') {
+    safe_str(T("#-1 MALFORMED NUMBER"), buff, bp);
+    return;
+  }
+
+  format_long(n, buff, bp, BUFFER_LEN, to);
+}
+
 MATH_FUNC(math_add)
 {
   NVAL result = 0;
@@ -1917,9 +2173,8 @@ MATH_FUNC(math_xor)
     safe_chr('0', buff, bp);
 }
 
-int nval_sort(const void *, const void *);
 
-int
+static int
 nval_sort(const void *a, const void *b)
 {
   const NVAL *x = a, *y = b;
@@ -1928,9 +2183,8 @@ nval_sort(const void *a, const void *b)
   return eq ? 0 : (*x > *y ? 1 : -1);
 }
 
-NVAL find_median(NVAL *, int);
 
-NVAL
+static NVAL
 find_median(NVAL *numbers, int nargs)
 {
   if (nargs == 0)
@@ -2014,6 +2268,7 @@ MATH_FUNC(math_stddev)
   safe_number(sqrt(s / (nptr - 1)), buff, bp);
 }
 
+/** A list of MATH_FUNCs that are suitable for lmath() */
 MATH mlist[] = {
   {"ADD", math_add}
   ,
@@ -2070,17 +2325,16 @@ math_hash_lookup(char *name)
   return (MATH *) hashfind(strupper(name), &htab_math);
 }
 
-void math_hash_insert(const char *, MATH *);
-void init_math_hashtab(void);
 
-void
+static void
 math_hash_insert(const char *name, MATH *func)
 {
   hashadd(name, (void *) func, &htab_math);
 }
 
+/** Initialize the math function hash table. */
 void
-init_math_hashtab()
+init_math_hashtab(void)
 {
   MATH *fp;
 
@@ -2089,7 +2343,7 @@ init_math_hashtab()
     math_hash_insert(fp->name, (MATH *) fp);
 }
 
-NVAL
+static NVAL
 angle_to_rad(NVAL angle, const char *from)
 {
   if (!from)
@@ -2109,7 +2363,7 @@ angle_to_rad(NVAL angle, const char *from)
   }
 }
 
-NVAL
+static NVAL
 rad_to_angle(NVAL angle, const char *to)
 {
   if (!to)
