@@ -1,9 +1,17 @@
+/**
+ * \file funstr.c
+ *
+ * \brief String functions for mushcode.
+ *
+ *
+ */
 #include "copyrite.h"
 
 #include "config.h"
 #include <string.h>
 #include <ctype.h>
 #include <limits.h>
+#include <locale.h>
 #include "conf.h"
 #include "ansi.h"
 #include "externs.h"
@@ -19,50 +27,108 @@
 #include "lock.h"
 #include "confmagic.h"
 
+
 #ifdef WIN32
+#define LC_MESSAGES 6
 #pragma warning( disable : 4761)	/* NJG: disable warning re conversion */
 #endif
 
-HASHTAB htab_tag;
+#ifdef __APPLE__
+#define LC_MESSAGES 6
+#endif
 
+HASHTAB htab_tag;  /**< Hash table of safe html tags */
+
+#define MAX_COLS 32  /**< Maximum number of columns for align() */
 static int wraplen(char *str, int maxlen);
+static int align_one_line(char *buff, char **bp, int ncols,
+			  int cols[MAX_COLS], int calign[MAX_COLS],
+			  char *ptrs[MAX_COLS], ansi_string *as[MAX_COLS],
+			  int linenum, char *fieldsep, int fslen, char *linesep,
+			  int lslen, char filler);
+static int comp_gencomp(dbref executor, char *left, char *right, char *type);
 void init_tag_hashtab(void);
+void init_pronouns(void);
 
+/** Return an indicator of a player's gender.
+ * \param player player whose gender is to be checked.
+ * \retval 0 neuter.
+ * \retval 1 female.
+ * \retval 2 male.
+ * \retval 3 plural.
+ */
 int
 get_gender(dbref player)
 {
-  /* 0 for error, 1 for neuter, 2 for female, 3 for male, 4 for plural */
-
   ATTR *a;
 
   a = atr_get(player, "SEX");
 
   if (!a)
-    return 1;
+    return 0;
 
-  switch (*uncompress(a->value)) {
+  switch (*atr_value(a)) {
   case 'T':
   case 't':
   case 'P':
   case 'p':
-    return 4;
+    return 3;
   case 'M':
   case 'm':
-    return 3;
+    return 2;
   case 'F':
   case 'f':
   case 'W':
   case 'w':
-    return 2;
-  default:
     return 1;
+  default:
+    return 0;
   }
 }
 
-char const *subj[5] = { "", "it", "she", "he", "they" };
-char const *poss[5] = { "", "its", "her", "his", "their" };
-char const *obj[5] = { "", "it", "her", "him", "them" };
-char const *absp[5] = { "", "its", "hers", "his", "theirs" };
+char *subj[4];	/**< Subjective pronouns */
+char *poss[4];	/**< Possessive pronouns */
+char *obj[4];	/**< Objective pronouns */
+char *absp[4];	/**< Absolute possessive pronouns */
+
+/** Macro to set a pronoun entry based on whether we're translating or not */
+#define SET_PRONOUN(p,v,u)  p = strdup((translate) ? (v) : (u))
+
+/** Initialize the pronoun translation strings.
+ * This function sets up the values of the arrays of subjective,
+ * possessive, objective, and absolute possessive pronouns with
+ * locale-appropriate values.
+ */
+void
+init_pronouns(void)
+{
+  int translate = 0;
+#ifdef HAS_SETLOCALE
+  char *loc;
+  if ((loc = setlocale(LC_MESSAGES, NULL))) {
+    if (strcmp(loc, "C") && strncmp(loc, "en", 2))
+      translate = 1;
+  }
+#endif
+  SET_PRONOUN(subj[0], T("pronoun:neuter,subjective"), "it");
+  SET_PRONOUN(subj[1], T("pronoun:feminine,subjective"), "she");
+  SET_PRONOUN(subj[2], T("pronoun:masculine,subjective"), "he");
+  SET_PRONOUN(subj[3], T("pronoun:plural,subjective"), "they");
+  SET_PRONOUN(poss[0], T("pronoun:neuter,possessive"), "its");
+  SET_PRONOUN(poss[1], T("pronoun:feminine,possessive"), "her");
+  SET_PRONOUN(poss[2], T("pronoun:masculine,possessive"), "his");
+  SET_PRONOUN(poss[3], T("pronoun:plural,possessive"), "their");
+  SET_PRONOUN(obj[0], T("pronoun:neuter,objective"), "it");
+  SET_PRONOUN(obj[1], T("pronoun:feminine,objective"), "her");
+  SET_PRONOUN(obj[2], T("pronoun:masculine,objective"), "him");
+  SET_PRONOUN(obj[3], T("pronoun:plural,objective"), "them");
+  SET_PRONOUN(absp[0], T("pronoun:neuter,absolute possessive"), "its");
+  SET_PRONOUN(absp[1], T("pronoun:feminine,absolute possessive"), "hers");
+  SET_PRONOUN(absp[2], T("pronoun:masculine,absolute possessive"), "his");
+  SET_PRONOUN(absp[3], T("pronoun:plural,absolute possessive "), "theirs");
+}
+
+#undef SET_PRONOUN
 
 /* ARGSUSED */
 FUNCTION(fun_isword)
@@ -180,7 +246,7 @@ FUNCTION(fun_alphamax)
   char amax[BUFFER_LEN];
   char *c;
   int j, m = 0;
-  Size_t len;
+  size_t len;
 
   c = remove_markup(args[0], &len);
   memcpy(amax, c, len);
@@ -200,7 +266,7 @@ FUNCTION(fun_alphamin)
   char amin[BUFFER_LEN];
   char *c;
   int j, m = 0;
-  Size_t len;
+  size_t len;
 
   c = remove_markup(args[0], &len);
   memcpy(amin, c, len);
@@ -360,6 +426,49 @@ FUNCTION(fun_delete)
   free_ansi_string(as);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_strreplace)
+{
+  ansi_string *as, *anew;
+  int start, len, end;
+
+  if (!is_integer(args[1]) || !is_integer(args[2])) {
+    safe_str(T(e_ints), buff, bp);
+    return;
+  }
+
+  start = parse_integer(args[1]);
+  len = parse_integer(args[2]);
+
+  if (start < 0 || len < 0) {
+    safe_str(T(e_range), buff, bp);
+    return;
+  }
+
+  as = parse_ansi_string(args[0]);
+  anew = parse_ansi_string(args[3]);
+
+  safe_ansi_string(as, 0, start, buff, bp);
+  safe_ansi_string(anew, 0, anew->len, buff, bp);
+
+  end = start + len;
+
+  if ((size_t) end < as->len)
+    safe_ansi_string(as, end, as->len - end, buff, bp);
+
+  free_ansi_string(as);
+  free_ansi_string(anew);
+
+}
+
+static int
+comp_gencomp(dbref executor, char *left, char *right, char *type)
+{
+  int c;
+  c = gencomp(executor, left, right, type);
+  return (c > 0 ? 1 : (c < 0 ? -1 : 0));
+}
+
 /* ARGSUSED */
 FUNCTION(fun_comp)
 {
@@ -376,23 +485,25 @@ FUNCTION(fun_comp)
   case 'A':			/* Case-sensitive lexicographic */
     {
       char left[BUFFER_LEN], right[BUFFER_LEN], *l, *r;
-      Size_t llen, rlen;
+      size_t llen, rlen;
       l = remove_markup(args[0], &llen);
       memcpy(left, l, llen);
       r = remove_markup(args[1], &rlen);
       memcpy(right, r, rlen);
-      safe_integer(gencomp(left, right, ALPHANUM_LIST), buff, bp);
+      safe_integer(comp_gencomp(executor, left, right, ALPHANUM_LIST), buff,
+		   bp);
       return;
     }
   case 'I':			/* Case-insensitive lexicographic */
     {
       char left[BUFFER_LEN], right[BUFFER_LEN], *l, *r;
-      Size_t llen, rlen;
+      size_t llen, rlen;
       l = remove_markup(args[0], &llen);
       memcpy(left, l, llen);
       r = remove_markup(args[1], &rlen);
       memcpy(right, r, rlen);
-      safe_integer(gencomp(left, right, INSENS_ALPHANUM_LIST), buff, bp);
+      safe_integer(comp_gencomp(executor, left, right, INSENS_ALPHANUM_LIST),
+		   buff, bp);
       return;
     }
   case 'N':			/* Integers */
@@ -400,25 +511,28 @@ FUNCTION(fun_comp)
       safe_str(T(e_ints), buff, bp);
       return;
     }
-    safe_integer(gencomp(args[0], args[1], NUMERIC_LIST), buff, bp);
+    safe_integer(comp_gencomp(executor, args[0], args[1], NUMERIC_LIST), buff,
+		 bp);
     return;
   case 'F':
     if (!is_strict_number(args[0]) || !is_strict_number(args[1])) {
       safe_str(T(e_nums), buff, bp);
       return;
     }
-    safe_integer(gencomp(args[0], args[1], FLOAT_LIST), buff, bp);
+    safe_integer(comp_gencomp(executor, args[0], args[1], FLOAT_LIST), buff,
+		 bp);
     return;
   case 'D':
     {
       dbref a, b;
-      a = parse_dbref(args[0]);
-      b = parse_dbref(args[1]);
+      a = parse_objid(args[0]);
+      b = parse_objid(args[1]);
       if (a == NOTHING || b == NOTHING) {
 	safe_str(T("#-1 INVALID DBREF"), buff, bp);
 	return;
       }
-      safe_integer(gencomp(args[0], args[1], DBREF_LIST), buff, bp);
+      safe_integer(comp_gencomp(executor, args[0], args[1], DBREF_LIST), buff,
+		   bp);
       return;
     }
   default:
@@ -432,7 +546,7 @@ FUNCTION(fun_pos)
 {
   char tbuf[BUFFER_LEN];
   char *pos;
-  Size_t len;
+  size_t len;
 
   pos = remove_markup(args[1], &len);
   memcpy(tbuf, pos, len);
@@ -443,12 +557,35 @@ FUNCTION(fun_pos)
     safe_str("#-1", buff, bp);
 }
 
+/* ARGSUSED */
+FUNCTION(fun_lpos)
+{
+  char *pos;
+  char c = ' ';
+  size_t n, len;
+  int first = 1;
+
+  if (args[1][0])
+    c = args[1][0];
+
+  pos = remove_markup(args[0], &len);
+  for (n = 0; n < len; n++)
+    if (pos[n] == c) {
+      if (first)
+	first = 0;
+      else
+	safe_chr(' ', buff, bp);
+      safe_integer(n, buff, bp);
+    }
+}
+
+
 /* ARGSUSED */
 FUNCTION(fun_strmatch)
 {
   char tbuf[BUFFER_LEN];
   char *t;
-  Size_t len;
+  size_t len;
   /* matches a wildcard pattern for an _entire_ string */
 
   t = remove_markup(args[0], &len);
@@ -494,33 +631,131 @@ FUNCTION(fun_flip)
 /* ARGSUSED */
 FUNCTION(fun_merge)
 {
-  /* given s1, s2, and a char, for each character in s1, if the char
-   * is the same as the given char, replace it with the corresponding
+  /* given s1, s2, and a list of characters, for each character in s1,
+   * if the char is in the list, replace it with the corresponding
    * char in s2.
    */
 
   char *str, *rep;
-  char c;
+  char matched[UCHAR_MAX + 1];
 
   /* do length checks first */
   if (arglens[0] != arglens[1]) {
     safe_str(T("#-1 STRING LENGTHS MUST BE EQUAL"), buff, bp);
     return;
   }
-  if (arglens[2] > 1) {
-    safe_str(T("#-1 TOO MANY CHARACTERS"), buff, bp);
-    return;
+
+  memset(matched, 0, sizeof matched);
+
+  /* find the characters to look for */
+  if (!args[2] || !*args[2])
+    matched[(unsigned char) ' '] = 1;
+  else {
+    unsigned char *p;
+    for (p = (unsigned char *) args[2]; p && *p; p++)
+      matched[*p] = 1;
   }
-  /* find the character to look for */
-  if (!*args[2])
-    c = ' ';
-  else
-    c = *args[2];
 
   /* walk strings, copy from the appropriate string */
   for (str = args[0], rep = args[1]; *str && *rep; str++, rep++) {
-    safe_chr((*str == c) ? *rep : *str, buff, bp);
+    *str = matched[(unsigned char) *str] ? *rep : *str;
   }
+  safe_str(args[0], buff, bp);
+}
+
+/* ARGSUSED */
+FUNCTION(fun_tr)
+{
+  /* given str, s1, s2, for each character in str, if the char
+   * is in s1, replace it with the char at the same index in s2.
+   */
+
+  char charmap[256];
+  char instr[BUFFER_LEN], outstr[BUFFER_LEN];
+  char rawstr[BUFFER_LEN];
+  char *ip, *op;
+  size_t i, len;
+  char *c;
+  ansi_string *as;
+
+  /* No ansi allowed in find or replace lists */
+  c = remove_markup(args[1], &len);
+  memcpy(rawstr, c, len);
+
+  /* do length checks first */
+
+  for (i = 0; i < 256; i++) {
+    charmap[i] = (char) i;
+  }
+
+  ip = instr;
+  op = outstr;
+
+  for (i = 0; i < len; i++) {
+    safe_chr(rawstr[i], instr, &ip);
+    /* Handle a range of characters */
+    if (i != len - 1 && rawstr[i + 1] == '-' && i != len - 2) {
+      int dir, sentinel, cur;
+
+      if (rawstr[i] < rawstr[i + 2])
+	dir = 1;
+      else
+	dir = -1;
+
+      sentinel = rawstr[i + 2] + dir;
+      cur = rawstr[i] + dir;
+
+      while (cur != sentinel) {
+	safe_chr((char) cur, instr, &ip);
+	cur += dir;
+      }
+      i += 2;
+    }
+  }
+
+  c = remove_markup(args[2], &len);
+  memcpy(rawstr, c, len);
+  for (i = 0; i < len; i++) {
+    safe_chr(rawstr[i], outstr, &op);
+    /* Handle a range of characters */
+    if (i != len - 1 && rawstr[i + 1] == '-' && i != len - 2) {
+      int dir, sentinel, cur;
+
+      if (rawstr[i] < rawstr[i + 2])
+	dir = 1;
+      else
+	dir = -1;
+
+      sentinel = rawstr[i + 2] + dir;
+      cur = rawstr[i] + dir;
+
+      while (cur != sentinel) {
+	safe_chr((char) cur, outstr, &op);
+	cur += dir;
+      }
+      i += 2;
+    }
+  }
+
+  if ((ip - instr) != (op - outstr)) {
+    safe_str(T("#-1 STRING LENGTHS MUST BE EQUAL"), buff, bp);
+    return;
+  }
+
+  len = ip - instr;
+
+  for (i = 0; i < len; i++)
+    charmap[(unsigned char) instr[i]] = outstr[i];
+
+  /* walk the string, translating characters */
+  as = parse_ansi_string(args[0]);
+  populate_codes(as);
+  len = as->len;
+  for (i = 0; i < len; i++) {
+    as->text[i] = charmap[(unsigned char) as->text[i]];
+  }
+  safe_ansi_string(as, 0, as->len, buff, bp);
+  free_ansi_string(as);
 }
 
 /* ARGSUSED */
@@ -746,7 +981,7 @@ FUNCTION(fun_foreach)
   char *oldbp;
   char start, end;
   char letters[BUFFER_LEN];
-  Size_t len;
+  size_t len;
 
   if (nargs >= 3) {
     if (!delim_check(buff, bp, nargs, args, 3, &start))
@@ -761,17 +996,18 @@ FUNCTION(fun_foreach)
   }
 
   /* find our object and attribute */
-  parse_attrib(executor, args[0], &thing, &attrib);
-  if (!GoodObject(thing) || !attrib || !Can_Read_Attr(executor, thing, attrib))
+  parse_anon_attrib(executor, args[0], &thing, &attrib);
+  if (!GoodObject(thing) || !attrib || !Can_Read_Attr(executor, thing, attrib)) {
+    free_anon_attrib(attrib);
     return;
-
+  }
   strcpy(place, "0");
-  asave = safe_uncompress(attrib->value);
+  asave = safe_atr_value(attrib);
 
   /* save our stack */
-  tptr[0] = wenv[0];
-  tptr[1] = wenv[1];
-  wenv[1] = place;
+  tptr[0] = global_eval_context.wenv[0];
+  tptr[1] = global_eval_context.wenv[1];
+  global_eval_context.wenv[1] = place;
 
   ap = remove_markup(args[1], &len);
   memcpy(letters, ap, len);
@@ -783,7 +1019,8 @@ FUNCTION(fun_foreach)
     if (!tmp) {
       safe_str(lp, buff, bp);
       free((Malloc_t) asave);
-      wenv[1] = tptr[1];
+      free_anon_attrib(attrib);
+      global_eval_context.wenv[1] = tptr[1];
       return;
     }
     oldbp = place;
@@ -797,7 +1034,7 @@ FUNCTION(fun_foreach)
   }
 
   cbuf[1] = '\0';
-  wenv[0] = cbuf;
+  global_eval_context.wenv[0] = cbuf;
   oldbp = *bp;
   funccount = pe_info->fun_invocations;
   while (*lp && *lp != end) {
@@ -817,66 +1054,41 @@ FUNCTION(fun_foreach)
   if (*lp)
     safe_str(lp + 1, buff, bp);
   free((Malloc_t) asave);
-  wenv[0] = tptr[0];
-  wenv[1] = tptr[1];
+  free_anon_attrib(attrib);
+  global_eval_context.wenv[0] = tptr[0];
+  global_eval_context.wenv[1] = tptr[1];
 }
 
+extern char escaped_chars[UCHAR_MAX + 1];
+
 /* ARGSUSED */
 FUNCTION(fun_secure)
 {
   /* this function smashes all occurences of "unsafe" characters in a string.
-   * "unsafe" characters are ( ) [ ] { } $ ^ % , ; \
+   * "unsafe" characters are defined by the escaped_chars table.
    * these characters get replaced by spaces
    */
-  char *p;
+  unsigned char *p;
 
-  p = args[0];
-  while (*p) {
-    switch (*p) {
-    case '(':
-    case ')':
-    case '[':
-    case ']':
-    case '{':
-    case '}':
-    case '$':
-    case '^':
-    case '%':
-    case ',':
-    case ';':
-    case '\\':
+  for (p = (unsigned char *) args[0]; *p; p++)
+    if (escaped_chars[*p])
       *p = ' ';
-      break;
-    }
-    p++;
-  }
+
   safe_strl(args[0], arglens[0], buff, bp);
 }
 
 /* ARGSUSED */
 FUNCTION(fun_escape)
 {
-  /* another function more or less right out of 2.0 code */
+  unsigned char *s;
 
-  char *s;
-
-  s = args[0];
-  while (*s) {
-    switch (*s) {
-    default:
-      if (s != args[0])
-	break;
-    case '%':
-    case '\\':
-    case '[':
-    case ']':
-    case '{':
-    case '}':
-    case ';':
-      safe_chr('\\', buff, bp);
+  if (arglens[0]) {
+    safe_chr('\\', buff, bp);
+    for (s = (unsigned char *) args[0]; *s; s++) {
+      if ((s != (unsigned char *) args[0]) && escaped_chars[*s])
+	safe_chr('\\', buff, bp);
+      safe_chr(*s, buff, bp);
     }
-    safe_chr(*s, buff, bp);
-    s++;
   }
 }
 
@@ -894,7 +1106,13 @@ FUNCTION(fun_trim)
   /* Alas, PennMUSH and TinyMUSH used different orders for the arguments.
    * We'll give the users an option about it
    */
-  if (TINY_TRIM_FUN) {
+  if (!strcmp(called_as, "TRIMTINY")) {
+    trim_style_arg = 2;
+    trim_char_arg = 3;
+  } else if (!strcmp(called_as, "TRIMPENN")) {
+    trim_style_arg = 3;
+    trim_char_arg = 2;
+  } else if (TINY_TRIM_FUN) {
     trim_style_arg = 2;
     trim_char_arg = 3;
   } else {
@@ -1027,7 +1245,7 @@ FUNCTION(fun_beep)
   safe_fill(BEEP_CHAR, k, buff, bp);
 }
 
-/* All the safe tags from HTML 4.0 */
+/** Initialize the html tag hash table with all the safe tags from HTML 4.0 */
 void
 init_tag_hashtab(void)
 {
@@ -1054,7 +1272,7 @@ init_tag_hashtab(void)
 FUNCTION(fun_ord)
 {
   char *m;
-  Size_t len = 0;
+  size_t len = 0;
   if (!args[0] || !args[0][0]) {
     safe_str(T("#-1 FUNCTION EXPECTS ONE CHARACTER"), buff, bp);
     return;
@@ -1100,7 +1318,10 @@ FUNCTION(fun_stripaccents)
 {
   int n;
   for (n = 0; n < arglens[0]; n++) {
-    safe_chr((char) accent_table[(unsigned char) args[0][n]].base, buff, bp);
+    if (accent_table[(unsigned char) args[0][n]].base)
+      safe_str(accent_table[(unsigned char) args[0][n]].base, buff, bp);
+    else
+      safe_chr(args[0][n], buff, bp);
   }
 }
 
@@ -1123,8 +1344,10 @@ FUNCTION(fun_tag)
     safe_chr(TAG_START, buff, bp);
     safe_strl(args[0], arglens[0], buff, bp);
     for (i = 1; i < nargs; i++) {
-      safe_chr(' ', buff, bp);
-      safe_strl(args[i], arglens[i], buff, bp);
+      if (ok_tag_attribute(executor, args[i])) {
+	safe_chr(' ', buff, bp);
+	safe_strl(args[i], arglens[i], buff, bp);
+      }
     }
     safe_chr(TAG_END, buff, bp);
   }
@@ -1146,46 +1369,45 @@ FUNCTION(fun_tagwrap)
     safe_str("#-1", buff, bp);
   else {
     if (nargs == 2)
-      safe_tag_wrap(args[0], NULL, args[1], buff, bp);
+      safe_tag_wrap(args[0], NULL, args[1], buff, bp, executor);
     else
-      safe_tag_wrap(args[0], args[1], args[2], buff, bp);
+      safe_tag_wrap(args[0], args[1], args[2], buff, bp, executor);
   }
 }
 
-#define COL_FLASH       (1)
-#define COL_HILITE      (2)
-#define COL_INVERT      (4)
-#define COL_UNDERSCORE  (8)
+#define COL_FLASH       (1)	/**< ANSI flash attribute bit */
+#define COL_HILITE      (2)	/**< ANSI hilite attribute bit */
+#define COL_INVERT      (4)	/**< ANSI inverse attribute bit */
+#define COL_UNDERSCORE  (8)	/**< ANSI underscore attribute bit */
 
-#define VAL_FLASH       (5)
-#define VAL_HILITE      (1)
-#define VAL_INVERT      (7)
-#define VAL_UNDERSCORE  (4)
+#define VAL_FLASH       (5)	/**< ANSI flag attribute value */
+#define VAL_HILITE      (1)	/**< ANSI hilite attribute value */
+#define VAL_INVERT      (7)	/**< ANSI inverse attribute value */
+#define VAL_UNDERSCORE  (4)	/**< ANSI underscore attribute value */
 
-#define COL_BLACK       (30)
-#define COL_RED         (31)
-#define COL_GREEN       (32)
-#define COL_YELLOW      (33)
-#define COL_BLUE        (34)
-#define COL_MAGENTA     (35)
-#define COL_CYAN        (36)
-#define COL_WHITE       (37)
+#define COL_BLACK       (30)	/**< ANSI color black */
+#define COL_RED         (31)	/**< ANSI color red */
+#define COL_GREEN       (32)	/**< ANSI color green */
+#define COL_YELLOW      (33)	/**< ANSI color yellow */
+#define COL_BLUE        (34)	/**< ANSI color blue */
+#define COL_MAGENTA     (35)	/**< ANSI color magenta */
+#define COL_CYAN        (36)	/**< ANSI color cyan */
+#define COL_WHITE       (37)	/**< ANSI color white */
 
+/** The ansi attributes associated with a character. */
 typedef struct {
-  char flags;
-  char fore;
-  char back;
+  char flags;		/**< Ansi text attributes */
+  char fore;		/**< Ansi foreground color */
+  char back;		/**< Ansi background color */
 } ansi_data;
 
 static void dump_ansi_codes(ansi_data * ad, char *buff, char **bp);
 
+/** If we're adding y to x, do we need to add z as well? */
 #define EDGE_UP(x,y,z)  (((y) & (z)) && !((x) & (z)))
 
 static void
-dump_ansi_codes(ad, buff, bp)
-    ansi_data *ad;
-    char *buff;
-    char **bp;
+dump_ansi_codes(ansi_data * ad, char *buff, char **bp)
 {
   static ansi_data old_ad = { 0, 0, 0 };
   int f = 0;
@@ -1513,7 +1735,7 @@ wraplen(char *str, int maxlen)
   return (maxlen ? maxlen : -1);
 }
 
-/* The integer in string a will be stored in v, 
+/** The integer in string a will be stored in v, 
  * if a is not an integer then d (efault) is stored in v. 
  */
 #define initint(a, v, d) \
@@ -1580,12 +1802,12 @@ FUNCTION(fun_wrap)
 
     if (ansilen < 0) {
       /* word doesn't fit on one line, so cut it */
-      safe_ansi_string(as, pstr - as->text, ansiwidth - 1, buff, bp);
+      safe_ansi_string2(as, pstr - as->text, ansiwidth - 1, buff, bp);
       safe_chr('-', buff, bp);
       pstr += ansiwidth - 1;	/* move to start of next line */
     } else {
       /* normal line */
-      safe_ansi_string(as, pstr - as->text, ansilen, buff, bp);
+      safe_ansi_string2(as, pstr - as->text, ansilen, buff, bp);
       if (pstr[ansilen] == '\r')
 	++ansilen;
       pstr += ansilen + 1;	/* move to start of next line */
@@ -1595,3 +1817,227 @@ FUNCTION(fun_wrap)
 }
 
 #undef initint
+
+#define AL_LEFT 1    /**< Align left */
+#define AL_RIGHT 2   /**< Align right */
+#define AL_CENTER 3  /**< Align center */
+#define AL_REPEAT 4  /**< Repeat column */
+
+static int
+align_one_line(char *buff, char **bp, int ncols,
+	       int cols[MAX_COLS], int calign[MAX_COLS], char *ptrs[MAX_COLS],
+	       ansi_string *as[MAX_COLS],
+	       int linenum, char *fieldsep, int fslen,
+	       char *linesep, int lslen, char filler)
+{
+  static char line[BUFFER_LEN];
+  static char segment[BUFFER_LEN];
+  char *sp;
+  char *ptr, *tptr;
+  char *lp;
+  char *lastspace;
+  int i, j;
+  int len;
+  int cols_done;
+  int skipspace;
+
+  lp = line;
+  memset(line, filler, BUFFER_LEN);
+  cols_done = 0;
+  for (i = 0; i < ncols; i++) {
+    if (!ptrs[i] || !*ptrs[i]) {
+      if (calign[i] & AL_REPEAT) {
+	ptrs[i] = as[i]->text;
+      } else {
+	lp += cols[i];
+	if (i < (ncols - 1) && fslen)
+	  safe_str(fieldsep, line, &lp);
+	cols_done++;
+	continue;
+      }
+    }
+    if (calign[i] & AL_REPEAT) {
+      cols_done++;
+    }
+    for (len = 0, ptr = ptrs[i], lastspace = NULL; len < cols[i]; ptr++, len++) {
+      if ((!*ptr) || (*ptr == '\n'))
+	break;
+      if (isspace((unsigned char) *ptr)) {
+	lastspace = ptr;
+      }
+    }
+    skipspace = 0;
+    sp = segment;
+    if (!*ptr) {
+      if (len > 0) {
+	safe_ansi_string2(as[i], ptrs[i] - (as[i]->text), len, segment, &sp);
+      }
+      ptrs[i] = ptr;
+    } else if (*ptr == '\n') {
+      for (tptr = ptr;
+	   *tptr && tptr >= ptrs[i] && isspace((unsigned char) *tptr); tptr--) ;
+      len = (tptr - ptrs[i]) + 1;
+      if (len > 0) {
+	safe_ansi_string2(as[i], ptrs[i] - (as[i]->text), len, segment, &sp);
+      }
+      ptrs[i] = ptr + 1;
+      ptr = tptr;
+    } else if (lastspace) {
+      ptr = lastspace;
+      skipspace = 1;
+      for (tptr = ptr;
+	   *tptr && tptr >= ptrs[i] && isspace((unsigned char) *tptr); tptr--) ;
+      len = (tptr - ptrs[i]) + 1;
+      if (len > 0) {
+	safe_ansi_string2(as[i], ptrs[i] - (as[i]->text), len, segment, &sp);
+      }
+      ptrs[i] = lastspace;
+    } else {
+      if (len > 0) {
+	safe_ansi_string2(as[i], ptrs[i] - (as[i]->text), len, segment, &sp);
+      }
+      ptrs[i] = ptr;
+    }
+    *sp = '\0';
+
+    if ((calign[i] & 3) == AL_LEFT) {
+      safe_str(segment, line, &lp);
+      lp += cols[i] - len;
+    } else if ((calign[i] & 3) == AL_RIGHT) {
+      lp += cols[i] - len;
+      safe_str(segment, line, &lp);
+    } else if ((calign[i] & 3) == AL_CENTER) {
+      j = cols[i] - len;
+      lp += j >> 1;
+      safe_str(segment, line, &lp);
+      lp += (j >> 1) + (j & 1);
+    }
+    if ((lp - line) > BUFFER_LEN)
+      lp = (line + BUFFER_LEN - 1);
+    if (i < (ncols - 1) && fslen)
+      safe_str(fieldsep, line, &lp);
+    if (skipspace)
+      for (;
+	   *ptrs[i] && (*ptrs[i] != '\n') && isspace((unsigned char) *ptrs[i]);
+	   ptrs[i]++) ;
+  }
+  if (cols_done == ncols)
+    return 0;
+  *lp = '\0';
+  if (linenum > 0 && lslen > 0)
+    safe_str(linesep, buff, bp);
+  safe_str(line, buff, bp);
+  return 1;
+}
+
+
+FUNCTION(fun_align)
+{
+  int nline;
+  char *ptr;
+  int ncols;
+  int i;
+  static int cols[MAX_COLS];
+  static int calign[MAX_COLS];
+  static ansi_string *as[MAX_COLS];
+  static char *ptrs[MAX_COLS];
+  char filler;
+  char *fieldsep;
+  int fslen;
+  char *linesep;
+  int lslen;
+
+  filler = ' ';
+  fieldsep = (char *) " ";
+  linesep = (char *) "\n";
+
+  /* Get column widths */
+  ncols = 0;
+  for (ptr = args[0]; *ptr; ptr++) {
+    while (isspace((unsigned char) *ptr))
+      ptr++;
+    if (*ptr == '>') {
+      calign[ncols] = AL_RIGHT;
+      ptr++;
+    } else if (*ptr == '-') {
+      calign[ncols] = AL_CENTER;
+      ptr++;
+    } else if (*ptr == '<') {
+      calign[ncols] = AL_LEFT;
+      ptr++;
+    } else if (isdigit((unsigned char) *ptr)) {
+      calign[ncols] = AL_LEFT;
+    } else {
+      safe_str(T("#-1 INVALID ALIGN STRING"), buff, bp);
+      return;
+    }
+    for (i = 0; *ptr && isdigit((unsigned char) *ptr); ptr++) {
+      i *= 10;
+      i += *ptr - '0';
+    }
+    if (*ptr == '.') {
+      calign[ncols] |= AL_REPEAT;
+      ptr++;
+    }
+    cols[ncols++] = i;
+    if (!*ptr)
+      break;
+  }
+
+  for (i = 0; i < ncols; i++) {
+    if (cols[i] < 1) {
+      safe_str(T("#-1 CANNOT HAVE COLUMN OF SIZE 0"), buff, bp);
+      return;
+    }
+  }
+
+  if (ncols < 1) {
+    safe_str(T("#-1 NOT ENOUGH COLUMNS FOR ALIGN"), buff, bp);
+    return;
+  }
+  if (ncols > MAX_COLS) {
+    safe_str(T("#-1 TOO MANY COLUMNS FOR ALIGN"), buff, bp);
+    return;
+  }
+  if (nargs < (ncols + 1) || nargs > (ncols + 4)) {
+    safe_str(T("#-1 INVALID NUMBER OF ARGUMENTS TO ALIGN"), buff, bp);
+    return;
+  }
+  if (nargs >= (ncols + 2)) {
+    if (!args[ncols + 1] || strlen(args[ncols + 1]) != 1) {
+      safe_str(T("#-1 FILLER MUST BE ONE CHARACTER"), buff, bp);
+      return;
+    }
+    filler = *(args[ncols + 1]);
+  }
+  if (nargs >= (ncols + 3)) {
+    fieldsep = args[ncols + 2];
+  }
+  if (nargs >= (ncols + 4)) {
+    linesep = args[ncols + 3];
+  }
+
+  fslen = strlen(fieldsep);
+  lslen = strlen(linesep);
+
+  for (i = 0; i < MAX_COLS; i++) {
+    as[i] = NULL;
+  }
+  for (i = 0; i < ncols; i++) {
+    as[i] = parse_ansi_string(args[i + 1]);
+    ptrs[i] = as[i]->text;
+  }
+
+  nline = 0;
+  while (1) {
+    if (!align_one_line(buff, bp, ncols, cols, calign, ptrs,
+			as, nline++, fieldsep, fslen, linesep, lslen, filler))
+      break;
+  }
+  **bp = '\0';
+  for (i = 0; i < ncols; i++) {
+    free_ansi_string(as[i]);
+    ptrs[i] = as[i]->text;
+  }
+  return;
+}
