@@ -1,9 +1,16 @@
+/**
+ * \file funtime.c
+ *
+ * \brief Time functions for mushcode.
+ *
+ *
+ */
 #include "copyrite.h"
 
 #include "config.h"
 #include <string.h>
 #include <ctype.h>
-#if  defined(__GNUC__) || defined(__LCC__)
+#if  (defined(__GNUC__) || defined(__LCC__)) && !defined(__USE_XOPEN_EXTENDED)
 /* Required to get the getdate() prototype on glibc. */
 #define __USE_XOPEN_EXTENDED
 #endif
@@ -14,9 +21,10 @@
 #include "parse.h"
 #include "dbdefs.h"
 #include "log.h"
+#include "match.h"
 #include "confmagic.h"
 
-int do_convtime(char *str, struct tm *ttm);
+int do_convtime(const char *str, struct tm *ttm);
 void do_timestring(char *buff, char **bp, const char *format,
 		   unsigned long secs);
 
@@ -90,25 +98,48 @@ FUNCTION(fun_timefmt)
 /* ARGSUSED */
 FUNCTION(fun_time)
 {
-  char *s;
-  struct tm *t;
+  int utc = 0;
+  int mytime;
+  double tz = 0;
+
+  mytime = mudtime;
 
   if (nargs == 1) {
-    if (!strcasecmp("UTC", args[0]))
-      t = gmtime(&mudtime);
-    else {
-      safe_str(T("#-1 INVALID TIME ZONE"), buff, bp);
-      return;
+    if (!strcasecmp("UTC", args[0])) {
+      utc = 1;
+    } else if (args[0] && *args[0] && is_strict_number(args[0])) {
+      utc = 1;
+      tz = strtod(args[0], NULL);
+      if (tz < -24.0 || tz > 24.0) {
+	safe_str(T("#-1 INVALID TIME ZONE"), buff, bp);
+	return;
+      }
+      mytime += (int) (tz * 3600);
+    } else if (args[0] && *args[0]) {
+      dbref thing;
+      ATTR *a;
+      char *ptr;
+      utc = 1;
+      thing = match_thing(executor, args[0]);
+      if (!GoodObject(thing)) {
+	safe_str(T(e_notvis), buff, bp);
+	return;
+      }
+      /* Always make time(player) return a time,
+       * even if player's TZ is unset or wonky */
+      a = atr_get(thing, "TZ");
+      if (a && is_strict_number(ptr = atr_value(a))) {
+	tz = strtod(ptr, NULL);
+	if (tz >= -24.0 || tz <= 24.0) {
+	  mytime += (int) (tz * 3600);
+	}
+      }
     }
-  } else if (!strcmp("UTCTIME", called_as))
-    t = gmtime(&mudtime);
-  else
-    t = localtime(&mudtime);
-  s = (char *) asctime(t);
-  s[strlen(s) - 1] = '\0';
-  if (s[8] == ' ')
-    s[8] = '0';
-  safe_str(s, buff, bp);
+  } else if (!strcmp("UTCTIME", called_as)) {
+    utc = 1;
+  }
+
+  safe_str(show_time(mytime, utc), buff, bp);
 }
 
 /* ARGSUSED */
@@ -124,7 +155,6 @@ FUNCTION(fun_convsecs)
 
   time_t tt;
   struct tm *ttm;
-  char *s;
   int utc = 0;
 
   if (strcmp(called_as, "CONVUTCSECS") == 0 ||
@@ -150,11 +180,7 @@ FUNCTION(fun_convsecs)
   else
     ttm = localtime(&tt);
 
-  s = (char *) asctime(ttm);
-  s[strlen(s) - 1] = '\0';
-  if (s[8] == ' ')
-    s[8] = '0';
-  safe_str(s, buff, bp);
+  safe_str(show_tm(ttm), buff, bp);
 }
 
 /* ARGSUSED */
@@ -219,11 +245,17 @@ FUNCTION(fun_timestring)
 }
 
 #ifdef HAS_GETDATE
-int do_convtime_gd(char *str, struct tm *ttm);
+int do_convtime_gd(const char *str, struct tm *ttm);
+/** Convert a time string to a struct tm using getdate().
+ * Formats for the time string are taken from the file referenced in
+ * the DATEMSK environment variable.
+ * \param str a time string.
+ * \param ttm pointer to a struct tm to fill.
+ * \retval 1 success.
+ * \retval 0 failure.
+ */
 int
-do_convtime_gd(str, ttm)
-    char *str;
-    struct tm *ttm;
+do_convtime_gd(const char *str, struct tm *ttm)
 {
   /* converts time string to a struct tm. Returns 1 on success, 0 on fail.
    * Formats of the time string are taken from the file listed in the
@@ -234,9 +266,11 @@ do_convtime_gd(str, ttm)
   tc = getdate(str);
 
   if (tc == NULL) {
+#ifdef NEVER
     if (getdate_err <= 7)
       do_rawlog(LT_ERR, "getdate returned error code %d for %s", getdate_err,
 		str);
+#endif
     return 0;
   }
 
@@ -264,10 +298,14 @@ static const char *month_table[] = {
   "Dec",
 };
 
+/** Convert a time string to a struct tm, without getdate().
+ * \param mystr a time string.
+ * \param ttm pointer to a struct tm to fill.
+ * \retval 1 success.
+ * \retval 0 failure.
+ */
 int
-do_convtime(mystr, ttm)
-    char *mystr;
-    struct tm *ttm;
+do_convtime(const char *mystr, struct tm *ttm)
 {
   /* converts time string to a struct tm. Returns 1 on success, 0 on fail.
    * Time string format is always 24 characters long, in format
@@ -373,13 +411,20 @@ FUNCTION(fun_isdaylight)
   safe_boolean(ltime->tm_isdst > 0, buff, bp);
 }
 
-/* Format codes:
-         $s - Seconds. $S - Seconds, force 2 digits.
-         $m - Minutes. $M - Minutes, force 2 digits.
-         $h - Hours.   $H - Hours, force 2 digits.
-         $d - Days.    $D - Days, force 2 digits.
-   $$ - Literal $.
-*/
+/** Convert seconds to a formatted time string.
+ * \verbatim
+ * Format codes:
+ *       $s - Seconds. $S - Seconds, force 2 digits.
+ *       $m - Minutes. $M - Minutes, force 2 digits.
+ *       $h - Hours.   $H - Hours, force 2 digits.
+ *       $d - Days.    $D - Days, force 2 digits.
+ * $$ - Literal $.
+ * \endverbatim
+ * \param buff string to store the result in.
+ * \param bp pointer into end of buff.
+ * \param format format code string.
+ * \param secs seconds to convert.
+ */
 void
 do_timestring(char *buff, char **bp, const char *format, unsigned long secs)
 {
@@ -388,6 +433,7 @@ do_timestring(char *buff, char **bp, const char *format, unsigned long secs)
   int width;
   const char *c;
   char *w;
+  int include_suffix, even_if_0, in_format_flags;
 
   days = secs / 86400;
   secs %= 86400;
@@ -404,66 +450,134 @@ do_timestring(char *buff, char **bp, const char *format, unsigned long secs)
 	pad = 0;
       else
 	pad = 1;
-      switch (*w) {
-      case '$':
-	if (pad)
-	  safe_format(buff, bp, "%*c", width, '$');
-	else
+      if (width < 0)
+	width = 0;
+      else if (width >= BUFFER_LEN)
+	width = BUFFER_LEN - 1;
+      even_if_0 = in_format_flags = 1;
+      include_suffix = 0;
+      while (in_format_flags) {
+	switch (*w) {
+	case 'x':
+	case 'X':
+	  include_suffix = 1;
+	  w++;
+	  break;
+	case 'z':
+	case 'Z':
+	  even_if_0 = 0;
+	  w++;
+	  break;
+	case '$':
+	  in_format_flags = 0;
+	  if (pad)
+	    safe_format(buff, bp, "%*c", width, '$');
+	  else
+	    safe_chr('$', buff, bp);
+	  break;
+	case 's':
+	  in_format_flags = 0;
+	  if (secs || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%*lu", width, secs);
+	    else
+	      safe_uinteger(secs, buff, bp);
+	    if (include_suffix)
+	      safe_chr('s', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	case 'S':
+	  in_format_flags = 0;
+	  if (secs || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%0*lu", width, secs);
+	    else
+	      safe_format(buff, bp, "%0lu", secs);
+	    if (include_suffix)
+	      safe_chr('s', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	case 'm':
+	  in_format_flags = 0;
+	  if (mins || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%*d", width, mins);
+	    else
+	      safe_integer(mins, buff, bp);
+	    if (include_suffix)
+	      safe_chr('m', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	case 'M':
+	  in_format_flags = 0;
+	  if (mins || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%0*d", width, mins);
+	    else
+	      safe_format(buff, bp, "%0d", mins);
+	    if (include_suffix)
+	      safe_chr('m', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	case 'h':
+	  in_format_flags = 0;
+	  if (hours || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%*d", width, hours);
+	    else
+	      safe_integer(hours, buff, bp);
+	    if (include_suffix)
+	      safe_chr('h', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	case 'H':
+	  in_format_flags = 0;
+	  if (hours || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%0*d", width, hours);
+	    else
+	      safe_format(buff, bp, "%0d", hours);
+	    if (include_suffix)
+	      safe_chr('h', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	case 'd':
+	  in_format_flags = 0;
+	  if (days || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%*d", width, days);
+	    else
+	      safe_integer(days, buff, bp);
+	    if (include_suffix)
+	      safe_chr('d', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	case 'D':
+	  in_format_flags = 0;
+	  if (days || even_if_0) {
+	    if (pad)
+	      safe_format(buff, bp, "%0*d", width, days);
+	    else
+	      safe_format(buff, bp, "%0d", days);
+	    if (include_suffix)
+	      safe_chr('d', buff, bp);
+	  } else if (pad)
+	    safe_fill(' ', width + (include_suffix ? 1 : 0), buff, bp);
+	  break;
+	default:
+	  in_format_flags = 0;
 	  safe_chr('$', buff, bp);
-	break;
-      case 's':
-	if (pad)
-	  safe_format(buff, bp, "%*lu", width, secs);
-	else
-	  safe_uinteger(secs, buff, bp);
-	break;
-      case 'S':
-	if (pad)
-	  safe_format(buff, bp, "%0*lu", width, secs);
-	else
-	  safe_format(buff, bp, "%0lu", secs);
-	break;
-      case 'm':
-	if (pad)
-	  safe_format(buff, bp, "%*d", width, mins);
-	else
-	  safe_integer(mins, buff, bp);
-	break;
-      case 'M':
-	if (pad)
-	  safe_format(buff, bp, "%0*d", width, mins);
-	else
-	  safe_format(buff, bp, "%0d", mins);
-	break;
-      case 'h':
-	if (pad)
-	  safe_format(buff, bp, "%*d", width, hours);
-	else
-	  safe_integer(hours, buff, bp);
-	break;
-      case 'H':
-	if (pad)
-	  safe_format(buff, bp, "%0*d", width, hours);
-	else
-	  safe_format(buff, bp, "%0d", hours);
-	break;
-      case 'd':
-	if (pad)
-	  safe_format(buff, bp, "%*d", width, days);
-	else
-	  safe_integer(days, buff, bp);
-	break;
-      case 'D':
-	if (pad)
-	  safe_format(buff, bp, "%0*d", width, days);
-	else
-	  safe_format(buff, bp, "%0d", days);
-	break;
-      default:
-	safe_chr('$', buff, bp);
-	for (; c != w; c++)
+	  for (; c != w; c++)
+	    safe_chr(*c, buff, bp);
 	  safe_chr(*c, buff, bp);
-	safe_chr(*c, buff, bp);
+	}
       }
       c = w;
     } else
