@@ -67,6 +67,7 @@ void Win32MUSH_setup(void);
 #include "function.h"
 #include "help.h"
 #include "dbio.h"
+#include "pcre.h"
 
 #ifdef hpux
 #include <sys/syscall.h>
@@ -96,7 +97,7 @@ static void errdb_grow(void);
 
 
 extern void initialize_mt(void);
-
+extern const unsigned char *tables;
 extern void conf_default_set(void);
 static int dump_database_internal(void);
 static FILE *db_open(const char *filename);
@@ -844,6 +845,7 @@ init_game_dbs(void)
     dbline = 0;
     if (db_read(f) < 0) {
       do_rawlog(LT_ERR, "ERROR LOADING %s", infile);
+      db_close(f);
       return -1;
     }
     do_rawlog(LT_ERR, "LOADING: %s (done)", infile);
@@ -917,13 +919,12 @@ init_game_dbs(void)
 	  do_rawlog(LT_ERR, "LOADING: %s (done)", options.chatdb);
 	} else {
 	  do_rawlog(LT_ERR, "ERROR LOADING %s", options.chatdb);
-	  db_close(f);
 	  return -1;
 	}
+	db_close(f);
       }
     }
 
-    db_close(f);
   } else {
     do_rawlog(LT_ERR, "ERROR READING DATABASE");
     return -1;
@@ -966,6 +967,61 @@ do_readcache(dbref player)
           } \
          } while(0)
 
+/** Attempt to tell if the command is a @password or @newpassword, so
+  * that the password isn't logged by Suspect or log_commands
+  * \param cmd The command to check
+  * \return A sanitized version of the command suitable for logging.
+  */
+static char *
+passwd_filter(const char *cmd)
+{
+  static int initialized = 0;
+  static pcre *pass_ptn, *newpass_ptn;
+  static char buff[BUFFER_LEN];
+  char *bp = buff;
+  int ovec[20];
+  size_t cmdlen;
+  int matched;
+
+  if (!initialized) {
+    const char *errptr;
+    int eo;
+
+    pass_ptn = pcre_compile("^(@pass.*?)\\s([^=]*)=(.*)",
+			    PCRE_CASELESS, &errptr, &eo, tables);
+    if (!pass_ptn)
+      do_log(LT_ERR, GOD, GOD, "pcre_compile: %s", errptr);
+    newpass_ptn = pcre_compile("^(@(?:newp|pcreate)[^=]*)=(.*)",
+			       PCRE_CASELESS, &errptr, &eo, tables);
+    if (!newpass_ptn)
+      do_log(LT_ERR, GOD, GOD, "pcre_compile: %s", errptr);
+    initialized = 1;
+  }
+
+  cmdlen = strlen(cmd);
+
+  if ((matched = pcre_exec(pass_ptn, NULL, cmd, cmdlen, 0, 0, ovec, 20)) > 0) {
+    /* It's a password */
+    pcre_copy_substring(cmd, ovec, matched, 1, buff, BUFFER_LEN);
+    bp = buff + strlen(buff);
+    safe_chr(' ', buff, &bp);
+    safe_fill('*', ovec[5] - ovec[4], buff, &bp);
+    safe_chr('=', buff, &bp);
+    safe_fill('*', ovec[7] - ovec[6], buff, &bp);
+  } else if ((matched = pcre_exec(newpass_ptn, NULL, cmd, cmdlen, 0, 0,
+				  ovec, 20)) > 0) {
+    pcre_copy_substring(cmd, ovec, matched, 1, buff, BUFFER_LEN);
+    bp = buff + strlen(buff);
+    safe_chr('=', buff, &bp);
+    safe_fill('*', ovec[5] - ovec[4], buff, &bp);
+  } else {
+    safe_strl(cmd, cmdlen, buff, &bp);
+  }
+  *bp = '\0';
+  return buff;
+}
+
+
 /** Attempt to match and execute a command.
  * This function performs some sanity checks and then attempts to
  * run a command. It checks, in order: home, built-in commands,
@@ -1040,14 +1096,6 @@ process_command(dbref player, char *command, dbref cause, int from_port)
   }
   orator = player;
 
-  log_activity(LA_CMD, player, command);
-  if (options.log_commands || Suspect(player))
-    do_log(LT_CMD, player, NOTHING, "%s", command);
-
-  if Verbose
-    (player)
-      raw_notify(Owner(player), tprintf("#%d] %s", player, command));
-
   /* eat leading whitespace */
   while (*command && isspace((unsigned char) *command))
     command++;
@@ -1062,6 +1110,17 @@ process_command(dbref player, char *command, dbref cause, int from_port)
   if ((!command || !*command) && !from_port)
     return;
 
+  {
+    char *msg = passwd_filter(command);
+
+    log_activity(LA_CMD, player, msg);
+    if (options.log_commands || Suspect(player))
+      do_log(LT_CMD, player, NOTHING, "%s", msg);
+    if Verbose
+      (player)
+	raw_notify(Owner(player), tprintf("#%d] %s", player, msg));
+  }
+
   /* important home checking comes first! */
   if (strcmp(command, "home") == 0) {
     if (!Mobile(player))
@@ -1072,6 +1131,8 @@ process_command(dbref player, char *command, dbref cause, int from_port)
       do_move(player, command, MOVE_NORMAL);
     return;
   }
+
+
   strcpy(unp, command);
 
   cptr = command_parse(player, cause, command, from_port);
@@ -2228,6 +2289,10 @@ do_list(dbref player, char *arg, int lc)
     do_motd(player, MOTD_LIST, "");
   else if (string_prefix("attribs", arg))
     do_list_attribs(player, lc);
+  else if (string_prefix("flags", arg))
+    do_list_flags("FLAG", player, "", lc, T("Flags"));
+  else if (string_prefix("powers", arg))
+    do_list_flags("POWER", player, "", lc, T("Powers"));
   else
     notify(player, T("I don't understand what you want to @list."));
 }
