@@ -1,8 +1,12 @@
-/* htab.c - table hashing routines */
-
-/* This code is ripped out of TinyMUSH 2.2. */
-/* Minor tweaking to make in Penn-compatible by Trivian (xeno@mix.hive.no) */
-
+/**
+ * \file htab.c
+ *
+ * \brief Hashtable routines.
+ * This code is largely ripped out of TinyMUSH 2.2.5, with tweaks
+ * to make it Penn-compatible by Trivian.
+ *
+ *
+ */
 
 #include "config.h"
 #include "copyrite.h"
@@ -11,10 +15,12 @@
 #include "externs.h"
 
 #include "htab.h"
-#include "memcheck.h"
 #include "mymalloc.h"
 #include "confmagic.h"
 
+static HASHENT *hash_new(HASHTAB *htab, const char *key);
+static int hash_val(register const char *k, int mask);
+
 /* ---------------------------------------------------------------------------
  * hash_val: Compute hash value of a string for a hash table.
  */
@@ -23,8 +29,8 @@
 
 /* This hash function adapted from http://burtleburtle.net/bob/hash/evahash.html */
 
-typedef unsigned long int u4;	/* unsigned 4-byte type */
-typedef unsigned char u1;	/* unsigned 1-byte type */
+typedef unsigned long int u4;	/**< unsigned 4-byte type */
+typedef unsigned char u1;	/**< unsigned 1-byte type */
 
 /* The mixing step */
 #define mix(a,b,c) \
@@ -41,10 +47,8 @@ typedef unsigned char u1;	/* unsigned 1-byte type */
 }
 
 /* The whole new hash function */
-int
-hash_val(k, mask)
-    register const char *k;	/* the key */
-    int mask;
+static int
+hash_val(register const char *k, int mask)
 {
   register u4 a, b, c;		/* the internal state */
   u4 len, length;		/* how many key bytes still need mixing */
@@ -100,20 +104,20 @@ hash_val(k, mask)
 
 
 #else				/* NEW_HASH_FUN */
+/** Compute a hash value for mask-style hashing.
+ * Given a null key, return 0. Otherwise, add up the numeric value
+ * of all the characters and return the sum modulo the size of the
+ * hash table.
+ * \param key key to hash.
+ * \param hashmask hash table size to use as modulus.
+ * \return hash value.
+ */
 int
-hash_val(key, hashmask)
-    const char *key;
-    int hashmask;
+hash_val(const char *key, int hashmask)
 {
   int hash = 0;
   const char *sp;
 
-  /*
-   * If the string pointer is null, return 0.  If not, add up the
-   * numeric value of all the characters and return the sum,
-   * modulo the size of the hash table.
-   */
-
   if (!key || !*key)
     return 0;
   for (sp = key; *sp; sp++)
@@ -126,29 +130,31 @@ hash_val(key, hashmask)
  * hash_getmask: Get hash mask for mask-style hashing.
  */
 
+/** Get the hash mask for mask-style hashing.
+ * Given the data size, return closest power-of-two less than that size.
+ * \param size data size.
+ * \return hash mask.
+ */
 int
-hash_getmask(size)
-    int *size;
+hash_getmask(int *size)
 {
   int tsize;
 
   if (!size || !*size)
     return 0;
 
-  /* Get next power-of-two >= size, return power-1 as the mask
-   * for ANDing
-   */
-
   for (tsize = 1; tsize < *size; tsize = tsize << 1) ;
   *size = tsize;
   return tsize - 1;
 }
 
+/** Initialize a hashtable.
+ * \param htab pointer to hash table to initialize.
+ * \param size size of hashtable.
+ * \param data_size size of an individual datum to store in the table.
+ */
 void
-hash_init(htab, size, data_size)
-    HASHTAB *htab;
-    int size;
-    int data_size;
+hash_init(HASHTAB *htab, int size, int data_size)
 {
   int i;
 
@@ -162,10 +168,13 @@ hash_init(htab, size, data_size)
   htab->entry_size = data_size;
 }
 
+/** Return a hashtable entry given a key.
+ * \param htab pointer to hash table to search.
+ * \param key key to look up in the table.
+ * \return pointer to hash table entry for given key.
+ */
 HASHENT *
-hash_find(htab, key)
-    HASHTAB *htab;
-    const char *key;
+hash_find(HASHTAB *htab, const char *key)
 {
   int hval, cmp;
   HASHENT *hptr;
@@ -184,9 +193,12 @@ hash_find(htab, key)
   return NULL;
 }
 
+/** Return the value stored in a hash entry.
+ * \param entry pointer to a hash table entry.
+ * \return generic pointer to the stored value.
+ */
 void *
-hash_value(entry)
-    HASHENT *entry;
+hash_value(HASHENT *entry)
 {
   if (entry)
     return entry->data;
@@ -194,9 +206,12 @@ hash_value(entry)
     return NULL;
 }
 
+/** Return the key stored in a hash entry.
+ * \param entry pointer to a hash table entry.
+ * \return pointer to the stored key.
+ */
 char *
-hash_key(entry)
-    HASHENT *entry;
+hash_key(HASHENT *entry)
 {
   if (entry)
     return entry->key;
@@ -204,10 +219,12 @@ hash_key(entry)
     return NULL;
 }
 
+/** Resize a hash table.
+ * \param htab pointer to hashtable.
+ * \param size new size.
+ */
 void
-hash_resize(htab, size)
-    HASHTAB *htab;
-    int size;
+hash_resize(HASHTAB *htab, int size)
 {
   int i;
   HASHENT **oldarr;
@@ -261,13 +278,11 @@ hash_resize(htab, size)
   return;
 }
 
-HASHENT *
-hash_new(htab, key)
-    HASHTAB *htab;
-    const char *key;
+static HASHENT *
+hash_new(HASHTAB *htab, const char *key)
 {
   int hval;
-  Size_t keylen;
+  size_t keylen;
   HASHENT *hptr, *curr, *old;
 
   hptr = hash_find(htab, key);
@@ -310,12 +325,17 @@ hash_new(htab, key)
   return hptr;
 }
 
+/** Add an entry to a hash table.
+ * \param htab pointer to hash table.
+ * \param key key string to store data under.
+ * \param hashdata void pointer to data to be stored.
+ * \param extra_size unused.
+ * \retval -1 failure.
+ * \retval 0 success.
+ */
 int
-hash_add(htab, key, hashdata, extra_size)
-    HASHTAB *htab;
-    const char *key;
-    void *hashdata;
-    int extra_size __attribute__ ((__unused__));
+hash_add(HASHTAB *htab, const char *key, void *hashdata,
+	 int extra_size __attribute__ ((__unused__)))
 {
   HASHENT *hptr;
 
@@ -333,10 +353,12 @@ hash_add(htab, key, hashdata, extra_size)
   return 0;
 }
 
+/** Delete an entry in a hash table.
+ * \param htab pointer to hash table.
+ * \param entry pointer to hash entry to delete (and free).
+ */
 void
-hash_delete(htab, entry)
-    HASHTAB *htab;
-    HASHENT *entry;
+hash_delete(HASHTAB *htab, HASHENT *entry)
 {
   int hval;
   HASHENT *hptr, *last;
@@ -362,10 +384,12 @@ hash_delete(htab, entry)
     hash_resize(htab, htab->hashsize >> 1);
 }
 
+/** Flush a hash table, freeing all entries.
+ * \param htab pointer to a hash table.
+ * \param size size of hash table.
+ */
 void
-hash_flush(htab, size)
-    HASHTAB *htab;
-    int size;
+hash_flush(HASHTAB *htab, int size)
 {
   HASHENT *hent, *thent;
   int i;
@@ -393,9 +417,14 @@ hash_flush(htab, size)
   }
 }
 
+/** Return the first entry of a hash table.
+ * This function is used with hash_nextentry() to iterate through a 
+ * hash table.
+ * \param htab pointer to hash table.
+ * \return first hash table entry.
+ */
 void *
-hash_firstentry(htab)
-    HASHTAB *htab;
+hash_firstentry(HASHTAB *htab)
 {
   int hval;
 
@@ -408,9 +437,35 @@ hash_firstentry(htab)
   return NULL;
 }
 
+/** Return the first key of a hash table.
+ * This function is used with hash_nextentry_key() to iterate through a 
+ * hash table.
+ * \param htab pointer to hash table.
+ * \return first hash table key.
+ */
+char *
+hash_firstentry_key(HASHTAB *htab)
+{
+  int hval;
+
+  for (hval = 0; hval < htab->hashsize; hval++)
+    if (htab->buckets[hval]) {
+      htab->last_hval = hval;
+      htab->last_entry = htab->buckets[hval];
+      return htab->buckets[hval]->key;
+    }
+  return NULL;
+}
+
+/** Return the next entry of a hash table.
+ * This function is used with hash_firstentry() to iterate through a 
+ * hash table. hash_firstentry() must be called before calling
+ * this function.
+ * \param htab pointer to hash table.
+ * \return next hash table entry.
+ */
 void *
-hash_nextentry(htab)
-    HASHTAB *htab;
+hash_nextentry(HASHTAB *htab)
 {
   int hval;
   HASHENT *hptr;
@@ -433,21 +488,55 @@ hash_nextentry(htab)
   return NULL;
 }
 
+/** Return the next key of a hash table.
+ * This function is used with hash_firstentry{,_key}() to iterate through a 
+ * hash table. hash_firstentry{,_key}() must be called before calling
+ * this function.
+ * \param htab pointer to hash table.
+ * \return next hash table key.
+ */
+char *
+hash_nextentry_key(HASHTAB *htab)
+{
+  int hval;
+  HASHENT *hptr;
+
+  hval = htab->last_hval;
+  hptr = htab->last_entry;
+  if (hptr->next) {
+    htab->last_entry = hptr->next;
+    return hptr->next->key;
+  }
+  hval++;
+  while (hval < htab->hashsize) {
+    if (htab->buckets[hval]) {
+      htab->last_hval = hval;
+      htab->last_entry = htab->buckets[hval];
+      return htab->buckets[hval]->key;
+    }
+    hval++;
+  }
+  return NULL;
+}
+
+/** Display a header for a stats listing.
+ * \param player player to notify with header.
+ */
 void
-hash_stats_header(player)
-    dbref player;
+hash_stats_header(dbref player)
 {
   notify_format(player,
 		"Table      Buckets Entries LChain  ECh  1Ch  2Ch  3Ch 4+Ch  AvgCh ~Memory");
 }
 
+/** Display stats on a hashtable.
+ * \param player player to notify with stats.
+ * \param htab pointer to the hash table.
+ * \param hname name of the hash table.
+ */
 void
-hash_stats(player, htab, hname)
-    dbref player;
-    HASHTAB *htab;
-    const char *hname;
+hash_stats(dbref player, HASHTAB *htab, const char *hname)
 {
-  /* Report some statistics on the hash table */
   int longest = 0, n;
   int lengths[5];
   double chainlens = 0.0;
