@@ -1,11 +1,13 @@
-/*
-   ** ident.c   High-level calls to the ident lib
-   **
-   ** Author: Pr Emanuelsson <pell@lysator.liu.se>
-   ** Hacked by: Peter Eriksson <pen@lysator.liu.se>
-
-   ** Many changes by Shawn Wagner to be protocol independent
-   ** for PennMUSH
+/**
+ * \file ident.c
+ *
+ * \brief High-level calls to the ident library.
+ *
+ * Author: Pr Emanuelsson <pell@lysator.liu.se>
+ * Hacked by: Peter Eriksson <pen@lysator.liu.se>
+ * 
+ * Many changes by Shawn Wagner to be protocol independent
+ * for PennMUSH
  */
 
 #include "config.h"
@@ -54,19 +56,19 @@
 #endif
 
 #include "conf.h"
+#include "externs.h"
 #include "attrib.h"
 #include "ident.h"
 #include "mymalloc.h"
-#include "memcheck.h"
-#include "externs.h"
 #include "mysocket.h"
 #include "confmagic.h"
 
   /* Low-level calls and macros */
 
+/** Structure to track an ident connection. */
 typedef struct {
-  int fd;
-  char buf[IDBUFSIZE];
+  int fd;		/**< file descriptor to read from. */
+  char buf[IDBUFSIZE];	/**< buffer to hold ident data. */
 } ident_t;
 
 
@@ -105,6 +107,14 @@ ident_lookup(int fd, int *timeout)
   return ident_query(&localaddr.addr, llen, &remoteaddr.addr, rlen, timeout);
 }
 
+/** Perform an ident query and return the result.
+ * \param laddr local socket address data.
+ * \param llen local socket address data length.
+ * \param raddr remote socket address data.
+ * \param rlen remote socket address data length.
+ * \param timeout pointer to timeout value for query.
+ * \return ident responses in IDENT pointer, or NULL.
+ */
 IDENT *
 ident_query(struct sockaddr *laddr, socklen_t llen,
 	    struct sockaddr *raddr, socklen_t rlen, int *timeout)
@@ -152,6 +162,13 @@ ident_query(struct sockaddr *laddr, socklen_t llen,
   return ident;			/* At last! */
 }
 
+/** Perform an ident lookup and return the remote identifier as a
+ * newly allocated string. This function allocates memory that
+ * should be freed by the caller.
+ * \param fd socket to use for ident lookup.
+ * \param timeout pointer to timeout value for lookup.
+ * \return allocated string containing identifier, or NULL.
+ */
 char *
 ident_id(int fd, int *timeout)
 {
@@ -166,6 +183,9 @@ ident_id(int fd, int *timeout)
   return id;
 }
 
+/** Free an IDENT structure and all elements.
+ * \param id pointer to IDENT structure to free.
+ */
 void
 ident_free(IDENT *id)
 {
@@ -207,8 +227,10 @@ id_open(struct sockaddr *faddr, socklen_t flen,
   memset(id, 0, sizeof(ident_t));
 
   if (getnameinfo(faddr, flen, host, sizeof(host), NULL, 0,
-		  NI_NUMERICHOST | NI_NUMERICSERV) != 0)
+		  NI_NUMERICHOST | NI_NUMERICSERV) != 0) {
+    free(id);
     return 0;
+  }
 
   /* Make sure we connect from the right interface. Changing the pointer
      directly doesn't seem to work. So... */
@@ -346,7 +368,7 @@ id_query(ident_t * id, struct sockaddr *laddr, socklen_t llen,
    ** Fiddling: Pr Emanuelsson <pell@lysator.liu.se> */
 
 static char *
-xstrtok(char *cp, const char *cs, char *dc)
+xstrtok(char *RESTRICT cp, const char *RESTRICT cs, char *RESTRICT dc)
 {
   static char *bp = 0;
 
@@ -395,7 +417,7 @@ id_parse(ident_t * id, int *timeout, IDENT **ident)
   char c, *cp, *tmp_charset;
   fd_set rs;
   int res = 0, lp, fp;
-  Size_t pos;
+  size_t pos;
   struct timeval to;
 
 #ifndef WIN32
