@@ -1,4 +1,14 @@
-/* info_slave.c */
+/**
+ * \file info_slave.c
+ *
+ * \brief The information slave process.
+ *
+ * When running PennMUSH under Unix, a second process (info_slave) is
+ * started and the server farms out DNS and ident lookups to the
+ * info_slave, and reads responses from the info_slave asynchronously. 
+ * Communication between server and slave is by means of a local socket.
+ *
+ */
 #include "copyrite.h"
 #include "config.h"
 
@@ -8,9 +18,6 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#ifdef I_MEMORY
-#include <memory.h>
-#endif
 #ifdef I_SYS_TYPES
 #include <sys/types.h>
 #endif
@@ -35,7 +42,9 @@
 #include "confmagic.h"
 
 /* Duplicate these, rather than trying to include strutil.o... */
+/** Arguments for functions that call APPEND_TO_BUF */
 #define APPEND_ARGS int len, blen, clen
+/** Add string c to buffer buff of max length mlen */
 #define APPEND_TO_BUF(mlen) \
   /* Trivial cases */  \
   if (c[0] == '\0') \
@@ -97,13 +106,14 @@ main(int argc, char *argv[])
   int len, size;
   IDENT *ident_result;
   char host[NI_MAXHOST];
+  char lport[NI_MAXSERV];
   int use_ident, use_dns, timeout;
   socklen_t llen, rlen;
   struct iovec dat[3];
 
   if (argc < 2) {
     fprintf(stderr, "info_slave needs a port number!\n");
-    exit(1);
+    return EXIT_FAILURE;
   }
   port = atoi(argv[1]);
   use_ident = 1;
@@ -127,7 +137,7 @@ main(int argc, char *argv[])
   mush = make_socket_conn("127.0.0.1", NULL, 0, port, NULL);
   if (mush == -1) {		/* Couldn't connect */
     fprintf(stderr, "Couldn't connect to mush!\n");
-    exit(1);
+    return EXIT_FAILURE;
   }
 #endif
   /* yes, we are _blocking_ */
@@ -138,8 +148,8 @@ main(int argc, char *argv[])
 
     len = read(mush, &rlen, sizeof rlen);
     if (len < (int) sizeof rlen) {
-      perror("info_slave reading remote size");
-      exit(1);
+      perror("info_slave reading remote size (Did the mush crash?)");
+      return EXIT_FAILURE;
     }
     /* Now the first address and len of the second. */
     dat[0].iov_base = (char *) &remote.data;
@@ -150,7 +160,7 @@ main(int argc, char *argv[])
     len = readv(mush, dat, 2);
     if (len < size) {
       perror("info_slave reading remote sockaddr and local size");
-      exit(1);
+      return EXIT_FAILURE;
     }
 
     /* Now the second address and fd. */
@@ -162,7 +172,7 @@ main(int argc, char *argv[])
     len = readv(mush, dat, 2);
     if (len < size) {
       perror("info_slave reading local sockaddr and fd");
-      exit(1);
+      return EXIT_FAILURE;
     }
 
     if (!fd)
@@ -177,6 +187,12 @@ main(int argc, char *argv[])
     else
       safe_str(host, buf, &bp);
     safe_chr('^', buf, &bp);
+    if (getnameinfo(&local.addr, llen, NULL, 0, lport, sizeof lport,
+		    NI_NUMERICHOST | NI_NUMERICSERV) != 0)
+      safe_str("An error occured", buf, &bp);
+    else
+      safe_str(lport, buf, &bp);
+    safe_chr('^', buf, &bp);
 
     if (use_ident > 0) {
       timeout = use_ident;
@@ -210,7 +226,8 @@ main(int argc, char *argv[])
     size = dat[0].iov_len + dat[1].iov_len + dat[2].iov_len;
     if (len < size) {
       perror("info_slave write packet");
-      exit(1);
+      return EXIT_FAILURE;
     }
   }
+  return EXIT_SUCCESS;
 }
