@@ -1,8 +1,12 @@
-/* lock.c */
-
-/*
+/**
+ * \file lock.c
+ *
+ * \brief Locks for PennMUSH.
+ *
+ * \verbatim
+ *
  * This is the core of Ralph Melton's rewrite of the @lock system.
- * These are some of my underlying assumptions:
+ * These are some of the underlying assumptions:
  *
  * 1) Locks are checked many more times than they are set, so it is
  * quite worthwhile to spend time when setting locks if it expedites
@@ -19,6 +23,8 @@
  * than for people to check for locks in MUSHcode. Therefore, it is
  * reasonable to incur a minor slowdown for checking locks in MUSHcode
  * in order to speed up the server's checking.
+ *
+ * \endverbatim
  */
 
 #include "copyrite.h"
@@ -28,19 +34,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include "conf.h"
+#include "externs.h"
 #include "boolexp.h"
 #include "mushdb.h"
 #include "attrib.h"
-#include "externs.h"
 #include "dbdefs.h"
 #include "lock.h"
 #include "match.h"
 #include "log.h"
 #include "flags.h"
 #include "dbdefs.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
 #include "mymalloc.h"
 #include "strtree.h"
 #include "privtab.h"
@@ -51,57 +54,74 @@
 /* If any lock_type ever contains the character '|', reading in locks
  * from the db will break.
  */
-const lock_type Basic_Lock = "Basic";
-const lock_type Enter_Lock = "Enter";
-const lock_type Use_Lock = "Use";
-const lock_type Zone_Lock = "Zone";
-const lock_type Page_Lock = "Page";
-const lock_type Tport_Lock = "Teleport";
-const lock_type Speech_Lock = "Speech";
-const lock_type Listen_Lock = "Listen";
-const lock_type Command_Lock = "Command";
-const lock_type Parent_Lock = "Parent";
-const lock_type Link_Lock = "Link";
-const lock_type Leave_Lock = "Leave";
-const lock_type Drop_Lock = "Drop";
-const lock_type Give_Lock = "Give";
-const lock_type Mail_Lock = "Mail";
-const lock_type Follow_Lock = "Follow";
-const lock_type Examine_Lock = "Examine";
-const lock_type Chzone_Lock = "Chzone";
-const lock_type Forward_Lock = "Forward";
-const lock_type Control_Lock = "Control";
-const lock_type Dropto_Lock = "Dropto";
-const lock_type Destroy_Lock = "Destroy";
+const lock_type Basic_Lock = "Basic";	  /**< Name of basic lock */
+const lock_type Enter_Lock = "Enter";	  /**< Name of enter lock */
+const lock_type Use_Lock = "Use";	  /**< Name of use lock */
+const lock_type Zone_Lock = "Zone";	  /**< Name of zone lock */
+const lock_type Page_Lock = "Page";	  /**< Name of page lock */
+const lock_type Tport_Lock = "Teleport";  /**< Name of teleport lock */
+const lock_type Speech_Lock = "Speech";	  /**< Name of speech lock */
+const lock_type Listen_Lock = "Listen";	  /**< Name of listen lock */
+const lock_type Command_Lock = "Command"; /**< Name of command lock */
+const lock_type Parent_Lock = "Parent";	  /**< Name of parent lock */
+const lock_type Link_Lock = "Link";	  /**< Name of link lock */
+const lock_type Leave_Lock = "Leave";	  /**< Name of leave lock */
+const lock_type Drop_Lock = "Drop";	  /**< Name of drop lock */
+const lock_type Give_Lock = "Give";	  /**< Name of give lock */
+const lock_type Mail_Lock = "Mail";	  /**< Name of mail lock */
+const lock_type Follow_Lock = "Follow";	  /**< Name of follow lock */
+const lock_type Examine_Lock = "Examine"; /**< Name of examine lock */
+const lock_type Chzone_Lock = "Chzone";	  /**< Name of chzone lock */
+const lock_type Forward_Lock = "Forward"; /**< Name of forward lock */
+const lock_type Control_Lock = "Control"; /**< Name of control lock */
+const lock_type Dropto_Lock = "Dropto";	  /**< Name of dropto lock */
+const lock_type Destroy_Lock = "Destroy"; /**< Name of destroy lock */
+const lock_type Interact_Lock = "Interact"; /**< Name of interaction lock */
 /* Define new lock types here. */
 
+/** Table of lock names and permissions */
 const lock_list lock_types[] = {
-  {"Basic", NULL, GOD, LF_PRIVATE, NULL},
-  {"Enter", NULL, GOD, LF_PRIVATE, NULL},
-  {"Use", NULL, GOD, LF_PRIVATE, NULL},
-  {"Zone", NULL, GOD, LF_PRIVATE, NULL},
-  {"Page", NULL, GOD, LF_PRIVATE, NULL},
-  {"Teleport", NULL, GOD, LF_PRIVATE, NULL},
-  {"Speech", NULL, GOD, LF_PRIVATE | LF_WIZARD, NULL},
-  {"Listen", NULL, GOD, LF_PRIVATE, NULL},
-  {"Command", NULL, GOD, LF_PRIVATE, NULL},
-  {"Parent", NULL, GOD, LF_PRIVATE, NULL},
-  {"Link", NULL, GOD, LF_PRIVATE, NULL},
-  {"Leave", NULL, GOD, LF_PRIVATE, NULL},
-  {"Drop", NULL, GOD, LF_PRIVATE, NULL},
-  {"Give", NULL, GOD, LF_PRIVATE, NULL},
-  {"Mail", NULL, GOD, LF_PRIVATE, NULL},
-  {"Follow", NULL, GOD, LF_PRIVATE, NULL},
-  {"Examine", NULL, GOD, LF_PRIVATE | LF_OWNER, NULL},
-  {"Chzone", NULL, GOD, LF_PRIVATE, NULL},
-  {"Forward", NULL, GOD, LF_PRIVATE | LF_OWNER, NULL},
-  {"Control", NULL, GOD, LF_PRIVATE | LF_OWNER, NULL},
-  {"Dropto", NULL, GOD, LF_PRIVATE, NULL},
-  {"Destroy", NULL, GOD, LF_PRIVATE | LF_OWNER, NULL},
+  {"Basic", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Enter", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Use", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Zone", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Page", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Teleport", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Speech", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Listen", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Command", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Parent", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Link", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Leave", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Drop", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Give", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Mail", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Follow", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Examine", TRUE_BOOLEXP, GOD, LF_PRIVATE | LF_OWNER, NULL},
+  {"Chzone", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Forward", TRUE_BOOLEXP, GOD, LF_PRIVATE | LF_OWNER, NULL},
+  {"Control", TRUE_BOOLEXP, GOD, LF_PRIVATE | LF_OWNER, NULL},
+  {"Dropto", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
+  {"Destroy", TRUE_BOOLEXP, GOD, LF_PRIVATE | LF_OWNER, NULL},
+  {"Interact", TRUE_BOOLEXP, GOD, LF_PRIVATE, NULL},
   /* Add new lock types just before this line. */
-  {NULL, NULL, GOD, 0, NULL}
+  {NULL, TRUE_BOOLEXP, GOD, 0, NULL}
 };
 
+/** Table of base attributes associated with success and failure of
+ * locks. These are the historical ones; we automatically generate
+ * such attribute names for those that aren't in this table using
+ * <lock>_LOCK`<message>
+ */
+const LOCKMSGINFO lock_msgs[] = {
+  {"Basic", "SUCCESS", "FAILURE"},
+  {"Enter", "ENTER", "EFAIL"},
+  {"Use", "USE", "UFAIL"},
+  {"Leave", "LEAVE", "LFAIL"},
+  {NULL, NULL, NULL}
+};
+
+/** Table of lock permissions */
 PRIV lock_privs[] = {
   {"visual", 'v', LF_VISUAL, LF_VISUAL},
   {"no_inherit", 'i', LF_PRIVATE, LF_PRIVATE},
@@ -112,26 +132,69 @@ PRIV lock_privs[] = {
   {NULL, '\0', 0, 0}
 };
 
-StrTree lock_names;
+StrTree lock_names;  /**< String tree of lock names */
 
 static void free_one_lock_list(lock_list *ll);
 static lock_type check_lock_type(dbref player, dbref thing, lock_type name);
 static int delete_lock(dbref player, dbref thing, lock_type type);
 static int can_write_lock(dbref player, dbref thing, lock_list *lock);
+static lock_list *getlockstruct_noparent(dbref thing, lock_type type);
 
-/* Assuming 4096 byte pages */
+/** Number of locks to store in a page, assuming 4096 byte pages */
 #define LOCKS_PER_PAGE 200
 
 static lock_list *free_list = NULL;
 
 static lock_list *next_free_lock(void);
 static void free_lock(lock_list *ll);
+
+/** Return a list of lock flag characters.
+ * \param ll pointer to a lock.
+ * \return string of lock flag characters.
+ */
 const char *
 lock_flags(lock_list *ll)
 {
   return privs_to_letters(lock_privs, L_FLAGS(ll));
 }
 
+/** List all lock flag characters on a buffer
+ * \param buff The buffer
+ * \param bp Pointer to a position in the buffer.
+ */
+
+void
+list_lock_flags(char *buff, char **bp)
+{
+  int i;
+  for (i = 0; lock_privs[i].name; i++) {
+    if (lock_privs[i].letter)
+      safe_chr(lock_privs[i].letter, buff, bp);
+  }
+}
+
+/** List all lock flag names on a buffer
+ * \param buff The buffer
+ * \param bp Pointer to a position in the buffer.
+ */
+
+void
+list_lock_flags_long(char *buff, char **bp)
+{
+  int i;
+  int first = 1;
+  for (i = 0; lock_privs[i].name; i++) {
+    if (!first)
+      safe_chr(' ', buff, bp);
+    first = 0;
+    safe_str(lock_privs[i].name, buff, bp);
+  }
+}
+
+/** Return a list of lock flag names.
+ * \param ll pointer to a lock.
+ * \return string of lock flag names, space-separated.
+ */
 const char *
 lock_flags_long(lock_list *ll)
 {
@@ -151,6 +214,7 @@ string_to_lockflag(dbref player, char const *p)
   return f;
 }
 
+/** Initialize the lock strtree. */
 void
 init_locks(void)
 {
@@ -182,16 +246,16 @@ next_free_lock(void)
   lock_list *ll;
 
   if (!free_list) {
-    Size_t n;
+    size_t n;
 
     ll = mush_malloc(sizeof(lock_list) * LOCKS_PER_PAGE, "lock_page");
 
     if (!ll)
-      panic("Unable to allocate memory for locks!");
+      mush_panic("Unable to allocate memory for locks!");
 
     for (n = 0; n < LOCKS_PER_PAGE - 1; n++) {
       ll[n].type = NULL;
-      ll[n].key = NULL;
+      ll[n].key = TRUE_BOOLEXP;
       ll[n].creator = NOTHING;
       ll[n].flags = 0;
       ll[n].next = &ll[n + 1];
@@ -199,7 +263,7 @@ next_free_lock(void)
 
     ll[n].next = NULL;
     ll[n].type = NULL;
-    ll[n].key = NULL;
+    ll[n].key = TRUE_BOOLEXP;
     ll[n].creator = NOTHING;
     ll[n].flags = 0;
 
@@ -210,7 +274,7 @@ next_free_lock(void)
   free_list = ll->next;
 
   ll->type = NULL;
-  ll->key = NULL;
+  ll->key = TRUE_BOOLEXP;
 
   return ll;
 }
@@ -219,17 +283,19 @@ static void
 free_lock(lock_list *ll)
 {
   ll->type = NULL;
-  ll->key = NULL;
+  ll->key = TRUE_BOOLEXP;
   ll->creator = NOTHING;
   ll->flags = 0;
   ll->next = free_list;
   free_list = ll;
 }
 
-/* Given a lock type, find a lock.
- * This depends on the implementation of lock_type as a string.
+/** Given a lock type, find a lock, possibly checking parents.
+ * \param thing object on which lock is to be found.
+ * \param type type of lock to find.
+ * \return pointer to boolexp of lock.
  */
-struct boolexp *
+boolexp
 getlock(dbref thing, lock_type type)
 {
   struct lock_list *ll = getlockstruct(thing, type);
@@ -239,7 +305,12 @@ getlock(dbref thing, lock_type type)
     return L_KEY(ll);
 }
 
-struct boolexp *
+/** Given a lock type, find a lock without checking parents. 
+ * \param thing object on which lock is to be found.
+ * \param type type of lock to find.
+ * \return pointer to boolexp of lock.
+ */
+boolexp
 getlock_noparent(dbref thing, lock_type type)
 {
   struct lock_list *ll = getlockstruct_noparent(thing, type);
@@ -253,27 +324,33 @@ lock_list *
 getlockstruct(dbref thing, lock_type type)
 {
   lock_list *ll;
-  dbref p;
-  int cmp, count;
+  dbref p = thing, ancestor = NOTHING;
+  int cmp, count = 0, ancestor_in_chain = 0;
 
-  count = 0;
-  for (p = thing; GoodObject(p); p = Parent(p)) {
-    if (count++ > 100)
-      return NULL;
-    ll = Locks(p);
-    while (ll && L_TYPE(ll)) {
-      cmp = strcasecmp(L_TYPE(ll), type);
-      if (cmp == 0)
-	return (p != thing && (ll->flags & LF_PRIVATE)) ? NULL : ll;
-      else if (cmp > 0)
-	break;
-      ll = ll->next;
+  if (GoodObject(thing))
+    ancestor = Ancestor_Parent(thing);
+  do {
+    for (; GoodObject(p); p = Parent(p)) {
+      if (count++ > 100)
+	return NULL;
+      if (p == ancestor)
+	ancestor_in_chain = 1;
+      ll = Locks(p);
+      while (ll && L_TYPE(ll)) {
+	cmp = strcasecmp(L_TYPE(ll), type);
+	if (cmp == 0)
+	  return (p != thing && (ll->flags & LF_PRIVATE)) ? NULL : ll;
+	else if (cmp > 0)
+	  break;
+	ll = ll->next;
+      }
     }
-  }
+    p = ancestor;
+  } while (!ancestor_in_chain && !Orphan(thing) && GoodObject(ancestor));
   return NULL;
 }
 
-lock_list *
+static lock_list *
 getlockstruct_noparent(dbref thing, lock_type type)
 {
   lock_list *ll = Locks(thing);
@@ -291,7 +368,10 @@ getlockstruct_noparent(dbref thing, lock_type type)
 }
 
 
-/* Is this lock in our set of canonical locks? */
+/** Determine if a lock type is one of the standard types or not.
+ * \param type type of lock to check.
+ * \return canonical lock type or NULL
+ */
 lock_type
 match_lock(lock_type type)
 {
@@ -304,29 +384,39 @@ match_lock(lock_type type)
   return NULL;
 }
 
-/* Return the proper entry from lock_types, or NULL */
+/** Return the proper entry from lock_types, or NULL.
+ * \param type of lock to look up.
+ * \return lock_types entry for lock.
+ */
 const lock_list *
 get_lockproto(lock_type type)
 {
   const lock_list *ll;
 
   for (ll = lock_types; ll->type; ll++)
-    if (strcmp(type, ll->type) == 0)
+    if (strcasecmp(type, ll->type) == 0)
       return ll;
 
   return NULL;
 
 }
 
-/* Set the lock type on thing to boolexp.
+/** Add a lock to an object (primitive).
+ * Set the lock type on thing to boolexp.
  * This is a primitive routine, to be called by other routines.
  * It will go somewhat wonky if given a NULL boolexp.
  * It will allocate memory if called with a string that is not already
  * in the lock table.
+ * \param player the enactor, for permission checking.
+ * \param thing object on which to set the lock.
+ * \param type type of lock to set.
+ * \param key lock boolexp pointer (should not be NULL!).
+ * \param flags lock flags.
+ * \retval 0 failure.
+ * \retval 1 success.
  */
 int
-add_lock(dbref player, dbref thing, lock_type type, struct boolexp *key,
-	 int flags)
+add_lock(dbref player, dbref thing, lock_type type, boolexp key, int flags)
 {
   lock_list *ll, **t;
   lock_type real_type = type;
@@ -382,13 +472,23 @@ add_lock(dbref player, dbref thing, lock_type type, struct boolexp *key,
   return 1;
 }
 
-/* Set the lock type on thing to boolexp.
+/** Add a lock to an object on db load.
+ * Set the lock type on thing to boolexp.
  * Used only on db load, when we can't safely test the player's
  * permissions because they're not loaded yet.
+ * This is a primitive routine, to be called by other routines.
+ * It will go somewhat wonky if given a NULL boolexp.
+ * It will allocate memory if called with a string that is not already
+ * in the lock table.
+ * \param player lock creator.
+ * \param thing object on which to set the lock.
+ * \param type type of lock to set.
+ * \param key lock boolexp pointer (should not be NULL!).
+ * \param flags lock flags.
+ * \retval 0 failure.
  */
 int
-add_lock_raw(dbref player, dbref thing, lock_type type, struct boolexp *key,
-	     int flags)
+add_lock_raw(dbref player, dbref thing, lock_type type, boolexp key, int flags)
 {
   lock_list *ll, **t;
   lock_type real_type = type;
@@ -435,7 +535,12 @@ free_one_lock_list(lock_list *ll)
   free_lock(ll);
 }
 
-/* Another primitive routine. */
+/** Delete a lock from an object (primitive).
+ * Another primitive routine.
+ * \param player the enactor, for permission checking.
+ * \param thing object on which to remove the lock.
+ * \param type type of lock to remove.
+ */
 int
 delete_lock(dbref player, dbref thing, lock_type type)
 {
@@ -459,9 +564,12 @@ delete_lock(dbref player, dbref thing, lock_type type)
     return 1;
 }
 
+/** Free all locks in a list.
+ * Used by the object destruction routines.
+ * \param ll pointer to list of locks.
+ */
 void
-free_locks(ll)
-    lock_list *ll;
+free_locks(lock_list *ll)
 {
   lock_list *ll2;
   while (ll) {
@@ -472,7 +580,7 @@ free_locks(ll)
 }
 
 
-/* Check to see that the lock type is a valid type.
+/** Check to see that the lock type is a valid type.
  * If it's not in our lock table, it's not valid,
  * unless it begins with 'user:' or an abbreviation thereof,
  * in which case the lock type is the part after the :.
@@ -480,6 +588,11 @@ free_locks(ll)
  * will confuse our db-reading routines.
  *
  * Might destructively modify name.
+ *
+ * \param player the enactor, for notification.
+ * \param thing object on which to check the lock.
+ * \param name name of lock type.
+ * \return lock type, or NULL.
  */
 static lock_type
 check_lock_type(dbref player, dbref thing, lock_type name)
@@ -522,6 +635,14 @@ check_lock_type(dbref player, dbref thing, lock_type name)
   return sp;
 }
 
+/** Unlock a lock (user interface).
+ * \verbatim
+ * This implements @unlock.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of object to unlock.
+ * \param type type of lock to unlock.
+ */
 void
 do_unlock(dbref player, const char *name, lock_type type)
 {
@@ -537,31 +658,37 @@ do_unlock(dbref player, const char *name, lock_type type)
   }
   if ((thing = match_controlled(player, name)) != NOTHING) {
     if ((real_type = check_lock_type(player, thing, type)) != NULL) {
-      if (getlock(thing, real_type) == TRUE_BOOLEXP)
-	notify_format(player, T("%s(%s) - (already) unlocked."), Name(thing),
-		      unparse_dbref(thing));
-      else if (delete_lock(player, thing, real_type)) {
-	notify_format(player, T("%s(%s) - unlocked."), Name(thing),
-		      unparse_dbref(thing));
-#ifdef CREATION_TIMES
-	ModTime(thing) = mudtime;
-#endif
+      if (getlock(thing, real_type) == TRUE_BOOLEXP) {
+	if (!AreQuiet(player, thing))
+	  notify_format(player, T("%s(%s) - %s (already) unlocked."),
+			Name(thing), unparse_dbref(thing), real_type);
+      } else if (delete_lock(player, thing, real_type)) {
+	if (!AreQuiet(player, thing))
+	  notify_format(player, T("%s(%s) - %s unlocked."), Name(thing),
+			unparse_dbref(thing), real_type);
+	if (!IsPlayer(thing))
+	  ModTime(thing) = mudtime;
       } else
 	notify(player, T("Permission denied."));
     }
   }
 }
 
+/** Set/lock a lock (user interface).
+ * \verbatim
+ * This implements @lock.
+ * \endverbatim
+ * \param player the enactor.
+ * \param name name of object to lock.
+ * \param keyname key to lock the lock to, as a string.
+ * \param type type of lock to lock.
+ */
 void
-do_lock(player, name, keyname, type)
-    dbref player;
-    const char *name;
-    const char *keyname;
-    lock_type type;
+do_lock(dbref player, const char *name, const char *keyname, lock_type type)
 {
   lock_type real_type;
   dbref thing;
-  struct boolexp *key;
+  boolexp key;
   char *sp;
 
   /* check for '@lock <object>/<atr>'  */
@@ -602,17 +729,22 @@ do_lock(player, name, keyname, type)
     if ((real_type = check_lock_type(player, thing, type)) != NULL) {
       /* everything ok, do it */
       if (add_lock(player, thing, real_type, key, -1)) {
-	notify_format(player, T("%s(%s) - locked."), Name(thing),
-		      unparse_dbref(thing));
-#ifdef CREATION_TIMES
-	ModTime(thing) = mudtime;
-#endif
+	if (!AreQuiet(player, thing))
+	  notify_format(player, T("%s(%s) - %s locked."), Name(thing),
+			unparse_dbref(thing), real_type);
+	if (!IsPlayer(thing))
+	  ModTime(thing) = mudtime;
       } else
 	notify(player, T("Permission denied."));
     }
   }
 }
 
+/** Copy the locks from one object to another.
+ * \param player the enactor.
+ * \param orig the source object.
+ * \param clone the destination object.
+ */
 void
 clone_locks(dbref player, dbref orig, dbref clone)
 {
@@ -624,13 +756,81 @@ clone_locks(dbref player, dbref orig, dbref clone)
 }
 
 
-/* Evaluate lock ltype on thing for player */
+/** Evaluate a lock.
+ * Evaluate lock ltype on thing for player.
+ * \param player dbref attempting to pass the lock.
+ * \param thing object containing the lock.
+ * \param ltype type of lock to check.
+ * \retval 1 player passes the lock.
+ * \retval 0 player does not pass the lock.
+ */
 int
 eval_lock(dbref player, dbref thing, lock_type ltype)
 {
-  return eval_boolexp(player, getlock(thing, ltype), thing);
+  boolexp b = getlock(thing, ltype);
+  log_activity(LA_LOCK, thing, unparse_boolexp(player, b, UB_DBREF));
+  return eval_boolexp(player, b, thing);
 }
 
+/** Active a lock's failure attributes.
+ * \param player dbref failing to pass the lock.
+ * \param thing object containing the lock.
+ * \param ltype type of lock failed.
+ * \param def default message if there is no appropriate failure attribute.
+ * \param loc location in which action is taking place.
+ * \retval 1 some attribute on the object was actually evaluated.
+ * \retval 0 no attributes were evaluated (only defaults used).
+ */
+int
+fail_lock(dbref player, dbref thing, lock_type ltype, const char *def,
+	  dbref loc)
+{
+  const LOCKMSGINFO *lm;
+  char atr[BUFFER_LEN];
+  char oatr[BUFFER_LEN];
+  char aatr[BUFFER_LEN];
+  char *bp;
+
+  /* Find the lock's failure attribute, if it's there */
+  for (lm = lock_msgs; lm->type; lm++) {
+    if (!strcmp(lm->type, ltype))
+      break;
+  }
+  if (lm->type) {
+    strcpy(atr, lm->failbase);
+    bp = oatr;
+    safe_format(oatr, &bp, "O%s", lm->failbase);
+    *bp = '\0';
+    strcpy(aatr, oatr);
+    aatr[0] = 'A';
+  } else {
+    /* Oops, it's not in the table. So we construct them on these lines:
+     * <LOCKNAME>_LOCK`<type>FAILURE
+     */
+    bp = atr;
+    safe_format(atr, &bp, "%s_LOCK`FAILURE", ltype);
+    *bp = '\0';
+    bp = oatr;
+    safe_format(oatr, &bp, "%s_LOCK`OFAILURE", ltype);
+    *bp = '\0';
+    bp = aatr;
+    safe_format(aatr, &bp, "%s_LOCK`AFAILURE", ltype);
+    *bp = '\0';
+  }
+  /* Now do the work */
+  upcasestr(atr);
+  upcasestr(oatr);
+  upcasestr(aatr);
+  return did_it(player, thing, atr, def, oatr, NULL, aatr, loc);
+}
+
+
+/** Determine if a lock is visual.
+ * \param thing object containing the lock.
+ * \param ltype type of lock to check.
+ * \retval (non-zero) lock is visual.
+ * \retval 0 lock is not visual.
+ */
 int
 lock_visual(dbref thing, lock_type ltype)
 {
@@ -641,6 +841,14 @@ lock_visual(dbref thing, lock_type ltype)
     return 0;
 }
 
+/** Set flags on a lock (user interface).
+ * \verbatim
+ * This implements @lset.
+ * \endverbatim
+ * \param player the enactor.
+ * \param what string in the form obj/lock.
+ * \param flags list of flags to set.
+ */
 void
 do_lset(dbref player, char *what, char *flags)
 {
@@ -685,9 +893,42 @@ do_lset(dbref player, char *what, char *flags)
   else
     L_FLAGS(l) |= flag;
 
-  notify_format(player, "%s/%s - %s.", Name(thing), L_TYPE(l),
-		unset ? "unset" : "set");
-#ifdef CREATION_TIMES
-  ModTime(thing) = mudtime;
-#endif
+  if (!Quiet(player) && !(Quiet(thing) && (Owner(thing) == player)))
+    notify_format(player, "%s/%s - %s.", Name(thing), L_TYPE(l),
+		  unset ? T("lock flags unset") : T("lock flags set"));
+  if (!IsPlayer(thing))
+    ModTime(thing) = mudtime;
+}
+
+/** Check to see if an object has a good zone lock set.
+ * If it doesn't have a lock at all, set one of '=Zone'.
+ * \param player The object responsible for having the lock checked.
+ * \param zone the object whose lock needs to be checked.
+ * \param noisy if 1, notify player of automatic locking
+ */
+void
+check_zone_lock(dbref player, dbref zone, int noisy)
+{
+  boolexp key = getlock(zone, Zone_Lock);
+  if (key == TRUE_BOOLEXP) {
+    add_lock(GOD, zone, Zone_Lock, parse_boolexp(zone, "=me", Zone_Lock), -1);
+    if (noisy)
+      notify_format(player,
+		    T
+		    ("Unlocked zone %s - automatically zone-locking to itself"),
+		    unparse_object(player, zone));
+  } else if (eval_lock(Location(player), zone, Zone_Lock)) {
+    /* Does #0 and #2 pass it? If so, probably trivial elock */
+    if (eval_lock(PLAYER_START, zone, Zone_Lock) &&
+	eval_lock(MASTER_ROOM, zone, Zone_Lock)) {
+      if (noisy)
+	notify_format(player,
+		      T("Zone %s really should have a more secure zone-lock."),
+		      unparse_object(player, zone));
+    } else			/* Probably inexact zone lock */
+      notify_format(player,
+		    T
+		    ("Warning: Zone %s may have loose zone lock. Lock zones to =player, not player"),
+		    unparse_object(player, zone));
+  }
 }
