@@ -955,7 +955,7 @@ do_score(dbref player)
 		  T("You have %d %s."),
 		  Pennies(player), Pennies(player) == 1 ? MONEY : MONIES);
     if (Moneybags(player))
-      notify(player, T("You may give unlimited ducats."));
+      notify_format(player, T("You may give unlimited %s"), MONIES);
   }
 }
 
