@@ -1,5 +1,9 @@
-/*-------------------------------------------------------------------*
- * malias.c - Global @mail aliases/lists
+/**
+ * \file malias.c
+ *
+ * \brief  Global mail aliases/lists
+ *
+ * \verbatim
  *
  * This code implements an extension to extended @mail which allows
  * admin (and others who are so em@powered) to create mail aliases
@@ -25,9 +29,11 @@
  * @malias/stat
  * @malias/chown !name=owner    (Admin only)
  * @malias/nuke                 (Admin only)
- *-------------------------------------------------------------------*/
+ *
+ * \endverbatim
+ */
 
-#define MA_INC 3
+#define MA_INC 3	/**< How many maliases we malloc at a time */
 #include "config.h"
 #include "copyrite.h"
 
@@ -40,16 +46,12 @@
 #ifdef I_SYS_TYPES
 #include <sys/types.h>
 #endif
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "dbdefs.h"
-#include "externs.h"
 #include "match.h"
 #include "parse.h"
 #include "malias.h"
@@ -58,18 +60,16 @@
 #include "flags.h"
 #include "pueblo.h"
 #include "log.h"
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
+#include "dbio.h"
 #include "confmagic.h"
 
 
-#ifdef MAIL_ALIASES
 
-int ma_size = 0;
-int ma_top = 0;
-struct mail_alias *malias;
+int ma_size = 0;   /**< Number of maliases */
+int ma_top = 0;	   /**< Top of alias array */
+struct mail_alias *malias; /**< Pointer to linked list of aliases */
 
+/** Privilege table for maliases. */
 static PRIV malias_priv_table[] = {
   {"Admin", 'A', ALIAS_ADMIN, ALIAS_ADMIN},
   {"Members", 'M', ALIAS_MEMBERS, ALIAS_MEMBERS},
@@ -77,7 +77,6 @@ static PRIV malias_priv_table[] = {
   {NULL, '\0', 0, 0}
 };
 
-static const char *get_privs(int flags);
 static const char *get_shortprivs(struct mail_alias *m);
 
 
@@ -86,11 +85,16 @@ static const char *get_shortprivs(struct mail_alias *m);
 ***********************************************************/
 
 
+/** List or create a malias.
+ * \verbatim
+ * This implements the @malias command (with no switches).
+ * \endverbatim
+ * \param player the enactor.
+ * \param arg1 name of malias to create or list, or NULL to list all.
+ * \param arg2 parameters for creation, or NULL to list.
+ */
 void
-do_malias(player, arg1, arg2)
-    dbref player;
-    char *arg1;
-    char *arg2;
+do_malias(dbref player, char *arg1, char *arg2)
 {
   if (!arg1 || !*arg1) {
     if (arg2 && *arg2) {
@@ -111,11 +115,16 @@ do_malias(player, arg1, arg2)
 }
 
 
+/** Create a malias.
+ * \verbatim
+ * This implements the @malias/create command.
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of malias to create.
+ * \param tolist parameters for creation.
+ */
 void
-do_malias_create(player, alias, tolist)
-    dbref player;
-    char *alias;
-    char *tolist;
+do_malias_create(dbref player, char *alias, char *tolist)
 {
   char *head, *tail, spot;
   struct mail_alias *m;
@@ -237,9 +246,7 @@ do_malias_create(player, alias, tolist)
   m->owner = player;
   m->name = mush_strdup(na, "malias_name");
   m->desc = compress(na);
-#ifdef MEM_CHECK
   add_check("malias_desc");
-#endif
   m->nflags = ALIAS_OWNER | ALIAS_MEMBERS;
   m->mflags = ALIAS_OWNER;
   ma_top++;
@@ -249,10 +256,14 @@ do_malias_create(player, alias, tolist)
 }
 
 
-
+/** List maliases.
+ * \verbatim
+ * This function implements @malias/list.
+ * \endverbatim
+ * \param player the enactor.
+ */
 void
-do_malias_list(player)
-    dbref player;
+do_malias_list(dbref player)
 {
   struct mail_alias *m;
   int i = 0;
@@ -279,10 +290,15 @@ do_malias_list(player)
   notify(player, T("*****  End of Mail Aliases *****"));
 }
 
+/** List malias members.
+ * \verbatim
+ * This function implements @malias/members.
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of the alias to list members of.
+ */
 void
-do_malias_members(player, alias)
-    dbref player;
-    char *alias;
+do_malias_members(dbref player, char *alias)
 {
   struct mail_alias *m;
   int i = 0;
@@ -314,13 +330,73 @@ do_malias_members(player, alias)
   notify(player, buff);
 }
 
+FUNCTION(fun_malias)
+{
+  /* With no arguments, list all alias names
+   * With one argument, it's either a delimiter or list all member dbrefs
+   * With two arguments, it's a malias name and a delimiter, and we
+   * list dbrefs, delimited
+   */
+  int i;
+  int count = 0;
+  char sep = ' ';
+  struct mail_alias *m;
+
+  if (nargs >= 1) {
+    m = get_malias(executor, args[0]);
+    if (m) {
+      if (!delim_check(buff, bp, nargs, args, 2, &sep))
+	return;
+      if ((m->owner == executor) || (m->mflags == 0) ||
+	  (Hasprivs(executor)) ||
+	  ((m->mflags & ALIAS_MEMBERS) && ismember(m, executor))) {
+	for (i = 0; i < m->size; i++) {
+	  if (count++)
+	    safe_chr(sep, buff, bp);
+	  safe_dbref(m->members[i], buff, bp);
+	}
+      } else {
+	safe_str(T(e_perm), buff, bp);
+      }
+      return;
+    } else {
+      /* Perhaps it's a delimiter? */
+      if (arglens[0] > 1) {
+	/* Oops, not if it's longer than one character */
+	safe_str(T(e_match), buff, bp);
+	return;
+      }
+      if (!delim_check(buff, bp, nargs, args, 1, &sep))
+	return;
+
+    }
+
+  }
+  /* List maliases, possibly with a delimiter */
+  for (i = 0; i < ma_top; i++) {
+    m = &malias[i];
+    if ((m->owner == executor) || (m->nflags == 0) ||
+	((m->nflags & ALIAS_ADMIN) && Hasprivs(executor)) ||
+	((m->nflags & ALIAS_MEMBERS) && ismember(m, executor))) {
+      if (count++)
+	safe_chr(sep, buff, bp);
+      safe_chr(MALIAS_TOKEN, buff, bp);
+      safe_str(m->name, buff, bp);
+    }
+  }
+}
 
 
+/** Describe a malias.
+ * \verbatim
+ * This implements the @malias/desc command.
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of the malias to describe.
+ * \param desc description to set.
+ */
 void
-do_malias_desc(player, alias, desc)
-    dbref player;
-    char *alias;
-    char *desc;
+do_malias_desc(dbref player, char *alias, char *desc)
 {
   struct mail_alias *m;
 
@@ -338,11 +414,16 @@ do_malias_desc(player, alias, desc)
 }
 
 
+/** Change ownership of a malias.
+ * \verbatim
+ * This implements the @malias/chown command.
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of the malias to chown.
+ * \param owner name of the new owner.
+ */
 void
-do_malias_chown(player, alias, owner)
-    dbref player;
-    char *alias;
-    char *owner;
+do_malias_chown(dbref player, char *alias, char *owner)
 {
   struct mail_alias *m;
   dbref no = NOTHING;
@@ -367,11 +448,16 @@ do_malias_chown(player, alias, owner)
 
 
 
+/** Change name of a malias.
+ * \verbatim
+ * This implements the @malias/rename command.
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of the malias to rename.
+ * \param newname new name for the malias.
+ */
 void
-do_malias_rename(player, alias, newname)
-    dbref player;
-    char *alias;
-    char *newname;
+do_malias_rename(dbref player, char *alias, char *newname)
 {
   struct mail_alias *m;
 
@@ -402,10 +488,15 @@ do_malias_rename(player, alias, newname)
 
 
 
+/** Delete a malias.
+ * \verbatim
+ * This implements the @malias/destroy command.
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of the malias to destroy.
+ */
 void
-do_malias_destroy(player, alias)
-    dbref player;
-    char *alias;
+do_malias_destroy(dbref player, char *alias)
 {
   struct mail_alias *m;
   m = get_malias(player, alias);
@@ -430,11 +521,16 @@ do_malias_destroy(player, alias)
 }
 
 
+/** Set the membership list for a malias.
+ * \verbatim
+ * This implements the @malias/set command.
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of the malias to set members for.
+ * \param tolist space-separated list of players to set as members.
+ */
 void
-do_malias_set(player, alias, tolist)
-    dbref player;
-    char *alias;
-    char *tolist;
+do_malias_set(dbref player, char *alias, char *tolist)
 {
   struct mail_alias *m;
   int i = 0;
@@ -528,9 +624,14 @@ do_malias_set(player, alias, tolist)
 
 
 
+/** List all maliases.
+ * \verbatim
+ * This implements the @malias/list command.
+ * \endverbatim
+ * \param player the enactor.
+ */
 void
-do_malias_all(player)
-    dbref player;
+do_malias_all(dbref player)
 {
   struct mail_alias *m;
   int i;
@@ -556,9 +657,14 @@ do_malias_all(player)
 
 
 
+/** Statistics on maliases.
+ * \verbatim
+ * This implements the @malias/stat command.
+ * \endverbatim
+ * \param player the enactor.
+ */
 void
-do_malias_stats(player)
-    dbref player;
+do_malias_stats(dbref player)
 {
   if (!Hasprivs(player))
     notify(player, T("MAIL: Permission denied."));
@@ -569,9 +675,14 @@ do_malias_stats(player)
   }
 }
 
+/** Remove all maliases.
+ * \verbatim
+ * This implements the @malias/nuke command.
+ * \endverbatim
+ * \param player the enactor.
+ */
 void
-do_malias_nuke(player)
-    dbref player;
+do_malias_nuke(dbref player)
 {
   struct mail_alias *m;
   int i;
@@ -597,13 +708,17 @@ do_malias_nuke(player)
 }
 
 
-
+/** Set permisions on maliases.
+ * \verbatim
+ * This implements @malias/use and @malias/see
+ * \endverbatim
+ * \param player the enactor.
+ * \param alias name of the malias.
+ * \param privs string of privs to set.
+ * \param type if 1, setting nprivs, if 0, mprivs.
+ */
 void
-do_malias_privs(player, alias, privs, type)
-    dbref player;
-    char *alias;
-    char *privs;
-    int type;			/* 0 = nprivs, 1 = mprivs */
+do_malias_privs(dbref player, char *alias, char *privs, int type)
 {
   struct mail_alias *m;
   int *p;
@@ -624,82 +739,13 @@ do_malias_privs(player, alias, privs, type)
 }
 
 
+/** Add players to a malias.
+ * \param player dbref of enactor.
+ * \param alias name of malias.
+ * \param tolist string with list of players to add.
+ */
 void
-do_malias_mprivs(player, alias, privs)
-    dbref player;
-    char *alias;
-    char *privs;
-{
-  const char TYPES[] = " everyone admin members owner";
-  struct mail_alias *m;
-  char *ch;
-
-  if (!(m = get_malias(player, alias))) {
-    notify(player, T("MAIL: I cannot find that alias!"));
-    return;
-  }
-  if (!Wizard(player) && (m->owner != player)) {
-    notify(player, T("MAIL: Permission denied."));
-    return;
-  }
-  if (!strchr("=+-", *privs)) {
-    notify(player,
-	   T
-	   ("MAIL: First character of permisisons must be one of '=', '+', '-',"));
-    notify(player, T("      to set, add, remove permissions, respectively."));
-    return;
-  }
-  ch = strstr(TYPES, tprintf(" %s", privs + 1));
-  if (!ch) {
-    notify_format(player, T("MAIL: Unknown permission type '%s'."), privs + 1);
-    notify(player,
-	   T
-	   ("Must be one of 'everyone', 'members', 'admin', 'owner' (can be abbreviated)."));
-    return;
-  }
-  if (*privs == '=') {		/* setting permissions */
-    if (*ch == 'e')
-      m->mflags = 0;
-    else if (*ch == 'm')
-      m->mflags = ALIAS_MEMBERS;
-    else if (*ch == 'a')
-      m->mflags = ALIAS_ADMIN;
-    else if (*ch == 'o')
-      m->mflags = ALIAS_OWNER;
-  } else if ((*privs == '+') && m->mflags) {	/* adding permissions */
-    if (*ch == 'm')
-      m->mflags |= ALIAS_MEMBERS;
-    else if (*ch == 'a')
-      m->mflags |= ALIAS_ADMIN;
-    else if (*ch == 'e') {	/* DUMMY */
-    } else if (*ch == 'o') {	/* DUMMY */
-    }
-  } else if (*privs == '-') {	/* removing permissions */
-    if (!m->mflags)
-      m->mflags = ALIAS_ADMIN | ALIAS_MEMBERS;
-    if (*ch == 'm')
-      m->mflags &= ~ALIAS_MEMBERS;
-    else if (*ch == 'a')
-      m->mflags &= ~ALIAS_ADMIN;
-    else if (*ch == 'e') {	/* DUMMY */
-    } else if (*ch == 'o') {	/* DUMMY */
-    }
-    if (!m->mflags)
-      m->mflags = ALIAS_OWNER;
-  }
-  notify_format(player,
-		T("MAIL: Permission to list alias '%s' changed to %s"),
-		alias, get_privs(m->mflags));
-}
-
-
-
-
-void
-do_malias_add(player, alias, tolist)
-    dbref player;
-    char *alias;
-    char *tolist;
+do_malias_add(dbref player, char *alias, char *tolist)
 {
   char *head, *tail, spot;
   struct mail_alias *m;
@@ -801,11 +847,13 @@ do_malias_add(player, alias, tolist)
 
 
 
+/** Remove players from a malias.
+ * \param player dbref of enactor.
+ * \param alias name of malias.
+ * \param tolist string with list of players to remove.
+ */
 void
-do_malias_remove(player, alias, tolist)
-    dbref player;
-    char *alias;
-    char *tolist;
+do_malias_remove(dbref player, char *alias, char *tolist)
 {
   char *head, *tail, spot;
   struct mail_alias *m;
@@ -890,32 +938,7 @@ do_malias_remove(player, alias, tolist)
 ***********************************************************/
 
 static const char *
-get_privs(flags)
-    int flags;
-{
-  static char privs[20];
-  privs[0] = '\0';
-  if (!flags)
-    strcpy(privs, "EVERYONE");
-  else {
-    if (flags & ALIAS_MEMBERS)
-      strcpy(privs, "MEMBERS");
-    if (flags & ALIAS_ADMIN) {
-      if (*privs)
-	strcat(privs, " | ");
-      strcat(privs, "ADMIN");
-    }
-    if (!(*privs))
-      strcpy(privs, "OWNER");
-  }
-
-  return privs;
-}
-
-
-static const char *
-get_shortprivs(m)
-    struct mail_alias *m;
+get_shortprivs(struct mail_alias *m)
 {
   static char privs[10];
   strcpy(privs, "--  -- ");
@@ -946,10 +969,14 @@ get_shortprivs(m)
 }
 
 
+/** Is a player a member of a malias?
+ * \param m pointer to malias.
+ * \param player dbref of player.
+ * \retval 1 player is a member of the malias.
+ * \retval 0 player is not a member of the malias.
+ */
 int
-ismember(m, player)
-    struct mail_alias *m;
-    dbref player;
+ismember(struct mail_alias *m, dbref player)
 {
   int i;
   for (i = 0; i < m->size; i++) {
@@ -959,10 +986,11 @@ ismember(m, player)
   return 0;
 }
 
-/* Remove a destroyed player from all maliases */
+/** Remove a destroyed player from all maliases.
+ * \param player player to remove from maliases.
+ */
 void
-malias_cleanup(player)
-    dbref player;
+malias_cleanup(dbref player)
 {
   struct mail_alias *m;
   int n, i = 0;
@@ -976,42 +1004,32 @@ malias_cleanup(player)
   }
 }
 
+/** Get a malias pointer with permission checking.
+ * \param player player dbref, for permission check.
+ * \param alias name of malias to retrieve.
+ * \return pointer to malias structure, or NULL if player can't see it.
+ */
 struct mail_alias *
-get_malias(player, alias)
-    dbref player;
-    char *alias;
+get_malias(dbref player, char *alias)
 {
   const char *mal;
   struct mail_alias *m;
   int i = 0;
-  int x = 0;
 
-  if ((*alias == '#') && Hasprivs(player)) {
-    x = atoi(alias + 1);
-    if (x < 0 || x > (ma_top - 1))
-      return NULL;
-    else
-      return &malias[x];
-  } else {
-    if (*alias != MALIAS_TOKEN)
-      return NULL;
+  if (*alias != MALIAS_TOKEN)
+    return NULL;
 
-    mal = alias + 1;
+  mal = alias + 1;
 
-    for (i = 0; i < ma_top; i++) {
-      m = &malias[i];
-      if ((m->owner == player) || (m->nflags == 0) ||
-	  /* ((m->nflags & ALIAS_ADMIN) && Hasprivs(player)) || */
-	  Hasprivs(player) ||
-	  ((m->nflags & ALIAS_MEMBERS) && ismember(m, player))) {
+  for (i = 0; i < ma_top; i++) {
+    m = &malias[i];
+    if ((m->owner == player) || (m->nflags == 0) ||
+	/* ((m->nflags & ALIAS_ADMIN) && Hasprivs(player)) || */
+	Hasprivs(player) ||
+	((m->nflags & ALIAS_MEMBERS) && ismember(m, player))) {
 
-#ifdef macintosh
-	if (!strcasecmp(mal, (char *) (m->name)))
-#else
-	if (!strcasecmp(mal, m->name))
-#endif
-	  return m;
-      }
+      if (!strcasecmp(mal, m->name))
+	return m;
     }
   }
   return NULL;
@@ -1021,17 +1039,15 @@ get_malias(player, alias)
 
 
 
-
-
-
 /***********************************************************
 ***** Loading and saving of mail-aliases *****
 ***********************************************************/
 
-
+/** Load maliases from the mail db.
+ * \param fp file pointer to read from.
+ */
 void
-load_malias(fp)
-    FILE *fp;
+load_malias(FILE * fp)
 {
   int i, j;
   char buffer[BUFFER_LEN];
@@ -1055,9 +1071,7 @@ load_malias(fp)
     m->owner = getref(fp);
     m->name = mush_strdup(getstring_noalloc(fp), "malias_name");
     m->desc = compress(getstring_noalloc(fp));
-#ifdef MEM_CHECK
     add_check("malias_desc");
-#endif
 
     m->nflags = getref(fp);
     m->mflags = getref(fp);
@@ -1080,10 +1094,11 @@ load_malias(fp)
   }
 }
 
-
+/** Write maliases to the maildb
+ * \param fp file pointer to write to.
+ */
 void
-save_malias(fp)
-    FILE *fp;
+save_malias(FILE * fp)
 {
   int i, j;
   struct mail_alias *m;
@@ -1104,28 +1119,4 @@ save_malias(fp)
       putref(fp, m->members[j]);
   }
   putstring(fp, "*** End of MALIAS ***");
-  /* putstring() of fputs() ???? 
-   * Stupid thing to do. Someone sets a malias desc of "*** End of MALIAS ***"
-   * and then that maildb is read into a mush without MAIL_ALIAS, and bad 
-   * stuff happens. It's rare, but it might happen. Will fix later. */
 }
-
-
-
-#else				/* MAIL_ALIASES */
-
-void
-load_malias(fp)			/* Skip mail aliases,           */
-    FILE *fp;			/* when they shall not be used, */
-{
-  char buffer[BUFFER_LEN];
-  char *s;
-
-  s = fgets(buffer, sizeof(buffer), fp);
-
-  while (s && strcmp(buffer, "\"*** End of MALIAS ***\"\n") != 0) {
-    s = fgets(buffer, sizeof(buffer), fp);
-  }
-}
-
-#endif				/* MAIL_ALIASES */
