@@ -1,32 +1,34 @@
+/**
+ * \file memcheck.c
+ *
+ * \brief A simple memory allocation tracker for PennMUSH.
+ *
+ * This code isn't usually compiled in, but it's handy to debug
+ * memory leaks sometimes.
+ *
+ *
+ */
 #include "config.h"
 #include "conf.h"
 #include "copyrite.h"
 
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
-
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 
-#ifdef MEM_CHECK
 
 #include "externs.h"
 #include "dbdefs.h"
-#include "memcheck.h"
 #include "mymalloc.h"
 #include "log.h"
 #include "confmagic.h"
 
 typedef struct mem_check MEM;
 
+/** A linked list for storing memory allocation counts */
 struct mem_check {
-  int ref_count;
-  MEM *next;
-  char ref_name[BUFFER_LEN];
+  int ref_count;		/**< Number of allocations of this type. */
+  MEM *next;			/**< Pointer to next in linked list. */
+  char ref_name[BUFFER_LEN];	/**< Name of this allocation type. */
 };
 
 static MEM *my_check = NULL;
@@ -35,14 +37,19 @@ static MEM *my_check = NULL;
  *** AN INFINITE LOOP. DANGER, WILL ROBINSON!
  ***/
 
+/** Add an allocation check.
+ * \param ref type of allocation.
+ */
 void
-add_check(ref)
-    const char *ref;
+add_check(const char *ref)
 {
   MEM *loop, *newcheck, *prev = NULL;
-  Size_t reflen;
+  size_t reflen;
   int cmp;
 
+  if (!options.mem_check)
+    return;
+
   for (loop = my_check; loop; loop = loop->next) {
     cmp = strcasecmp(ref, loop->ref_name);
     if (cmp == 0) {
@@ -64,13 +71,18 @@ add_check(ref)
   return;
 }
 
+/** Remove an allocation check.
+ * \param ref type of allocation to remove.
+ */
 void
-del_check(ref)
-    const char *ref;
+del_check(const char *ref)
 {
   MEM *loop, *prev = NULL;
   int cmp;
 
+  if (!options.mem_check)
+    return;
+
   for (loop = my_check; loop; loop = loop->next) {
     cmp = strcasecmp(ref, loop->ref_name);
     if (cmp == 0) {
@@ -92,27 +104,33 @@ del_check(ref)
 	    ref);
 }
 
+/** List allocations.
+ * \param player the enactor.
+ */
 void
-list_mem_check(player)
-    dbref player;
+list_mem_check(dbref player)
 {
-
   MEM *loop;
 
+  if (!options.mem_check)
+    return;
   for (loop = my_check; loop; loop = loop->next) {
     notify_format(player, "%s : %d", loop->ref_name, loop->ref_count);
   }
 }
 
+/** Log all allocations.
+ */
 void
-log_mem_check()
+log_mem_check(void)
 {
-
   MEM *loop;
 
+  if (!options.mem_check)
+    return;
+  do_rawlog(LT_CHECK, "MEMCHECK dump starts");
   for (loop = my_check; loop; loop = loop->next) {
     do_rawlog(LT_CHECK, "%s : %d", loop->ref_name, loop->ref_count);
   }
+  do_rawlog(LT_CHECK, "MEMCHECK dump ends");
 }
-
-#endif				/* MEM_CHECK */
