@@ -400,7 +400,8 @@ do_move(dbref player, const char *direction, enum move_type type)
       /* we got one */
       /* check to see if we're allowed to pass */
       if (!eval_lock(player, Location(player), Leave_Lock)) {
-	notify(player, T("You can't go that way."));
+	fail_lock(player, Location(player), Leave_Lock,
+		  T("You can't go that way."), NOTHING);
 	return;
       }
 
