@@ -1133,7 +1133,7 @@ process_expression(char *buff, char **bp, char const **str,
 		for (logi = 0; logi < nfargs; logi++) {
 		  safe_str(fargs[logi], logstr, &logp);
 		  if (logi + 1 < nfargs)
-		    safe_str(fargs[logi], logstr, &logp);
+		    safe_chr(',', logstr, &logp);
 		}
 		safe_chr(')', logstr, &logp);
 		*logp = '\0';
