@@ -1,4 +1,10 @@
-/* player.c */
+/**
+ * \file player.c
+ *
+ * \brief Player creation and connection for PennMUSH.
+ *
+ *
+ */
 
 #include "copyrite.h"
 #include "config.h"
@@ -6,14 +12,8 @@
 #ifdef I_UNISTD
 #include <unistd.h>
 #endif
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #ifdef I_SYS_TIME
 #include <sys/time.h>
 #else
@@ -25,17 +25,18 @@
 #include <fcntl.h>
 
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "attrib.h"
-#include "externs.h"
 #include "access.h"
 #include "mymalloc.h"
 #include "log.h"
 #include "dbdefs.h"
 #include "flags.h"
 #include "lock.h"
+#include "parse.h"
 
-#if (CRYPT_SYSTEM == 1) || (CRYPT_SYSTEM == 3)
+#ifdef HAS_CRYPT
 #ifdef I_CRYPT
 #include <crypt.h>
 #else
@@ -46,78 +47,82 @@ extern char *crypt(const char *, const char *);
 #include "extmail.h"
 #include "confmagic.h"
 
-extern int reserved;
-
 dbref email_register_player
   (const char *name, const char *email, const char *host, const char *ip);
 static dbref make_player
   (const char *name, const char *password, const char *host, const char *ip);
-void do_password(dbref player, const char *old, const char *newobj);
+void do_password
+  (dbref player, dbref cause, const char *old, const char *newobj);
 
 static const char pword_attr[] = "XYXXY";
 
 extern struct db_stat_info current_state;
 
+/** Check a player's password against a given string.
+ * \param player dbref of player.
+ * \param password plaintext password string to check.
+ * \retval 1 password matches (or player has no password).
+ * \retval 0 password fails to match.
+ */
 int
-password_check(player, password)
-    dbref player;
-    const char *password;
+password_check(dbref player, const char *password)
 {
   ATTR *a;
   char *saved;
+  char *passwd;
 
   /* read the password and compare it */
   if (!(a = atr_get_noparent(player, pword_attr)))
     return 1;			/* No password attribute */
 
-  saved = strdup(uncompress(a->value));
+  saved = strdup(atr_value(a));
 
   if (!saved)
     return 0;
 
-  if (strcmp(mush_crypt(password), saved) != 0) {
-#if (CRYPT_SYSTEM == 3) || (CRYPT_SYSTEM == 1)
-    /* shs encryption didn't match. Try crypt(3) */
-    if (strcmp(crypt(password, "XX"), saved) != 0) {
-      /* crypt(3) didn't match either. */
-      free(saved);
-      return 0;
-    }
-    /* crypt(3) worked. Change password to SHS-encrypted */
-    (void) atr_add(player, pword_attr, mush_crypt(password), GOD, NOTHING);
-#else
-#if (CRYPT_SYSTEM == 4)
-    /* prevent direct entry of the raw encrypted password */
-    if ((strlen(password) >= 4) && (password[0] == 'X') && (password[1] == 'X')) {
-      free(saved);
-      return 0;
-    }
+  passwd = mush_crypt(password);
 
-    if (strcmp(saved, password) != 0) {
-      free(saved);
-      return 0;			/* Plaintext didn't match */
-    }
-    (void) atr_add(player, pword_attr, mush_crypt(password), GOD, NOTHING);
-#else
-    free(saved);
-    return 0;
-#endif				/* CRYPT_SYSTEM == 4 */
-#endif				/* CRYPT_SYSTEM == 3 or 1 */
+  if (strcmp(passwd, saved) != 0) {
+    /* Nope. Try non-SHS. */
+#ifdef HAS_CRYPT
+    /* shs encryption didn't match. Try crypt(3) */
+    if (strcmp(crypt(password, "XX"), saved) != 0)
+      /* Nope */
+#endif				/* HAS_CRYPT */
+      /* crypt() didn't work. Try plaintext, being sure to not 
+       * allow unencrypted entry of encrypted password */
+      if ((strcmp(saved, password) != 0)
+	  || (strlen(password) < 4)
+	  || ((password[0] == 'X') && (password[1] == 'X'))) {
+	/* Nothing worked. You lose. */
+	free(saved);
+	return 0;
+      }
+    /* Something worked. Change password to SHS-encrypted */
+    (void) atr_add(player, pword_attr, passwd, GOD, NOTHING);
   }
   free(saved);
-  /* we're okay */
   return 1;
 }
 
+/** Check to see if someone can connect to a player.
+ * \param name name of player to connect to.
+ * \param password password of player to connect to.
+ * \param host host from which connection is being attempted.
+ * \param ip ip address from which connection is being attempted.
+ * \param errbuf buffer to return connection errors.
+ * \return dbref of connected player object or NOTHING for failure
+ * (with reason for failure returned in errbuf).
+ */
 dbref
-connect_player(name, password, host, ip)
-    const char *name;
-    const char *password;
-    const char *host;
-    const char *ip;
+connect_player(const char *name, const char *password, const char *host,
+	       const char *ip, char *errbuf)
 {
   dbref player;
-  dbref i;
+
+  /* Default error */
+  strcpy(errbuf,
+	 T("Either that player does not exist, or has a different password."));
 
   if (!name || !*name)
     return NOTHING;
@@ -138,6 +143,7 @@ connect_player(name, password, host, ip)
       do_log(LT_CONN, 0, 0,
 	     T("Connection to %s (GUEST) not allowed from %s (%s)"), name,
 	     host, ip);
+      strcpy(errbuf, T("Guest connections not allowed."));
     }
     return NOTHING;
   } else if (!Guest(player)
@@ -147,54 +153,51 @@ connect_player(name, password, host, ip)
       do_log(LT_CONN, 0, 0,
 	     T("Connection to %s (Non-GUEST) not allowed from %s (%s)"), name,
 	     host, ip);
+      strcpy(errbuf, T("Player connections not allowed."));
     }
     return NOTHING;
   }
   /* validate password */
   if (!Guest(player))
     if (!password_check(player, password)) {
-#ifdef CREATION_TIMES
       /* Increment count of login failures */
       ModTime(player)++;
-#endif
       check_lastfailed(player, host);
       return NOTHING;
     }
   /* If it's a Guest player, and already connected, search the
    * db for another Guest player to connect them to. */
-  if (Guest(player) && Connected(player)) {
-    /* Search db for an unconnected Guest */
-    for (i = 0; i < db_top; i++) {
-      if (IsPlayer(i) && !Hasprivs(i) && Guest(i) && !Connected(i)) {
-	player = i;
-	break;
-      }
-    }
-    if (i == db_top) {
-      /* We failed to find an unconnected Guest */
-      do_log(LT_CONN, 0, 0, T("Multiple connection to Guest #%d"), player);
+  if (Guest(player)) {
+    /* Enforce guest limit */
+    player = guest_to_connect(player);
+    if (!GoodObject(player)) {
+      do_log(LT_CONN, 0, 0, T("Can't connect to a guest (too many connected)"));
+      strcpy(errbuf, T("Too many guests are connected now."));
+      return NOTHING;
     }
   }
   if (Suspect_Site(host, player) || Suspect_Site(ip, player)) {
     do_log(LT_CONN, 0, 0,
 	   T("Connection from Suspect site. Setting %s(#%d) suspect."),
 	   Name(player), player);
-    Toggles(player) |= PLAYER_SUSPECT;
+    set_flag_internal(player, "SUSPECT");
   }
   return player;
 }
 
-/* This function returns NOTHING if the player name was bad, and
- * AMBIGUOUS if the player password was bad
+/** Attempt to create a new player object.
+ * \param name name of player to create.
+ * \param password initial password of created player.
+ * \param host host from which creation is attempted.
+ * \param ip ip address from which creation is attempted.
+ * \return dbref of created player, NOTHING if bad name, AMBIGUOUS if bad
+ *  password.
  */
 dbref
-create_player(name, password, host, ip)
-    const char *name;
-    const char *password;
-    const char *host;
-    const char *ip;
+create_player(const char *name, const char *password, const char *host,
+	      const char *ip)
 {
-  if (!ok_player_name(name)) {
+  if (!ok_player_name(name, NOTHING)) {
     do_log(LT_CONN, 0, 0, T("Failed creation (bad name) from %s"), host);
     return NOTHING;
   }
@@ -220,23 +223,32 @@ create_player(name, password, host, ip)
 #endif
 
 #ifdef HAS_SENDMAIL
+
+/** Size of the elems array */
+#define NELEMS (sizeof(elems)-1)
+
+/** Attempt to register a new player at the connect screen.
+ * If registration is allowed, a new player object is created with
+ * a random password which is emailed to the registering player.
+ * \param name name of player to register.
+ * \param email email address to send registration details.
+ * \param host host from which registration is being attempted.
+ * \param ip ip address from which registration is being attempted.
+ * \return dbref of created player or NOTHING if creation failed.
+ */
 dbref
-email_register_player(name, email, host, ip)
-    const char *name;
-    const char *email;
-    const char *host;
-    const char *ip;
+email_register_player(const char *name, const char *email, const char *host,
+		      const char *ip)
 {
   char *p;
   char passwd[BUFFER_LEN];
   static char elems[] =
     "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";
-#define NELEMS (sizeof(elems)-1)
   int i, len;
   dbref player;
   FILE *fp;
 
-  if (!ok_player_name(name)) {
+  if (!ok_player_name(name, NOTHING)) {
     do_log(LT_CONN, 0, 0, T("Failed registration (bad name) from %s"), host);
     return NOTHING;
   }
@@ -297,9 +309,7 @@ email_register_player(name, email, host, ip)
    *  1) We'll use sendmail -t, so we don't pass user-given values to a shell.
    */
 
-#ifndef WIN32
-  close(reserved);
-#endif
+  release_fd();
   if ((fp =
 #ifdef __LCC__
        (FILE *)
@@ -308,9 +318,7 @@ email_register_player(name, email, host, ip)
     do_log(LT_CONN, 0, 0,
 	   T("Failed registration of %s by %s: unable to open sendmail"),
 	   name, email);
-#ifndef WIN32
-    reserved = open("/dev/null", O_RDWR);
-#endif
+    reserve_fd();
     return NOTHING;
   }
   fprintf(fp, T("Subject: [%s] Registration of %s\n"), MUDNAME, name);
@@ -324,12 +332,10 @@ email_register_player(name, email, host, ip)
   fprintf(fp, "\n");
   fprintf(fp, T("To access this character, connect to %s and type:\n"),
 	  MUDNAME);
-  fprintf(fp, "\tconnect %s %s\n", name, passwd);
+  fprintf(fp, "\tconnect \"%s\" %s\n", name, passwd);
   fprintf(fp, "\n");
   pclose(fp);
-#ifndef WIN32
-  reserved = open("/dev/null", O_RDWR);
-#endif
+  reserve_fd();
   /* Ok, all's well, make a player */
   player = make_player(name, passwd, host, ip);
   (void) atr_add(player, "REGISTERED_EMAIL", email, GOD, NOTHING);
@@ -337,11 +343,8 @@ email_register_player(name, email, host, ip)
 }
 #else
 dbref
-email_register_player(name, email, host, ip)
-    const char *name;
-    const char *email;
-    const char *host;
-    const char *ip;
+email_register_player(const char *name, const char *email, const char *host,
+		      const char *ip)
 {
   do_log(LT_CONN, 0, 0, T("Failed registration (no sendmail) from %s"), host);
   return NOTHING;
@@ -349,24 +352,14 @@ email_register_player(name, email, host, ip)
 #endif
 
 static dbref
-make_player(name, password, host, ip)
-    const char *name;
-    const char *password;
-    const char *host;
-    const char *ip;
+make_player(const char *name, const char *password, const char *host,
+	    const char *ip)
 {
 
   dbref player;
-  char *s;
-#ifdef QUOTA
   char temp[SBUF_LEN];
-#endif
+  object_flag_type flags;
 
-  /* else he doesn't already exist, create him */
-  s = ctime(&mudtime);
-  s[strlen(s) - 1] = '\0';
-  if (s[8] == ' ')
-    s[8] = '0';
   player = new_object();
 
   /* initialize everything */
@@ -375,44 +368,28 @@ make_player(name, password, host, ip)
   Home(player) = PLAYER_START;
   Owner(player) = player;
   Parent(player) = NOTHING;
-  Flags(player) = TYPE_PLAYER;
-  Flags(player) |= options.player_flags;
-  Toggles(player) |= options.player_toggles;
+  Type(player) = TYPE_PLAYER;
+  flags = string_to_bits("FLAG", options.player_flags);
+  copy_flag_bitmask("FLAG", Flags(player), flags);
+  destroy_flag_bitmask(flags);
   if (Suspect_Site(host, player) || Suspect_Site(ip, player))
-    Toggles(player) |= PLAYER_SUSPECT;
-#ifdef USE_WARNINGS
+    set_flag_internal(player, "SUSPECT");
   set_initial_warnings(player);
-#endif
-#ifdef CREATION_TIMES
   /* Modtime tracks login failures */
   ModTime(player) = (time_t) 0;
-#endif
   (void) atr_add(player, "XYXXY", mush_crypt(password), GOD, NOTHING);
   giveto(player, START_BONUS);	/* starting bonus */
-  (void) atr_add(player, "LAST", s, GOD, NOTHING);
+  (void) atr_add(player, "LAST", show_time(mudtime, 0), GOD, NOTHING);
   (void) atr_add(player, "LASTSITE", host, GOD, NOTHING);
   (void) atr_add(player, "LASTIP", ip, GOD, NOTHING);
   (void) atr_add(player, "LASTFAILED", " ", GOD, NOTHING);
-#ifdef QUOTA
   sprintf(temp, "%d", START_QUOTA);
   (void) atr_add(player, "RQUOTA", temp, GOD, NOTHING);
-#endif				/* QUOTA */
-#ifdef FIXED_FLAG
-#ifndef EMPTY_ATTRS
-
-  (void) atr_add(player, "ICLOC", " ", GOD,
+  (void) atr_add(player, "ICLOC", EMPTY_ATTRS ? "" : " ", GOD,
 		 AF_MDARK | AF_PRIVATE | AF_WIZARD | AF_NOCOPY);
-#else
-
-  (void) atr_add(player, "ICLOC", "", GOD,
-		 AF_MDARK | AF_PRIVATE | AF_WIZARD | AF_NOCOPY);
-#endif
-#endif
-#ifdef USE_MAILER
   (void) atr_add(player, "MAILCURF", "0", GOD,
 		 AF_LOCKED | AF_NOPROG | AF_WIZARD);
   add_folder_name(player, 0, "inbox");
-#endif
   /* link him to PLAYER_START */
   PUSH(player, Contents(PLAYER_START));
 
@@ -425,35 +402,64 @@ make_player(name, password, host, ip)
 
   current_state.players++;
 
-#ifdef LOCAL_DATA
   local_data_create(player);
-#endif
 
   return player;
 }
 
 
+/** Change a player's password.
+ * \verbatim
+ * This function implements @password.
+ * \endverbatim
+ * \param player the executor.
+ * \param cause the enactor.
+ * \param old player's current password.
+ * \param newobj player's desired new password.
+ */
 void
-do_password(player, old, newobj)
-    dbref player;
-    const char *old;
-    const char *newobj;
+do_password(dbref player, dbref cause, const char *old, const char *newobj)
 {
+  if (!global_eval_context.process_command_port) {
+    char old_eval[BUFFER_LEN];
+    char new_eval[BUFFER_LEN];
+    char const *sp;
+    char *bp;
+
+    sp = old;
+    bp = old_eval;
+    process_expression(old_eval, &bp, &sp, player, player, cause,
+		       PE_DEFAULT, PT_DEFAULT, NULL);
+    *bp = '\0';
+    old = old_eval;
+
+    sp = newobj;
+    bp = new_eval;
+    process_expression(new_eval, &bp, &sp, player, player, cause,
+		       PE_DEFAULT, PT_DEFAULT, NULL);
+    *bp = '\0';
+    newobj = new_eval;
+  }
+
   if (!password_check(player, old)) {
-    notify(player, T("Suffering from memory loss? See a wizard!"));
+    notify(player, T("The old password that you entered was incorrect."));
   } else if (!ok_password(newobj)) {
     notify(player, T("Bad new password."));
   } else {
     (void) atr_add(player, "XYXXY", mush_crypt(newobj), GOD, NOTHING);
-    notify(player, T("Password changed."));
+    notify(player, T("You have changed your password."));
   }
 }
 
+/** Processing related to players' last connections.
+ * Here we check to see if a player gets a paycheck, tell them their
+ * last connection site, and update all their LAST* attributes.
+ * \param player dbref of player.
+ * \param host hostname of player's current connection.
+ * \param ip ip address of player's current connection.
+ */
 void
-check_last(player, host, ip)
-    dbref player;
-    const char *host;
-    const char *ip;
+check_last(dbref player, const char *host, const char *ip)
 {
   char *s;
   ATTR *a;
@@ -461,27 +467,24 @@ check_last(player, host, ip)
   char last_time[MAX_COMMAND_LEN / 8];
   char last_place[MAX_COMMAND_LEN];
 
-  s = ctime(&mudtime);
-  s[strlen(s) - 1] = '\0';
-  if (s[8] == ' ')
-    s[8] = '0';
   /* compare to last connect see if player gets salary */
+  s = show_time(mudtime, 0);
   a = atr_get_noparent(player, "LAST");
-  if (a && (strncmp(uncompress(a->value), s, 10) != 0))
+  if (a && (strncmp(atr_value(a), s, 10) != 0))
     giveto(player, Paycheck(player));
   /* tell the player where he last connected from */
   if (!Guest(player)) {
     h = atr_get_noparent(player, "LASTSITE");
     if (h && a) {
-      strcpy(last_place, uncompress(h->value));
-      strcpy(last_time, uncompress(a->value));
+      strcpy(last_place, atr_value(h));
+      strcpy(last_time, atr_value(a));
       notify_format(player, T("Last connect was from %s on %s."),
 		    last_place, last_time);
     }
     /* How about last failed connection */
     h = atr_get_noparent(player, "LASTFAILED");
     if (h && a) {
-      strcpy(last_place, uncompress(h->value));
+      strcpy(last_place, atr_value(h));
       if (strlen(last_place) > 2)
 	notify_format(player, T("Last FAILED connect was from %s."),
 		      last_place);
@@ -500,19 +503,17 @@ check_last(player, host, ip)
 }
 
 
-/* This is called on a failed connection */
+/** Update the LASTFAILED attribute on a failed connection.
+ * \param player dbref of player.
+ * \param host host from which connection attempt failed.
+ */
 void
-check_lastfailed(player, host)
-    dbref player;
-    const char *host;
+check_lastfailed(dbref player, const char *host)
 {
-  char *s;
   char last_place[BUFFER_LEN], *bp;
 
-  s = ctime(&mudtime);
-  s[strlen(s) - 1] = '\0';
   bp = last_place;
-  safe_format(last_place, &bp, T("%s on %s"), host, s);
+  safe_format(last_place, &bp, T("%s on %s"), host, show_time(mudtime, 0));
   *bp = '\0';
   (void) atr_add(player, "LASTFAILED", last_place, GOD, NOTHING);
 }
