@@ -1,5 +1,11 @@
+/**
+ * \file plyrlist.c
+ *
+ * \brief Player list management for PennMUSH.
+ *
+ *
+ */
 
-/* plyrlist.c */
 #include "config.h"
 
 #include <ctype.h>
@@ -9,28 +15,30 @@
 #include "copyrite.h"
 
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "dbdefs.h"
 #include "flags.h"
-#include "externs.h"
 #include "htab.h"
 #include "confmagic.h"
 
 
+/** Hash table of player names */
 HASHTAB htab_player_list;
 
 static int hft_initialized = 0;
 static void init_hft(void);
 
 static void
-init_hft()
+init_hft(void)
 {
   hashinit(&htab_player_list, 256, sizeof(dbref));
   hft_initialized = 1;
 }
 
+/** Clear the player list htab. */
 void
-clear_players()
+clear_players(void)
 {
   if (hft_initialized)
     hashflush(&htab_player_list, 256);
@@ -39,6 +47,10 @@ clear_players()
 }
 
 
+/** Add a player to the player list htab.
+ * \param player dbref of player to add.
+ * \param alias name to use as hash table key for player, if given.
+ */
 void
 add_player(dbref player, const char *alias)
 {
@@ -50,6 +62,10 @@ add_player(dbref player, const char *alias)
     hashadd(strupper(Name(player)), (void *) player, &htab_player_list);
 }
 
+/** Look up a player in the player list htab (or by dbref).
+ * \param name name of player to find.
+ * \return dbref of player, or NOTHING.
+ */
 dbref
 lookup_player(const char *name)
 {
@@ -80,6 +96,10 @@ lookup_player(const char *name)
    */
 }
 
+/** Remove a player from the player list htab.
+ * \param player dbref of player to remove.
+ * \param alias key to remove if given.
+ */
 void
 delete_player(dbref player, const char *alias)
 {
