@@ -98,8 +98,7 @@ static Signal_t lostconn(int sig);
 static int how_many_fds(void);
 
 static Signal_t
-wait_on_child(sig)
-    int sig;
+wait_on_child(int sig __attribute__ ((__unused__)))
 {
   WAIT_TYPE status;
 
@@ -118,17 +117,13 @@ wait_on_child(sig)
 }
 
 Signal_t
-lostconn(sig)
-    int
-     sig;
+lostconn(int sig __attribute__ ((__unused__)))
 {
   exit(1);
 }
 
 int
-main(argc, argv)
-    int argc;
-    char **argv;
+main(int argc, char **argv)
 {
   int msgfd, fd, n;
   struct stat statBuf;
@@ -196,7 +191,7 @@ main(argc, argv)
 #endif
 
   signal(SIGCLD, (void *) wait_on_child);
-  bzero((char *) &tcp_srv_addr, sizeof(tcp_srv_addr));
+  memset((char *) &tcp_srv_addr, 0, sizeof(tcp_srv_addr));
   tcp_srv_addr.sin_family = AF_INET;
 #ifdef SINGLE_IP_ADDR
   tcp_srv_addr.sin_addr.s_addr = inet_addr(MUSH_IP_ADDR);
@@ -256,7 +251,7 @@ main_again:
 }
 
 static int
-how_many_fds()
+how_many_fds(void)
 {
   /* Determine how many open file descriptors we're allowed
    * In order, we'll try:
