@@ -1,32 +1,35 @@
-/* privtab.c */
-/* Functions to work with privilege tables */
+/**
+ * \file privtab.c
+ *
+ * \brief Privilege tables for PennMUSH.
+ *
+ * A privilege table is a respresentation of different privilege
+ * flags with associated names, characters, and bitmasks.
+ *
+ */
 
 #include "copyrite.h"
 #include "config.h"
 #include <ctype.h>
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #include "conf.h"
 #include "privtab.h"
 #include "externs.h"
 #include "confmagic.h"
 
 
-/* Given a privs table, a string, and an original set of privileges,
+/** Convert a string to a set of privilege bits, masked by an original set.
+ * Given a privs table, a string, and an original set of privileges,
  * return a modified set of privileges by applying the privs in the
- * string to the original set of privileges
+ * string to the original set of privileges.
+ * \param table pointer to a privtab.
+ * \param str a space-separated string of privilege names to apply.
+ * \param origprivs the original privileges.
+ * \return a privilege bitmask.
  */
 int
-string_to_privs(table, str, origprivs)
-    PRIV *table;
-    const char *str;
-    long int origprivs;
+string_to_privs(PRIV *table, const char *str, long int origprivs)
 {
   PRIV *c;
   long int yes = 0;
@@ -65,15 +68,17 @@ string_to_privs(table, str, origprivs)
   return ((origprivs | yes) & ~no);
 }
 
-/* Given a privs table, a string of letters, and an original set of privileges,
+/** Convert a letter string to a set of privilege bits, masked by an original set.
+ * Given a privs table, a letter string, and an original set of privileges,
  * return a modified set of privileges by applying the privs in the
- * letters to the original set of privileges
+ * string to the original set of privileges.
+ * \param table pointer to a privtab.
+ * \param str a string of privilege letters to apply.
+ * \param origprivs the original privileges.
+ * \return a privilege bitmask.
  */
 int
-letter_to_privs(table, str, origprivs)
-    PRIV *table;
-    const char *str;
-    long int origprivs;
+letter_to_privs(PRIV *table, const char *str, long int origprivs)
 {
   PRIV *c;
   long int yes = 0, no = 0;
@@ -103,11 +108,13 @@ letter_to_privs(table, str, origprivs)
   return ((origprivs | yes) & ~no);
 }
 
-/* Given a table and a bitmask, return a privs string (static allocation) */
+/** Given a table and a bitmask, return a privs string (static allocation).
+ * \param table pointer to a privtab.
+ * \param privs bitmask of privileges.
+ * \return statically allocated space-separated string of priv names.
+ */
 const char *
-privs_to_string(table, privs)
-    PRIV *table;
-    int privs;
+privs_to_string(PRIV *table, int privs)
 {
   PRIV *c;
   static char buf[BUFFER_LEN];
@@ -127,12 +134,13 @@ privs_to_string(table, privs)
 }
 
 
-/* Given a table and a bitmask, return a privs letter string 
- * (static allocation) */
+/** Given a table and a bitmask, return a privs letter string (static allocation).
+ * \param table pointer to a privtab.
+ * \param privs bitmask of privileges.
+ * \return statically allocated string of priv letters.
+ */
 const char *
-privs_to_letters(table, privs)
-    PRIV *table;
-    int privs;
+privs_to_letters(PRIV *table, int privs)
 {
   PRIV *c;
   static char buf[BUFFER_LEN];
