@@ -9,9 +9,7 @@
 
 #include <windows.h>		/* for service and thread routines */
 
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
 #include <process.h>
 #include <direct.h>
 
@@ -23,6 +21,8 @@
 #include "mymalloc.h"
 #include "confmagic.h"
 
+#ifdef WIN32SERVICES
+
 #define THIS_SERVICE "PennMUSH"
 #define THIS_SERVICE_DISPLAY "PennMUSH for Win32"
 
@@ -363,7 +363,7 @@ shut_down_handler(DWORD dwCtrlType)
       TerminateThread(threadHandle, 1);
     threadHandle = NULL;
 
-    panic(T("System shutdown by system operator"));
+    mush_panic(T("System shutdown by system operator"));
 
     _exit(99);
 
@@ -830,4 +830,5 @@ convert_error(DWORD error)
 }				/*  end of convert_error */
 
 
+#endif				/* WIN32SERVICES */
 #endif				/* WIN32 */
