@@ -613,8 +613,6 @@ do_set(dbref player, const char *name, char *flag)
     notify(player, T("Only God can set himself!"));
     return 0;
   }
-  her = Hearer(thing);
-  listener = Listener(thing);
   /* check for attribute set first */
   if ((p = strchr(flag, ':')) != NULL) {
     *p++ = '\0';
@@ -632,6 +630,8 @@ do_set(dbref player, const char *name, char *flag)
     return 0;
   }
   do {
+    her = Hearer(thing);	/* Must be in loop, can change! */
+    listener = Listener(thing);	/* Must be in loop, can change! */
     f = split_token(&p, ' ');
     negate = 0;
     if (*f == NOT_TOKEN && *(f + 1)) {
@@ -1124,7 +1124,7 @@ do_wipe(dbref player, char *name)
   we_are_wiping = 1;
 
   if (!atr_iter_get(player, thing, pattern, 0, wipe_helper, NULL))
-    notify(player, T("No matching attributes."));
+    notify(player, T("No attributes wiped."));
   else
     notify(player, T("Attributes wiped."));
 
