@@ -1,38 +1,32 @@
-/* ------------------------ SHS.C ----------------------------- */
-
-/* Secure Hash Algorithm */
-
-/* 
-
-   Written 2 September 1992. Peter C. Gutmann.
-   This implementation placed in the public domain.
-
-   Comments to pgutl@cs.aukuni.ac.nz  
-
-   Keyed in from: "Applied Cryptography by Bruce Schneier"
-
-   by Nick Gammon.
-
-
-   Note: include files below are slightly different from the original source
-   in order to confirm to the MUSH include files standards.
-
+/**
+ * \file shs.c
+ *
+ * \brief The secure hash algorithm.
+ *
+ * Written 2 September 1992. Peter C. Gutmann.
+ * This implementation placed in the public domain.
+ *
+ * Comments to pgutl@cs.aukuni.ac.nz  
+ *
+ * Keyed in from: "Applied Cryptography by Bruce Schneier"
+ *
+ * by Nick Gammon.
+ *
+ * Note: include files below are slightly different from the original source
+ * in order to confirm to the MUSH include files standards.
  */
 
-
 #include "copyrite.h"
 #include "config.h"
 
-#ifdef I_STRING
+#ifndef HAS_OPENSSL
+
 #include <string.h>
-#else
-#include <strings.h>
-#endif
-
 #include "shs.h"
-
 #include "confmagic.h"
 
+#ifndef DOXYGEN_SHOULD_SKIP_THIS
+
 /* The SHS f() functions */
 
 #define f1(x, y, z) ( (x & y) | (~x & z) )	/* rounds 0-19 */
@@ -114,14 +108,17 @@
 LONG h0, h1, h2, h3, h4;
 LONG A, B, C, D, E;
 
+#endif				/* DOXYGEN_SHOULD_SKIP_THIS */
+
 /* Initialise the SHS values */
 
+/** Initialize SHS values. 
+ * \param shsInfo pointer to shs data structure.
+ */
 void
-shsInit(shsInfo)
-    SHS_INFO *shsInfo;
+shsInit(SHS_INFO *shsInfo)
 {
   /* Set the h-vars to their initial values */
-
   shsInfo->digest[0] = h0init;
   shsInfo->digest[1] = h1init;
   shsInfo->digest[2] = h2init;
@@ -129,19 +126,19 @@ shsInit(shsInfo)
   shsInfo->digest[4] = h4init;
 
   /* Initialise bit count */
-
   shsInfo->countLo = shsInfo->countHi = 0;
 
-}				/* end of shsInit */
+}
 
-/* Perform the SHS transformation. Note that this code, like MD5, seems to break */
-/* some optimizing compilers - it may be necessary to split it into sections, e.g. */
-/* based on the four subrounds */
 
-static void shsTransform(SHS_INFO * shsInfo);
+static void shsTransform(SHS_INFO *shsInfo);
 
+/* Perform the SHS transformation. Note that this code, like MD5, seems to
+ * break some optimizing compilers - it may be necessary to split it into 
+ * sections, e.g. based on the four subrounds.
+ */
 static void
-shsTransform(SHS_INFO * shsInfo)
+shsTransform(SHS_INFO *shsInfo)
 {
   LONG W[80], temp;
   int i;
@@ -335,12 +332,16 @@ byteReverse(LONG * buffer, int byteCount)
 }				/* end of byteReverse */
 
 
-/* Update SHS for a block of data. This code assumes that the buffer size is a multiple */
-/* of SHS_BLOCKSIZE bytes long, which makes the code a lot more efficient since it does */
-/* away with the need to handle partial blocks between calls to shsUpdate () */
-
+/** Update SHS for a block of data. This code assumes that the buffer
+ * size is a multiple of SHS_BLOCKSIZE bytes long, which makes the
+ * code a lot more efficient since it does away with the need to
+ * handle partial blocks between calls to shsUpdate() 
+ * \param shsInfo pointer to shs data.
+ * \param buffer block of data to update the hash with.
+ * \param count size of buffer in bytes.
+ */
 void
-shsUpdate(SHS_INFO * shsInfo, const BYTE * buffer, int count)
+shsUpdate(SHS_INFO *shsInfo, const BYTE * buffer, int count)
 {
   /* Update bitcount */
 
@@ -365,10 +366,13 @@ shsUpdate(SHS_INFO * shsInfo, const BYTE * buffer, int count)
 
   memcpy(shsInfo->data, buffer, count);
 
-}				/* end of shsUpdate */
+}
 
+/** Finalize the SHS hash.
+ * \param shsInfo pointer to shs data.
+ */
 void
-shsFinal(SHS_INFO * shsInfo)
+shsFinal(SHS_INFO *shsInfo)
 {
   int count;
   LONG lowBitcount = shsInfo->countLo, highBitcount = shsInfo->countHi;
@@ -415,4 +419,6 @@ shsFinal(SHS_INFO * shsInfo)
 
   if (shsInfo->reverse_wanted)
     byteReverse(shsInfo->data, SHS_DIGESTSIZE);
-}				/* end of shsFinal */
+}
+
+#endif				/* HAS_OPENSSL */
