@@ -1,22 +1,32 @@
-/* Signal handling routines for Penn. */
+/**
+ * \file sig.c
+ *
+ * \brief Signal handling routines for PennMUSH.
+ *
+ *
+ */
 
 #include "config.h"
-#include "externs.h"
 #include <signal.h>
-
+#include "conf.h"
+#include "externs.h"
 #include "confmagic.h"
 
 #ifndef HAS_SIGPROCMASK
 static Sigfunc saved_handlers[NSIG];
 #endif
 
-/* We're going to rewrite the signal() function in terms of
+/** Our own version of signal().
+ * We're going to rewrite the signal() function in terms of
  * sigaction, where available, to ensure consistent semantics.
  * We want signal handlers to remain installed, and we want
  * signals (except SIGALRM) to restart system calls which they
  * interrupt. This is how bsd signals work, and what we'd like.
  * This function is essentially example 10.12 from Stevens'
- * _Advanced Programming in the Unix Environment_
+ * _Advanced Programming in the Unix Environment_.
+ * \param signo signal number.
+ * \param func signal handler function to install.
+ * \return former signal handler for signo.
  */
 Sigfunc
 install_sig_handler(int signo, Sigfunc func)
@@ -37,6 +47,10 @@ install_sig_handler(int signo, Sigfunc func)
 #endif
 }
 
+/** Reinstall a signal handler.
+ * \param signo the signal number.
+ * \param func signal handler function to reload.
+ */
 void
 reload_sig_handler(int signo __attribute__ ((__unused__)),
 		   Sigfunc func __attribute__ ((__unused__)))
@@ -46,6 +60,9 @@ reload_sig_handler(int signo __attribute__ ((__unused__)),
 #endif
 }
 
+/** Set a signal to be ignored.
+ * \param signo signal number to ignore.
+ */
 void
 ignore_signal(int signo)
 {
@@ -60,7 +77,10 @@ ignore_signal(int signo)
 #endif
 }
 
-/* These don't really work right without sigprocmask(), but we try */
+/** Set a signal to block.
+ * These don't really work right without sigprocmask(), but we try.
+ * \param signo signal number to block.
+ */
 void
 block_a_signal(int signo)
 {
@@ -75,6 +95,10 @@ block_a_signal(int signo)
 #endif
 }
 
+/** Unblock a signal.
+ * These don't really work right without sigprocmask(), but we try.
+ * \param signo signal number to unblock.
+ */
 void
 unblock_a_signal(int signo)
 {
@@ -91,6 +115,8 @@ unblock_a_signal(int signo)
 #endif
 }
 
+/** Block all signals.
+ */
 void
 block_signals(void)
 {
