@@ -365,8 +365,8 @@ do_whisper(dbref player, const char *arg1, const char *arg2, int noisy)
   *tp = '\0';
 
   if (key == 1) {
-    notify_format(player, T("%s senses: %s%s%s"),
-		  tbuf + 4, Name(player), gap, arg2);
+    notify_format(player, (gcount > 1) ? T("%s sense: %s%s%s") :
+		  T("%s senses: %s%s%s"), tbuf + 4, Name(player), gap, arg2);
     p = tprintf("You sense: %s%s%s", Name(player), gap, arg2);
   } else {
     notify_format(player, T("You whisper, \"%s\"%s."), arg2, tbuf);
