@@ -1,21 +1,16 @@
+/**
+ * \file strdup.c
+ *
+ * \brief strdup routine for systems without one.
+ *
+ *
+ */
 
-/* strdup.c */
-
-/* Strdup routine for systems without one. */
 #include "config.h"
 
 #include <ctype.h>
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
-#ifdef I_STDLIB
 #include <stdlib.h>
-#endif
-#ifdef I_MEMORY
-#include <memory.h>
-#endif
 #include "conf.h"
 #include "copyrite.h"
 #include "mymalloc.h"
@@ -24,9 +19,12 @@
 #ifndef HAS_STRDUP
 char *strdup(const char *s);
 
+/** strdup for systems without one.
+ * \param s string to duplicate.
+ * \return newly-allocated copy of s
+ */
 char *
-strdup(s)
-    const char *s;
+strdup(const char *s)
 {
   int len;
   char *dup = NULL;
