@@ -1,18 +1,21 @@
-/* unparse.c */
+/**
+ * \file unparse.c
+ *
+ * \brief Convert lots of things into strings for PennMUSH.
+ *
+ *
+ */
+
 #include "copyrite.h"
 #include "config.h"
 
-#ifdef I_STRING
 #include <string.h>
-#else
-#include <strings.h>
-#endif
 #include "conf.h"
+#include "externs.h"
 #include "mushdb.h"
 #include "dbdefs.h"
 #include "flags.h"
 #include "lock.h"
-#include "externs.h"
 #include "attrib.h"
 #include "ansi.h"
 #include "pueblo.h"
@@ -22,10 +25,16 @@
 /* Hack added by Thorvald for object_header Pueblo */
 static int couldunparse;
 
+/** Format an object's name (and dbref and flags).
+ * This is a wrapper for real_unparse() that conditionally applies
+ * pueblo xch_cmd tags to make the object's name a hyperlink to examine
+ * it.
+ * \param player the looker.
+ * \param loc the object being looked at.
+ * \return static formatted object name string.
+ */
 const char *
-unparse_object(player, loc)
-    dbref player;
-    dbref loc;
+unparse_object(dbref player, dbref loc)
 {
   static PUEBLOBUFF;
   const char *result;
@@ -40,10 +49,16 @@ unparse_object(player, loc)
   }
 }
 
+/** Format an object's name, obeying MYOPIC/ownership rules.
+ * This is a wrapper for real_unparse() that conditionally applies
+ * pueblo xch_cmd tags to make the object's name a hyperlink to examine
+ * it.
+ * \param player the looker.
+ * \param loc the object being looked at.
+ * \return static formatted object name string.
+ */
 const char *
-unparse_object_myopic(player, loc)
-    dbref player;
-    dbref loc;
+unparse_object_myopic(dbref player, dbref loc)
 {
   static PUEBLOBUFF;
   const char *result;
@@ -58,15 +73,19 @@ unparse_object_myopic(player, loc)
   }
 }
 
-/* Like unparse_object, but tell real_unparse to use @NAMEFORMAT if present
+/** Format an object's name, obeying MYOPIC/ownership and NAMEFORMAT.
+ * \verbatim
+ * Like unparse_object, but tell real_unparse to use @NAMEFORMAT if present
  * This should only be used if we're looking at our container, to prevent
  * confusion in matching, since you can't match containers with their
  * names anyway, but only with 'here'.
+ * \endverbatim
+ * \param player the looker.
+ * \param loc the object being looked at.
+ * \return static formatted object name string.
  */
 const char *
-unparse_room(player, loc)
-    dbref player;
-    dbref loc;
+unparse_room(dbref player, dbref loc)
 {
   static PUEBLOBUFF;
   const char *result;
@@ -81,6 +100,18 @@ unparse_room(player, loc)
   }
 }
 
+/** Format an object's name in several ways.
+ * This function does the real work of converting a dbref to a
+ * name, possibly including the dbref and flags, possibly using
+ * a nameformat or nameaccent if present.
+ * \param player the looker.
+ * \param loc dbref of the object being looked at.
+ * \param obey_myopic if 0, always show Name(#xxxFLAGS); if 1, don't
+ * do so if player is MYOPIC or doesn't own loc.
+ * \param use_nameformat if 1, apply a NAMEFORMAT attribute if available.
+ * \param use_nameaccent if 1, apply a NAMEACCENT attribute if available.
+ * \return address of a static buffer containing the formatted name.
+ */
 const char *
 real_unparse(dbref player, dbref loc, int obey_myopic, int use_nameformat,
 	     int use_nameaccent)
@@ -145,15 +176,18 @@ real_unparse(dbref player, dbref loc, int obey_myopic, int use_nameformat,
   }
 }
 
-/* The name of loc as seen by a player inside it if nameformat is set */
-/* This function needs to avoid using a static buffer, so pass in 
- * a pointer to an allocated BUFFER_LEN array
+/** Build the name of loc as seen by a player inside it, but only
+ * if it has a NAMEFORMAT.
+ * This function needs to avoid using a static buffer, so pass in 
+ * a pointer to an allocated BUFFER_LEN array.
+ * \param player the looker.
+ * \param loc dbref of location being looked at.
+ * \param tbuf1 address to store formatted name of loc.
+ * \retval 1 a NAMEFORMAT was found, and tbuf1 contains formatted name.
+ * \retval 0 no NAMEFORMAT on loc, tbuf1 is undefined.
  */
 int
-nameformat(player, loc, tbuf1)
-    dbref player;
-    dbref loc;
-    char *tbuf1;
+nameformat(dbref player, dbref loc, char *tbuf1)
 {
   ATTR *a;
   char *wsave[10], *rsave[NUMQ];
@@ -165,24 +199,24 @@ nameformat(player, loc, tbuf1)
   if (a) {
     arg = (char *) mush_malloc(BUFFER_LEN, "string");
     if (!arg)
-      panic("Unable to allocate memory in nameformat");
+      mush_panic("Unable to allocate memory in nameformat");
     save_global_regs("nameformat", rsave);
     for (j = 0; j < 10; j++) {
-      wsave[j] = wenv[j];
-      wenv[j] = NULL;
+      wsave[j] = global_eval_context.wenv[j];
+      global_eval_context.wenv[j] = NULL;
     }
     for (j = 0; j < NUMQ; j++)
-      renv[j][0] = '\0';
+      global_eval_context.renv[j][0] = '\0';
     strcpy(arg, unparse_dbref(loc));
-    wenv[0] = arg;
-    sp = save = safe_uncompress(a->value);
+    global_eval_context.wenv[0] = arg;
+    sp = save = safe_atr_value(a);
     bp = tbuf1;
     process_expression(tbuf1, &bp, &sp, loc, player, player,
 		       PE_DEFAULT, PT_DEFAULT, NULL);
     *bp = '\0';
     free((Malloc_t) save);
     for (j = 0; j < 10; j++) {
-      wenv[j] = wsave[j];
+      global_eval_context.wenv[j] = wsave[j];
     }
     restore_global_regs("nameformat", rsave);
     mush_free((Malloc_t) arg, "string");
@@ -193,10 +227,12 @@ nameformat(player, loc, tbuf1)
   }
 }
 
-/* Give a string representation of a dbref */
+/** Give a string representation of a dbref.
+ * \param num value to stringify
+ * \return address of static buffer containing stringified value.
+ */
 char *
-unparse_dbref(num)
-    dbref num;
+unparse_dbref(dbref num)
 {
   /* Not BUFFER_LEN, but no dbref will come near this long */
   static char str[SBUF_LEN];
@@ -208,10 +244,12 @@ unparse_dbref(num)
   return str;
 }
 
-/* Give a string representation of an integer */
+/** Give a string representation of an integer.
+ * \param num value to stringify
+ * \return address of static buffer containing stringified value.
+ */
 char *
-unparse_integer(num)
-    int num;
+unparse_integer(int num)
 {
   static char str[SBUF_LEN];
   char *strp;
@@ -222,10 +260,12 @@ unparse_integer(num)
   return str;
 }
 
-/* Give a string representation of an unsigned integer */
+/** Give a string representation of an unsigned integer.
+ * \param num value to stringify
+ * \return address of static buffer containing stringified value.
+ */
 char *
-unparse_uinteger(num)
-    unsigned int num;
+unparse_uinteger(unsigned int num)
 {
   static char str[16];
 
@@ -234,10 +274,12 @@ unparse_uinteger(num)
 }
 
 
-/* Give a string representation of a number */
+/** Give a string representation of a number.
+ * \param num value to stringify
+ * \return address of static buffer containing stringified value.
+ */
 char *
-unparse_number(num)
-    NVAL num;
+unparse_number(NVAL num)
 {
   static char str[100];		/* Should be large enough for even the HUGE floats */
   char *p;
@@ -254,6 +296,11 @@ unparse_number(num)
   return str;
 }
 
+/** Return the name of an object, applying NAMEACCENT if set.
+ * \param thing dbref of object.
+ * \return address of static buffer containing object name, with accents
+ * if object has a valid NAMEACCENT attribute.
+ */
 const char *
 accented_name(dbref thing)
 {
@@ -268,7 +315,7 @@ accented_name(dbref thing)
     char *bp = fbuf;
     size_t len;
 
-    strcpy(tbuf, uncompress(AL_STR(na)));
+    strcpy(tbuf, atr_value(na));
 
     len = strlen(Name(thing));
 
