@@ -1,4 +1,10 @@
-/* utils.c */
+/**
+ * \file utils.c
+ *
+ * \brief Utility functions for PennMUSH.
+ *
+ *
+ */
 
 #include "copyrite.h"
 #include "config.h"
@@ -18,10 +24,7 @@
 #ifdef I_SYS_STAT
 #include <sys/stat.h>
 #endif
-#define I_FCNTL
-#ifdef I_FCNTL
 #include <fcntl.h>
-#endif
 #ifdef I_UNISTD
 #include <unistd.h>
 #endif
@@ -31,9 +34,6 @@
 #endif
 #include "conf.h"
 
-#ifdef MEM_CHECK
-#include "memcheck.h"
-#endif
 #include "match.h"
 #include "externs.h"
 #include "mushdb.h"
@@ -42,49 +42,64 @@
 #include "flags.h"
 #include "dbdefs.h"
 #include "attrib.h"
+#include "lock.h"
 #include "confmagic.h"
 
 dbref find_entrance(dbref door);
 void initialize_mt(void);
-unsigned long genrand_int32(void);
-long genrand_int31(void);
-void init_genrand(unsigned long);
-void init_by_array(unsigned long *, int);
+static unsigned long genrand_int32(void);
+static long genrand_int31(void);
+static void init_genrand(unsigned long);
+static void init_by_array(unsigned long *, int);
 
+/** A malloc wrapper that tracks type of allocation.
+ * This should be used in preference to malloc() when possible,
+ * to enable memory leak tracing with MEM_CHECK.
+ * \param size bytes to allocate.
+ * \param check string to label allocation with.
+ * \return allocated block of memory or NULL.
+ */
 Malloc_t
-mush_malloc(Size_t size, const char *check)
+mush_malloc(size_t size, const char *check)
 {
   Malloc_t ptr;
-#ifdef MEM_CHECK
   add_check(check);
-#endif
   ptr = malloc(size);
   if (ptr == NULL)
-    do_log(LT_ERR, 0, 0, "mush_malloc failed to malloc %d bytes for %s",
-	   size, check);
+    do_log(LT_ERR, 0, 0, "mush_malloc failed to malloc %lu bytes for %s",
+	   (unsigned long) size, check);
   return ptr;
 }
 
+/** A free wrapper that tracks type of allocation.
+ * If mush_malloc() gets the memory, mush_free() should free it
+ * to enable memory leak tracing with MEM_CHECK.
+ * \param ptr pointer to block of member to free.
+ * \param check string to label allocation with.
+ */
 void
-mush_free(Malloc_t ptr, const char *check __attribute__ ((__unused__)))
+mush_free(Malloc_t RESTRICT ptr, const char *RESTRICT check
+	  __attribute__ ((__unused__)))
 {
-#ifdef MEM_CHECK
   del_check(check);
-#endif
   free(ptr);
   return;
 }
 
 
+/** Parse object/attribute strings into components.
+ * This function takes a string which is of the format obj/attr or attr,
+ * and returns the dbref of the object, and a pointer to the attribute.
+ * If no object is specified, then the dbref returned is the player's.
+ * str is destructively modified. This function is probably underused.
+ * \param player the default object.
+ * \param str the string to parse.
+ * \param thing pointer to dbref of object parsed out of string.
+ * \param attrib pointer to pointer to attribute structure retrieved.
+ */
 void
 parse_attrib(dbref player, char *str, dbref *thing, ATTR **attrib)
 {
-  /* takes a string which is of the format <obj>/<attr> or <attr>,
-   * and returns the dbref of the object, and a pointer to the attribute.
-   * If no object is specified, then the dbref returned is the player's.
-   * str is destructively modified.
-   */
-
   char *name;
 
   /* find the object */
@@ -101,7 +116,61 @@ parse_attrib(dbref player, char *str, dbref *thing, ATTR **attrib)
   *attrib = (ATTR *) atr_get(*thing, upcasestr(name));
 }
 
+/** Parse an attribute or anonymous attribute into dbref and pointer.
+ * This function takes a string which is of the format #lambda/code, 
+ * <obj>/<attr> or <attr>,  and returns the dbref of the object, 
+ * and a pointer to the attribute.
+ * \param player the executor, for permissions checks.
+ * \param str string to parse.
+ * \param thing pointer to address to return dbref parsed, or NOTHING
+ * if none could be parsed.
+ * \param attrib pointer to address to return ATTR * of attrib parsed,
+ * or NULL if none could be parsed.
+ */
+void
+parse_anon_attrib(dbref player, char *str, dbref *thing, ATTR **attrib)
+{
 
+  if (string_prefix(str, "#lambda/")) {
+    unsigned char *t;
+    str += 8;
+    if (!*str) {
+      *attrib = NULL;
+      *thing = NOTHING;
+    } else {
+      *attrib = mush_malloc(sizeof(ATTR), "anon_attr");
+      AL_CREATOR(*attrib) = player;
+      AL_NAME(*attrib) = strdup("#lambda");
+      t = compress(str);
+      (*attrib)->data = chunk_create(t, (u_int_16) u_strlen(t), 0);
+      AL_FLAGS(*attrib) = AF_ANON;
+      AL_NEXT(*attrib) = NULL;
+      *thing = player;
+      return;
+    }
+  }
+  parse_attrib(player, str, thing, attrib);
+}
+
+/** Free the memory allocated for an anonymous attribute.
+ * \param attrib pointer to attribute.
+ */
+void
+free_anon_attrib(ATTR *attrib)
+{
+  if (attrib && (AL_FLAGS(attrib) & AF_ANON)) {
+    free((char *) AL_NAME(attrib));
+    chunk_delete(attrib->data);
+    mush_free(attrib, "anon_attr");
+  }
+}
+
+/** Given an exit, find the room that is its source through brute force.
+ * This is used in pathological cases where the exit's own source
+ * element is invalid.
+ * \param door dbref of exit to find source of.
+ * \return dbref of exit's source room, or NOTHING.
+ */
 dbref
 find_entrance(dbref door)
 {
@@ -119,7 +188,12 @@ find_entrance(dbref door)
   return NOTHING;
 }
 
-/* remove the first occurence of what in list headed by first */
+/** Remove the first occurence of what in chain headed by first.
+ * This works for contents and exit chains.
+ * \param first dbref of first object in chain.
+ * \param what dbref of object to remove from chain.
+ * \return new head of chain.
+ */
 dbref
 remove_first(dbref first, dbref what)
 {
@@ -139,6 +213,12 @@ remove_first(dbref first, dbref what)
   }
 }
 
+/** Is an object on a chain?
+ * \param thing object to look for.
+ * \param list head of chain to search.
+ * \retval 1 found thing on list.
+ * \retval 0 did not find thing on list.
+ */
 int
 member(dbref thing, dbref list)
 {
@@ -151,9 +231,16 @@ member(dbref thing, dbref list)
 }
 
 
-/* Return 1 if disallow is inside of from, i.e., if loc(disallow) = from,
- * or loc(loc(disallow)) = from, etc.
- * Actually, it's not recursive any more.
+/** Is an object inside another, at any level of depth?
+ * That is, we check if disallow is inside of from, i.e., if 
+ * loc(disallow) = from, or loc(loc(disallow)) = from, etc., with a 
+ * depth limit of 50.
+ * Despite the name of this function, it's not recursive any more.
+ * \param disallow interior object to check.
+ * \param from check if disallow is inside of this object.
+ * \param count depths of nesting checked so far.
+ * \retval 1 disallow is inside of from.
+ * \retval 0 disallow is not inside of from.
  */
 int
 recursive_member(dbref disallow, dbref from, int count)
@@ -173,7 +260,11 @@ recursive_member(dbref disallow, dbref from, int count)
   return 1;
 }
 
-/* Return one if the dbref, or its location, etc. is set unfindable */
+/** Is an object or its location unfindable?
+ * \param thing object to check.
+ * \retval 1 object or location is unfindable.
+ * \retval 0 neither object nor location is unfindable.
+ */
 int
 unfindable(dbref thing)
 {
@@ -192,6 +283,10 @@ unfindable(dbref thing)
 }
 
 
+/** Reverse the order of a dbref chain.
+ * \param list dbref at the head of the chain.
+ * \return dbref at the head of the reversed chain.
+ */
 dbref
 reverse(dbref list)
 {
@@ -207,7 +302,7 @@ reverse(dbref list)
 }
 
 
-#define N 624
+#define N 624 /**< PRNG constant */
 
 /* We use the Mersenne Twister PRNG. It's quite good as PRNGS go,
  * much better than the typical ones provided in system libc's.
@@ -223,7 +318,7 @@ reverse(dbref list)
  */
 
 
-/* Initializing the array with a seed */
+/** Wrapper to choose a seed and initialize the Mersenne Twister PRNG. */
 void
 initialize_mt(void)
 {
@@ -274,16 +369,18 @@ initialize_mt(void)
 /* email: matumoto@math.keio.ac.jp                                 */
 
 /* Period parameters */
-#define M 397
-#define MATRIX_A 0x9908b0dfUL	/* constant vector a */
-#define UPPER_MASK 0x80000000UL	/* most significant w-r bits */
-#define LOWER_MASK 0x7fffffffUL	/* least significant r bits */
+#define M 397  /**< PRNG constant */
+#define MATRIX_A 0x9908b0dfUL	/**< PRNG constant vector a */
+#define UPPER_MASK 0x80000000UL	/**< PRNG most significant w-r bits */
+#define LOWER_MASK 0x7fffffffUL	/**< PRNG least significant r bits */
 
 static unsigned long mt[N];	/* the array for the state vector  */
 static int mti = N + 1;		/* mti==N+1 means mt[N] is not initialized */
 
-/* initializes mt[N] with a seed */
-void
+/** initializes mt[N] with a seed.
+ * \param a seed value.
+ */
+static void
 init_genrand(unsigned long s)
 {
   mt[0] = s & 0xffffffffUL;
@@ -298,10 +395,11 @@ init_genrand(unsigned long s)
   }
 }
 
-/* initialize by an array with array-length */
-/* init_key is the array for initializing keys */
-/* key_length is its length */
-void
+/** initialize by an array with array-length
+ * \param init_key the array for initializing keys 
+ * \param key_length the array's length 
+ */
+static void
 init_by_array(unsigned long init_key[], int key_length)
 {
   int i, j, k;
@@ -337,7 +435,7 @@ init_by_array(unsigned long init_key[], int key_length)
 }
 
 /* generates a random number on [0,0xffffffff]-interval */
-unsigned long
+static unsigned long
 genrand_int32(void)
 {
   unsigned long y;
@@ -376,13 +474,18 @@ genrand_int32(void)
 }
 
 /* generates a random number on [0,0x7fffffff]-interval */
-long
+static long
 genrand_int31(void)
 {
   return (long) (genrand_int32() >> 1);
 }
 
-/* Based on MUX's RandomINT32() */
+/** Get a uniform random long between low and high values, inclusive.
+ * Based on MUX's RandomINT32()
+ * \param low lower bound for random number.
+ * \param high upper bound for random number.
+ * \return random number between low and high, or 0 or -1 for error.
+ */
 long
 get_random_long(long low, long high)
 {
@@ -427,8 +530,10 @@ get_random_long(long low, long high)
   return low + (n % x);
 }
 
-/* Return an object's name, but for exits, return just the first
+/** Return an object's name, but for exits, return just the first
  * component. We expect a valid object.
+ * \param it dbref of object.
+ * \return object's short name.
  */
 char *
 shortname(dbref it)
@@ -445,9 +550,11 @@ shortname(dbref it)
   return n;
 }
 
-/* Return the absolute room (outermost container) of an object 
- * or NOTHING if it's in an invalid object or in an invalid
- * location or AMBIGUOUS if there are too many containers
+/** Return the absolute room (outermost container) of an object.
+ * Return  NOTHING if it's in an invalid object or in an invalid
+ * location or AMBIGUOUS if there are too many containers.
+ * \param it dbref of object.
+ * \return absolute room of object, NOTHING, or AMBIGUOUS.
  */
 dbref
 absolute_room(dbref it)
@@ -467,3 +574,62 @@ absolute_room(dbref it)
   }
   return room;
 }
+
+
+/** Can one object interact with/perceive another in a given way?
+ * This funtion checks to see if 'to' can perceive something from
+ * 'from'. The types of interactions currently supported include:
+ * INTERACT_SEE (will light rays from 'from' reach 'to'?), INTERACT_HEAR
+ * (will sound from 'from' reach 'to'?), INTERACT_MATCH (can 'to'
+ * match the name of 'from'?), and INTERACT_PRESENCE (will the arrival/
+ * departure/connection/disconnection/growing ears/losing ears of 
+ * 'from' be noticed by 'to'?).
+ * \param from object of interaction.
+ * \param to subject of interaction, attempting to interact with from.
+ * \param type type of interaction.
+ * \retval 1 to can interact with from in this way.
+ * \retval 0 to can not interact with from in this way.
+ */
+int
+can_interact(dbref from, dbref to, int type)
+{
+  int lci;
+
+  /* This shouldn't even be checked for rooms and garbage, but we're
+   * paranoid. Trying to stop interaction with yourself will not work 99%
+   * of the time, so we don't allow it anyway. */
+  if (IsGarbage(from) || IsGarbage(to))
+    return 0;
+
+  if ((from == to) || IsRoom(from) || IsRoom(to))
+    return 1;
+
+  /* This function can override standard checks! */
+  lci = local_can_interact_first(from, to, type);
+  if (lci != NOTHING)
+    return lci;
+
+  /* Standard checks */
+
+  /* If it's an audible message, it must pass your Interact_Lock
+   * (or be from a privileged speaker)
+   */
+  if ((type == INTERACT_HEAR) && !Pemit_All(from)
+      && !eval_lock(from, to, Interact_Lock))
+    return 0;
+
+  /* You can interact with the object you are in or any objects
+   * you're holding.
+   * You can interact with objects you control, but not
+   * specifically the other way around
+   */
+  if ((from == Location(to)) || (to == Location(from)) || controls(to, from))
+    return 1;
+
+
+  lci = local_can_interact_last(from, to, type);
+  if (lci != NOTHING)
+    return lci;
+
+  return 1;
+}
