@@ -484,12 +484,20 @@ do_teleport(dbref player, const char *arg1, const char *arg2, int silent,
        * player. */
 
       /* now check to see if the absolute room is set NO_TEL */
-      if ((NoTel(absroom) || !eval_lock(player, absroom, Leave_Lock))
-	  && !controls(player, absroom)
+      if (NoTel(absroom) && !controls(player, absroom)
 	  && !Tel_Anywhere(player)) {
 	notify(player, T("Teleports are not allowed in this room."));
 	return;
       }
+
+      /* Check leave lock on room, if necessary */
+      if (!controls(player, absroom) && !Tel_Anywhere(player) &&
+	  !eval_lock(player, absroom, Leave_Lock)) {
+	fail_lock(player, absroom, Leave_Lock,
+		  T("Teleports are not allowed in this room."), NOTHING);
+	return;
+      }
+
       /* Now check the Z_TEL status of the victim's room.
        * Just like NO_TEL above, except that if the room (or its
        * Zone Master Room, if any, is Z_TEL,
@@ -1941,8 +1949,11 @@ FUNCTION(fun_objmem)
   else {
     thing = noisy_match_result(executor, args[0], NOTYPE, MAT_OBJECTS);
   }
-
-  if (!GoodObject(thing) || !Can_Examine(executor, thing)) {
+  if (!GoodObject(thing)) {
+    safe_str(T(e_match), buff, bp);
+    return;
+  }
+  if (!Can_Examine(executor, thing)) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
@@ -1970,7 +1981,11 @@ FUNCTION(fun_playermem)
     thing = atoi(args[0] + 1);
   else
     thing = lookup_player(args[0]);
-  if (!GoodObject(thing) || !Can_Examine(executor, thing)) {
+  if (!GoodObject(thing) || !IsPlayer(thing)) {
+    safe_str(T(e_match), buff, bp);
+    return;
+  }
+  if (!Can_Examine(executor, thing)) {
     safe_str(T(e_perm), buff, bp);
     return;
   }
